@@ -19,9 +19,9 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from itools
+from itools.core import merge_dicts
 from itools.datatypes import Boolean, Email, Tokens, Unicode, String
 from itools.gettext import MSG
-from itools.handlers import merge_dicts
 from itools.web import AccessControl as BaseAccessControl, STLForm, INFO
 from itools.web import ERROR
 from itools.xapian import AndQuery, OrQuery, PhraseQuery, StartQuery
@@ -62,7 +62,7 @@ class RoleAware_BrowseUsers(SearchForm):
 
     def get_query_schema(self):
         return merge_dicts(SearchForm.get_query_schema(self),
-                          sort_by=String(default='login_name'))
+                           sort_by=String(default='login_name'))
 
 
     search_schema = {
@@ -73,7 +73,8 @@ class RoleAware_BrowseUsers(SearchForm):
         ('', MSG(u'All Fields')),
         ('username', MSG(u'Login')),
         ('lastname', MSG(u'Last Name')),
-        ('firstname', MSG(u'First Name'))]
+        ('firstname', MSG(u'First Name')),
+        ('email_domain', MSG(u'Domain'))]
 
 
     def get_items(self, resource, context):
@@ -236,15 +237,15 @@ class RoleAware_AddUser(STLForm):
             'roles': resource.get_roles_namespace()}
 
 
-    def action(self, resource, context, form):
+    def _add(self, resource, context, form):
         root = context.root
         user = context.user
         users = root.get_resource('users')
 
         # Check whether the user already exists
-        email = form['email']
+        email = form['email'].strip()
         results = root.search(email=email)
-        if results.get_n_documents():
+        if len(results):
             user_id = results.get_documents()[0].name
         else:
             user_id = None
@@ -259,7 +260,7 @@ class RoleAware_AddUser(STLForm):
                 # Check the password is right
                 if password != password2:
                     context.message = messages.MSG_PASSWORD_MISMATCH
-                    return
+                    return None
                 if not password:
                     # Admin can set no password
                     # so the user must activate its account
@@ -278,21 +279,25 @@ class RoleAware_AddUser(STLForm):
             members = resource.get_members()
             if user_id in members:
                 context.message = ERROR(u'The user is already here.')
-                return
+                return None
 
         # Set the role
         role = form['role']
         resource.set_user_role(user_id, role)
+        return user_id
 
-        # Come back
-        if context.has_form_value('add_and_return'):
-            return
 
-        goto = '/users/%s/' % user.name
-        message = INFO(u'User added.')
-        return context.come_back(message, goto=goto)
+    def action_add_and_return(self, resource, context, form):
+        self._add(resource, context, form)
 
 
+    def action_add_and_view(self, resource, context, form):
+        user_id = self._add(resource, context, form)
+        if user_id is not None:
+            goto = '/users/%s/' % user_id
+            message = INFO(u'User added.')
+            return context.come_back(message, goto=goto)
+
 
 ###########################################################################
 # Model
@@ -325,7 +330,7 @@ class AccessControl(BaseAccessControl):
 
     # By default all other change operations (add, remove, copy, etc.)
     # are equivalent to "edit".
-    def is_allowed_to_lock(self, user, resource):
+    def is_allowed_to_put(self, user, resource):
         return self.is_allowed_to_edit(user, resource)
 
 
@@ -356,10 +361,13 @@ class AccessControl(BaseAccessControl):
         return self.is_allowed_to_trans(user, resource, 'publish')
 
 
+    def is_allowed_to_retire(self, user, resource):
+        return self.is_allowed_to_trans(user, resource, 'retire')
+
+
     def is_allowed_to_view_folder(self, user, resource):
-        try:
-            index = resource.get_resource('index')
-        except LookupError:
+        index = resource.get_resource('index', soft=True)
+        if index is None:
             return False
         return self.is_allowed_to_view(user, index)
 
@@ -383,7 +391,7 @@ class RoleAware(AccessControl):
     def get_metadata_schema(cls):
         schema = {}
         for rolename in cls.get_role_names():
-            schema[rolename] = Tokens(default=())
+            schema[rolename] = Tokens
         schema['website_is_open'] = Boolean
         return schema
 
