@@ -21,8 +21,8 @@ from urllib import quote, unquote
 from zlib import compress, decompress
 
 # Import from itools
+from itools.core import freeze, guess_type
 from itools.datatypes import DataType, Enumerate
-from itools.utils import freeze, guess_type
 
 
 """This module defines some datatypes used in ikaaro, whose inclusion in
