@@ -19,55 +19,51 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from the Standard Library
-from datetime import datetime, timedelta
+from os.path import basename
 
 # Import from itools
-from itools.datatypes import Integer, Unicode
+from itools.core import merge_dicts
+from itools.datatypes import Integer, Unicode, String, HTTPDate
 from itools.gettext import MSG
 from itools.handlers import get_handler_class_by_mimetype, guess_encoding
-from itools.handlers import merge_dicts
 from itools.html import HTMLParser, stream_to_str_as_xhtml
 from itools.i18n import guess_language
 from itools.uri import get_reference
-from itools.vfs import FileName
-from itools.web import BaseView, STLView, STLForm, INFO, ERROR
-from itools.xapian import PhraseQuery
+from itools.fs import FileName
+from itools.web import BaseView, STLView, INFO, ERROR
 
 # Import from ikaaro
 from datatypes import FileDataType, ImageWidth
-from folder_views import Folder_BrowseContent
-from forms import title_widget, file_widget, description_widget
-from forms import subject_widget
+from forms import title_widget, file_widget, description_widget, subject_widget
+from forms import timestamp_widget
+from forms import FileWidget, TextWidget
 import messages
 from multilingual import Multilingual
 from registry import get_resource_class
-from resource_views import AddResourceMenu, DBResource_Edit
-from views import NewInstanceForm
+from resource_views import DBResource_Edit
+from views_new import NewInstance
 
 
-class File_NewInstance(NewInstanceForm):
+class File_NewInstance(NewInstance):
 
-    access = 'is_allowed_to_add'
     title = MSG(u'Upload File')
-    template = '/ui/file/new_instance.xml'
     schema = {
         'title': Unicode,
-        'file': FileDataType(mandatory=True),
-    }
-    context_menus = [AddResourceMenu()]
-
-
-    def get_namespace(self, resource, context):
-        type = context.get_query_value('type')
-        cls = get_resource_class(type)
-        return {
-            'class_id': cls.class_id,
-            'class_title': cls.class_title,
-            'title': context.get_form_value('title', type=Unicode),
-        }
+        'name': String,
+        'file': FileDataType(mandatory=True)}
+    widgets = [
+        title_widget,
+        TextWidget('name', title=MSG(u'Name'), default=''),
+        FileWidget('file', title=MSG(u'File'), size=35)]
+    submit_value = MSG(u'Upload')
 
 
     def get_new_resource_name(self, form):
+        # If the name is not explicitly given, use the title
+        # or get it from the file
+        name = form['name']
+        if name:
+            return name
         filename, mimetype, body = form['file']
         name, type, language = FileName.decode(filename)
 
@@ -124,19 +120,33 @@ class File_Download(BaseView):
 
 
     def get_mtime(self, resource):
-        return resource.get_mtime()
+        return resource.handler.get_mtime()
+
+
+    def get_content_type(self, resource, context):
+        return resource.get_content_type()
+
+
+    def get_filename(self, resource, context):
+        return resource.get_property('filename')
+
+
+    def get_bytes(self, resource, context):
+        return resource.handler.to_str()
 
 
     def GET(self, resource, context):
-        response = context.response
-        # Filename
-        filename = resource.get_property('filename')
-        if filename is not None:
-            response.set_header('Content-Disposition',
-                                'inline; filename="%s"' % filename)
         # Content-Type
-        response.set_header('Content-Type', resource.get_content_type())
-        return resource.handler.to_str()
+        content_type = self.get_content_type(resource, context)
+        context.set_content_type(content_type)
+        # Content-Disposition
+        disposition = 'inline'
+        if content_type.startswith('application/vnd.oasis.opendocument.'):
+            disposition = 'attachment'
+        filename = self.get_filename(resource, context)
+        context.set_content_disposition(disposition, filename)
+        # Ok
+        return self.get_bytes(resource, context)
 
 
 
@@ -158,32 +168,37 @@ class File_View(STLView):
 
 
 
-class File_Upload(STLForm):
+class File_Edit(DBResource_Edit):
 
-    access = 'is_allowed_to_edit'
-    title = MSG(u'Replace')
-    icon = 'upload.png'
-    template = '/ui/file/upload.xml'
-    schema = {
-        'file': FileDataType(mandatory=True),
-    }
+    schema = merge_dicts(DBResource_Edit.schema, file=FileDataType)
+    widgets = [
+        timestamp_widget, title_widget, file_widget, description_widget,
+        subject_widget]
 
 
-    def action(self, resource, context, form, file='file'):
-        file = form[file]
-        # Added for views reusing this method but where the file is not
-        # mandatory.
-        if file is None:
+    def get_value(self, resource, context, name, datatype):
+        if name == 'file':
+            return None
+        return DBResource_Edit.get_value(self, resource, context, name,
+                                         datatype)
+
+
+    def action(self, resource, context, form):
+        DBResource_Edit.action(self, resource, context, form)
+        if context.edit_conflict:
             return
 
+        # Upload file
+        file = form['file']
+        if file is None:
+            return
         filename, mimetype, body = file
 
         # Check wether the handler is able to deal with the uploaded file
         handler = resource.handler
         handler_class = get_handler_class_by_mimetype(mimetype)
         if not isinstance(handler, handler_class):
-            message = ERROR(u'Unexpected file of mimetype ${type}',
-                            type=mimetype)
+            message = messages.MSG_UNEXPECTED_MIMETYPE(mimetype=mimetype)
             context.message = message
             return
 
@@ -192,8 +207,7 @@ class File_Upload(STLForm):
             handler.load_state_from_string(body)
         except Exception, e:
             handler.load_state()
-            message = ERROR(u'Failed to load the file: ${error}',
-                            error=str(e))
+            message = ERROR(u'Failed to load the file: {error}', error=str(e))
             context.message = message
             return
 
@@ -206,7 +220,7 @@ class File_Upload(STLForm):
                 metadata.format = mimetype
 
         # Update handler name
-        handler_name = handler.uri.path.get_name()
+        handler_name = basename(handler.key)
         old_name, old_extension, old_lang = FileName.decode(handler_name)
         new_name, new_extension, new_lang = FileName.decode(filename)
         # FIXME Should 'FileName.decode' return lowercase extensions?
@@ -218,27 +232,21 @@ class File_Upload(STLForm):
             folder.move_handler(handler_name, filename)
 
         # Ok
-        context.server.change_resource(resource)
+        context.database.change_resource(resource)
         context.message = INFO(u'Version uploaded')
 
 
 
-class File_Edit(DBResource_Edit, File_Upload):
-    schema = merge_dicts(DBResource_Edit.schema, file=FileDataType)
-    widgets = [title_widget, file_widget, description_widget, subject_widget]
+class File_ExternalEdit_View(STLView):
+    access = 'is_allowed_to_edit'
+    template = '/ui/file/externaledit.xml'
+    title = MSG(u'External Editor')
+    icon = 'external.png'
+    encodings = None
 
 
-    def get_value(self, resource, context, name, datatype):
-        if name == 'file':
-            return None
-        return DBResource_Edit.get_value(self, resource, context, name,
-                                         datatype)
-
-
-    def action(self, resource, context, form):
-        DBResource_Edit.action(self, resource, context, form)
-        File_Upload.action(self, resource, context, form)
-
+    def get_namespace(self, resource, context):
+        return {'encodings': self.encodings}
 
 
 
@@ -248,14 +256,19 @@ class File_ExternalEdit(BaseView):
 
 
     def GET(self, resource, context):
-        # Get the request and response
-        request, response = context.request, context.response
+        """Protocol used (with restedit.py):
+        1- we add a header to the content of the file.
+        2- the header is separated from the rest of the file by a "\n\n".
+        3- an entry in the header is:
 
+           header-name:header-body\n
+
+           The header-name does not contain ":" and the header-body does not
+           contain "\n"
+        """
         encoding = context.get_form_value('encoding')
 
         uri = context.uri
-        uri_string = '%s://%s/%s' % (uri.scheme, uri.authority, uri.path[:-1])
-        uri = get_reference(uri_string)
         handler = resource.handler
         title = resource.get_property('title')
         if title:
@@ -263,82 +276,37 @@ class File_ExternalEdit(BaseView):
         else:
             title = resource.name
 
-        r = [
-            'url:%s' % str(uri),
-            'meta_type:toto', # FIXME Check if zopeedit really needs this
+        soup_message = context.soup_message
+        header = [
+            'url:%s://%s%s' % (uri.scheme, uri.authority, uri.path[:-1]),
+            'last-modified:%s' % HTTPDate.encode(resource.get_mtime()),
             'content_type:%s' % handler.get_mimetype(),
-            'cookie:%s' % request.get_cookies_as_str(),
-            'title:%s' % title,
-            ]
+            'cookie:%s' % soup_message.get_header('Cookie'),
+            'title:%s' % title]
 
-        if resource.is_locked():
-            lock = resource.get_lock()
-            # locks expire after 1 hour
-            if lock.lock_timestamp + timedelta(hours=1) < datetime.now():
-                resource.unlock()
-                context.commit = True
-            else:
-                # always borrow lock from same user
-                if lock.username == context.user.name:
-                    r.append('lock-token:%s' % lock.key)
-                    r.append('borrow_lock:1')
-                else:
-                    message = ERROR(u'This page is locked by another user')
-                    return context.come_back(message, goto='.')
+        auth = context.get_header('Authorization')
+        if auth:
+            header.append('auth:%s' % auth)
 
-        if request.has_header('Authorization'):
-            r.append('auth:%s' % request.get_header('Authorization'))
-
-        r.append('')
+        # Add the "\n\n" and make the header
+        header.append('\n')
+        header = '\n'.join(header)
 
         # TODO known bug from ExternalEditor requires rfc1123_date()
         # Using RESPONSE.setHeader('Pragma', 'no-cache') would be better, but
         # this chokes crappy most MSIE versions when downloads happen on SSL.
         # cf. http://support.microsoft.com/support/kb/articles/q316/4/31.asp
-        #response.set_header('Last-Modified', rfc1123_date())
-        response.set_header('Pragma', 'no-cache')
+        #context.set_header('Last-Modified', rfc1123_date())
+        context.set_header('Pragma', 'no-cache')
 
         # Encoding
         if encoding is None:
-            r.append(handler.to_str())
+            data = handler.to_str()
         else:
-            r.append(handler.to_str(encoding))
+            data = handler.to_str(encoding)
 
-        data = '\n'.join(r)
-
-        response.set_header('Content-Type', 'application/x-zope-edit')
-        return data
-
-
-
-class File_Backlinks(Folder_BrowseContent):
-    """Backlinks are the list of resources pointing to this resource.  This
-    view answers the question "where is this resource used?" You'll see all
-    WebPages and WikiPages (for example) referencing it.  If the list is
-    empty, you can consider it is "orphan".
-    """
-
-    access = 'is_allowed_to_view'
-    title = MSG(u"Backlinks")
-    icon = 'rename.png'
-
-    search_template = None
-    search_schema = {}
-
-    # Skip AddResourceMenu
-    context_menus = []
-
-    def get_table_columns(self, resource, context):
-        cols = Folder_BrowseContent.get_table_columns(self, resource, context)
-        return [ col for col in cols if col[0] != 'checkbox' ]
-
-
-    def get_items(self, resource, context):
-        query = PhraseQuery('links', str(resource.get_abspath()))
-        return context.root.search(query)
-
-
-    table_actions = []
+        context.content_type = 'application/x-restedit'
+        return header + data
 
 
 
@@ -347,27 +315,28 @@ class Image_Thumbnail(BaseView):
     access = 'is_allowed_to_view'
 
     def get_mtime(self, resource):
-        return resource.get_mtime()
+        return resource.handler.get_mtime()
 
 
     def GET(self, resource, context):
         width = context.get_form_value('width', type=Integer, default=48)
         height = context.get_form_value('height', type=Integer, default=48)
 
-        data, format = resource.handler.get_thumbnail(width, height)
+        # TODO generate the thumbnail in the resource format
+        format = 'png' if resource.metadata.format == 'image/png' else 'jpeg'
+        data, format = resource.handler.get_thumbnail(width, height, format)
         if data is None:
             default = resource.get_resource('/ui/icons/48x48/image.png')
             data = default.to_str()
             format = 'png'
 
-        response = context.response
-        # Filename
+        # Headers
+        context.set_content_type('image/%s' % format)
         filename = resource.get_property('filename')
-        if filename is not None:
-            response.set_header('Content-Disposition',
-                                'inline; filename="%s"' % filename)
-        # Content-Type
-        response.set_header('Content-Type', 'image/%s' % format)
+        if filename:
+            context.set_content_disposition('inline', filename)
+
+        # Ok
         return data
 
 
@@ -377,6 +346,9 @@ class Image_View(STLView):
     access = 'is_allowed_to_view'
     title = MSG(u'View')
     template = '/ui/binary/Image_view.xml'
+    styles = ['/ui/gallery/style.css']
+    scripts = ['/ui/gallery/javascript.js']
+
     # Image default size as a string (empty = full size)
     default_width = ''
     default_height = ''
@@ -393,8 +365,6 @@ class Image_View(STLView):
 
 
     def get_namespace(self, resource, context):
-        context.styles.append('/ui/gallery/style.css')
-        context.scripts.append('/ui/gallery/javascript.js')
         size = context.get_form_value('size', type=Integer)
         width = context.get_form_value('width', default=self.default_width)
         height = context.get_form_value('height', default=self.default_height)
@@ -435,11 +405,11 @@ class Image_View(STLView):
             # Preload with same size preferences than the current one
             if width and height:
                 # Preload a thumbnail
-                uri = prefix.resolve2(';thumb').replace(width=width,
-                                                        height=height)
+                uri = prefix.resolve_name(';thumb').replace(width=width,
+                                                            height=height)
             else:
                 # Preload the full size
-                uri = prefix.resolve2(';download')
+                uri = prefix.resolve_name(';download')
             preload.append(str(uri))
 
         # Real width and height (displayed for reference)
