@@ -21,17 +21,19 @@
 # Import from itools
 from itools.gettext import MSG
 from itools.handlers import Folder as FolderHandler
-from itools import vfs
+from itools.uri import Path
 from itools.web import get_context, BaseView
-from itools.xapian import PhraseQuery
+from itools.xapian import AndQuery, NotQuery, PhraseQuery
 
 # Import from ikaaro
 from exceptions import ConsistencyError
-from folder_views import Folder_BrowseContent, Folder_LastChanges
+from folder_views import Folder_BrowseContent
 from folder_views import Folder_NewResource, Folder_Orphans, Folder_Thumbnail
 from folder_views import Folder_PreviewContent, Folder_Rename, Folder_View
 from registry import register_resource_class, get_resource_class
+from registry import get_document_types
 from resource_ import DBResource
+from utils import get_base_path_query
 
 
 
@@ -43,14 +45,12 @@ class Folder(DBResource):
     class_description = MSG(u'Organize your files and documents with folders.')
     class_icon16 = 'icons/16x16/folder.png'
     class_icon48 = 'icons/48x48/folder.png'
-    class_views = ['view', 'browse_content', 'preview_content', 'edit']
+    class_views = ['view', 'browse_content', 'preview_content', 'edit',
+                   'backlinks', 'commit_log']
     class_handler = FolderHandler
 
 
-    #########################################################################
     # Aggregation relationship (what a generic folder can contain)
-    class_document_types = []
-
     __fixed_handlers__ = []
 
 
@@ -62,13 +62,26 @@ class Folder(DBResource):
     SIZE_STEPS = (32, 48, 64, 128, 256, 512)
 
 
-    @classmethod
-    def register_document_type(cls, handler_class):
-        cls.class_document_types.append(handler_class)
-
-
     def get_document_types(self):
-        return self.class_document_types
+        return get_document_types()
+
+
+    def get_files_to_archive(self, content=False):
+        metadata = self.metadata.key
+        if content is True:
+            folder = self.handler.key
+            return [metadata, folder]
+        return [metadata]
+
+
+    ########################################################################
+    # Cut & Paste Resources
+    ########################################################################
+    def can_paste(self, source):
+        """Is the source resource can be pasted into myself.
+        """
+        allowed_types = tuple(self.get_document_types())
+        return isinstance(source, allowed_types)
 
 
     #######################################################################
@@ -81,159 +94,159 @@ class Folder(DBResource):
 
 
     def _get_resource(self, name):
+        # Look for the resource
         folder = self.handler
-        metadata = folder.get_handler('%s.metadata' % name)
+        try:
+            metadata = folder.get_handler('%s.metadata' % name)
+        except LookupError:
+            return None
+
+        # Format (class id)
         format = metadata.format
 
-        uri = folder.uri.resolve2(name)
-        if vfs.exists(uri):
-            is_file = vfs.is_file(uri)
+        # File or folder
+        fs = metadata.database.fs
+        key = fs.resolve2(folder.key, name)
+        if fs.exists(key):
+            is_file = fs.is_file(key)
         else:
             # FIXME This is just a guess, it may fail.
             is_file = '/' in format
 
+        # Ok
         cls = get_resource_class(format, is_file=is_file)
         return cls(metadata)
 
 
-    def update_links(self, new_name, base_path):
-        """The resource must update its links to itself and to its content
-        """
-        # Check referencial-integrity
-        catalog = get_context().server.catalog
-        # The catalog is not available when updating (icms-update.py)
-        # FIXME We do not guarantee referencial-integrity when updating
-        if catalog is None:
+    def del_resource(self, name, soft=False):
+        database = get_context().database
+        resource = self.get_resource(name, soft=soft)
+        if soft and resource is None:
             return
 
-        old_base_path = self.get_abspath()
-        new_base_path = base_path.resolve2(new_name)
-        for resource in self.traverse_resources():
-            # Old and new paths
-            old_path = resource.get_abspath()
-            pathto = old_base_path.get_pathto(old_path)
-            new_path = new_base_path.resolve2(pathto)
-
-            # Get all the resources that have a link to this resource
-            query = PhraseQuery('links', str(old_path))
-            results = catalog.search(query).get_documents()
-            for result in results:
-                resource = self.get_resource(result.abspath)
-                resource.change_link(old_path, new_path)
-
-
-    def del_resource(self, name):
-        context = get_context()
-        resource = self.get_resource(name)
-
         # Check referencial-integrity
-        catalog = context.server.catalog
-        # The catalog is not available when updating (icms-update.py)
-        # FIXME We do not guarantee referencial-integrity when updating
-        if catalog is not None:
-            # FIXME Check sub-resources too
-            path = str(resource.get_abspath())
-            results = catalog.search(links=path)
-            if results.get_n_documents() > 0:
-                message = 'cannot delete, resource "%s" is referenced' % path
-                raise ConsistencyError, message
+        catalog = database.catalog
+        # FIXME Check sub-resources too
+        path = str(resource.get_canonical_path())
+        query_base_path = get_base_path_query(path)
+        query = AndQuery(PhraseQuery('links', path),
+                         NotQuery(query_base_path))
+        results = catalog.search(query)
+        if len(results):
+            message = 'cannot delete, resource "%s" is referenced' % path
+            raise ConsistencyError, message
 
         # Events, remove
-        context.server.remove_resource(resource)
+        database.remove_resource(resource)
         # Remove
-        folder = self.handler
-        folder.del_handler('%s.metadata' % name)
+        fs = database.fs
         for handler in resource.get_handlers():
-            if folder.has_handler(handler.uri):
-                folder.del_handler(handler.uri)
+            # Skip empty folders and phantoms
+            if fs.exists(handler.key):
+                database.del_handler(handler.key)
+        self.handler.del_handler('%s.metadata' % name)
 
 
-    def _resolve_source_target(self, source, target):
-        # Find out the source and target absolute URIs
-        folder = self.handler
-        if source[0] == '/':
-            source_uri = self.get_root().handler.uri.resolve2(source[1:])
-        else:
-            source_uri = folder.uri.resolve2(source)
-        if target[0] == '/':
-            target_uri = self.get_root().handler.uri.resolve2(target[1:])
-        else:
-            target_uri = folder.uri.resolve2(target)
+    def _resolve_source_target(self, source_path, target_path):
+        if type(source_path) is not Path:
+            source_path = Path(source_path)
+        if type(target_path) is not Path:
+            target_path = Path(target_path)
 
         # Load the handlers so they are of the right class, for resources
         # like that define explicitly the handler class.  This fixes for
         # instance copy&cut&paste of a tracker in a just started server.
         # TODO this is a work-around, there should be another way to define
         # explicitly the handler class.
-        resource = self.get_resource(source)
-        if isinstance(resource, Folder):
-            for resource in resource.traverse_resources():
+        source = self.get_resource(source_path)
+        if isinstance(source, Folder):
+            for resource in source.traverse_resources():
                 resource.load_handlers()
         else:
-            resource.load_handlers()
+            source.load_handlers()
 
-        return source_uri, target_uri
+        return source_path, target_path
 
 
-    def copy_resource(self, source, target):
-        server = get_context().server
-
+    def copy_resource(self, source_path, target_path):
         # Find out the source and target absolute URIs
-        source_uri, target_uri = self._resolve_source_target(source, target)
-        new_name = target_uri.path[-1]
+        source_path, target_path = self._resolve_source_target(source_path,
+                                                               target_path)
 
-        # Get the source resource
-        resource = self.get_resource(source)
+        # Get the source and target resources
+        source = self.get_resource(source_path)
+        parent_path = target_path.resolve2('..')
+        target_parent = self.get_resource(parent_path)
+
+        # Check compatibility
+        if (not target_parent.can_paste(source)
+                or not source.can_paste_into(target_parent)):
+            message = 'resource type "%r" cannot be copied into type "%r"'
+            raise ConsistencyError, message % (source, target_parent)
 
         # Copy the metadata
         folder = self.handler
-        folder.copy_handler('%s.metadata' % source_uri,
-                            '%s.metadata' % target_uri)
+        folder.copy_handler('%s.metadata' % source_path,
+                            '%s.metadata' % target_path)
+
         # Copy the content
-        for old_name, new_name in resource.rename_handlers(new_name):
+        database = self.metadata.database
+        fs =  database.fs
+        new_name = target_path.get_name()
+        for old_name, new_name in source.rename_handlers(new_name):
             if old_name is None:
                 continue
-            src_uri = source_uri.resolve(old_name)
-            dst_uri = target_uri.resolve(new_name)
-            if folder.has_handler(src_uri):
-                folder.copy_handler(src_uri, dst_uri)
+            src_key = fs.resolve(source_path, old_name)
+            dst_key = fs.resolve(target_path, new_name)
+            if folder.has_handler(src_key):
+                folder.copy_handler(src_key, dst_key)
 
         # Events, add
-        resource = self.get_resource(target)
-        server.add_resource(resource)
+        resource = self.get_resource(target_path)
+        database.add_resource(resource)
 
 
-    def move_resource(self, source, target):
-        server = get_context().server
-
+    def move_resource(self, source_path, target_path):
         # Find out the source and target absolute URIs
-        source_uri, target_uri = self._resolve_source_target(source, target)
-        new_name = target_uri.path[-1]
+        source_path, target_path = self._resolve_source_target(source_path,
+                                                               target_path)
 
-        # Get the source resource
-        resource = self.get_resource(source)
+        # Cannot move a resource to a subdirectory of itself
+        abspath = self.get_canonical_path()
+        if source_path.get_prefix(abspath) == source_path:
+            message = 'cannot move a resource to a subdirectory of itself'
+            raise ConsistencyError, message
+
+        # Get the source and target resources
+        source = self.get_resource(source_path)
+        parent_path = target_path.resolve2('..')
+        target_parent = self.get_resource(parent_path)
+
+        # Check compatibility
+        if (not target_parent.can_paste(source)
+                or not source.can_paste_into(target_parent)):
+            message = 'resource type "%r" cannot be moved into type "%r"'
+            raise ConsistencyError, message % (source, target_parent)
 
-        # Update the links to the resources that are to be moved
-        resource.update_links(target, self.get_abspath())
         # Events, remove
-        server.remove_resource(resource)
+        database = self.metadata.database
+        new_path = self.get_canonical_path().resolve2(target_path)
+        database.move_resource(source, new_path)
 
         # Move the metadata
         folder = self.handler
-        folder.move_handler('%s.metadata' % source_uri,
-                            '%s.metadata' % target_uri)
+        folder.move_handler('%s.metadata' % source_path,
+                            '%s.metadata' % target_path)
         # Move the content
-        for old_name, new_name in resource.rename_handlers(new_name):
+        fs = database.fs
+        new_name = target_path.get_name()
+        for old_name, new_name in source.rename_handlers(new_name):
             if old_name is None:
                 continue
-            src_uri = source_uri.resolve(old_name)
-            dst_uri = target_uri.resolve(new_name)
-            if folder.has_handler(src_uri):
-                folder.move_handler(src_uri, dst_uri)
-
-        # Events, add
-        resource = self.get_resource(target)
-        server.add_resource(resource)
+            src_key = fs.resolve(source_path, old_name)
+            dst_key = fs.resolve(target_path, new_name)
+            if folder.has_handler(src_key):
+                folder.move_handler(src_key, dst_key)
 
 
     def traverse_resources(self):
@@ -269,7 +282,7 @@ class Folder(DBResource):
         names = self.get_names()
         size = len(names)
 
-        return MSG(u'$n obs').gettext(n=size)
+        return MSG(u'{n} obs').gettext(n=size)
 
 
     #######################################################################
@@ -294,7 +307,6 @@ class Folder(DBResource):
     browse_content = Folder_BrowseContent()
     rename = Folder_Rename()
     preview_content = Folder_PreviewContent()
-    last_changes = Folder_LastChanges()
     orphans = Folder_Orphans()
     thumb = Folder_Thumbnail()
 
@@ -303,5 +315,4 @@ class Folder(DBResource):
 ###########################################################################
 # Register
 ###########################################################################
-register_resource_class(Folder)
 register_resource_class(Folder, format="application/x-not-regular-file")
