@@ -16,7 +16,6 @@
 
 # Import from the Standard Library
 from urllib import quote
-from datetime import datetime
 
 # Import from the Python Image Library
 try:
@@ -25,27 +24,26 @@ except ImportError:
     PILImage = None
 
 # Import from itools
+from itools.core import merge_dicts
 from itools.datatypes import Boolean, Integer, String, Unicode
 from itools.gettext import MSG
-from itools.handlers import checkid, merge_dicts
+from itools.handlers import checkid
 from itools.i18n import format_datetime
 from itools.stl import set_prefix
 from itools.uri import get_reference, Path
 from itools.web import BaseView, STLForm, ERROR
-from itools.xapian import AndQuery, OrQuery, PhraseQuery
-from itools.xml import XMLParser
+from itools.xapian import AndQuery, OrQuery, PhraseQuery, split_unicode
 
 # Import from ikaaro
 from buttons import RemoveButton, RenameButton, CopyButton, CutButton
-from buttons import PasteButton, PublishButton
+from buttons import PasteButton, PublishButton, RetireButton
 from datatypes import CopyCookie, ImageWidth
 from exceptions import ConsistencyError
 import messages
-from resource_views import AddResourceMenu
 from utils import generate_name, get_base_path_query
-from versioning import VersioningAware
 from views import IconsView, SearchForm, ContextMenu
-from workflow import WorkflowAware
+from views_new import AddResourceMenu
+from workflow import WorkflowAware, get_workflow_preview
 
 
 
@@ -89,9 +87,10 @@ class Folder_View(BaseView):
 
 
     def GET(self, resource, context):
-        try:
-            index = resource.get_resource('index')
-        except LookupError:
+        from webpage import WebPage
+
+        index = resource.get_resource('index', soft=True)
+        if not isinstance(index, WebPage):
             context.message = ERROR(
                 u'There is not an "index" web page. Could not render this '
                 u'view.')
@@ -135,6 +134,7 @@ class Folder_Rename(STLForm):
     schema = {
         'paths': String(multiple=True, mandatory=True),
         'new_names': String(multiple=True, mandatory=True)}
+    goto_after = ';browse_content'
 
 
     def get_namespace(self, resource, context):
@@ -171,7 +171,7 @@ class Folder_Rename(STLForm):
         paths.sort()
         paths.reverse()
         # Clean the copy cookie if needed
-        cut, cp_paths = context.get_cookie('ikaaro_cp', type=CopyCookie)
+        cut, cp_paths = context.get_cookie('ikaaro_cp', datatype=CopyCookie)
 
         renamed = []
         referenced = []
@@ -194,7 +194,7 @@ class Folder_Rename(STLForm):
             if new_name == old_name:
                 continue
             # Check there is not another resource with the same name
-            if container.has_resource(new_name):
+            if container.get_resource(new_name, soft=True) is not None:
                 context.message = messages.MSG_EXISTANT_FILENAME
                 return
             # Clean cookie (FIXME Do not clean the cookie, update it)
@@ -215,7 +215,7 @@ class Folder_Rename(STLForm):
             message = messages.MSG_RESOURCES_REFERENCED(resources=resources)
         else:
             message = messages.MSG_RENAMED
-        return context.come_back(message, goto=';browse_content')
+        return context.come_back(message, goto=self.goto_after)
 
 
 
@@ -224,6 +224,9 @@ class Folder_BrowseContent(SearchForm):
     access = 'is_allowed_to_view'
     title = MSG(u'Browse Content')
     context_menus = [AddResourceMenu()]
+    query_schema = merge_dicts(SearchForm.query_schema,
+        sort_by=String(default='mtime'),
+        reverse=Boolean(default=True))
     schema = {
         'ids': String(multiple=True, mandatory=True),
     }
@@ -270,7 +273,10 @@ class Folder_BrowseContent(SearchForm):
         else:
             args.append(PhraseQuery('parent_path', abspath))
         if search_term:
-            args.append(PhraseQuery(field, search_term))
+            language = resource.get_content_language(context)
+            terms_query = [ PhraseQuery(field, term)
+                            for term in split_unicode(search_term, language) ]
+            args.append(AndQuery(*terms_query))
         if len(args) == 1:
             query = args[0]
         else:
@@ -288,89 +294,76 @@ class Folder_BrowseContent(SearchForm):
         items = results.get_documents(sort_by=sort_by, reverse=reverse,
                                       start=start, size=size)
 
+        # FIXME This must be done in the catalog.
+        if sort_by == 'title':
+            items.sort(cmp=lambda x,y: cmp(x.title, y.title))
+            if reverse:
+                items.reverse()
+
         # Access Control (FIXME this should be done before batch)
         user = context.user
         root = context.root
         allowed_items = []
         for item in items:
-            item = root.get_resource(item.abspath)
-            ac = item.get_access_control()
-            if ac.is_allowed_to_view(user, item):
-                allowed_items.append(item)
+            resource = root.get_resource(item.abspath)
+            ac = resource.get_access_control()
+            if ac.is_allowed_to_view(user, resource):
+                allowed_items.append((item, resource))
 
         return allowed_items
 
 
     def get_item_value(self, resource, context, item, column):
+        brain, item_resource = item
         if column == 'checkbox':
             # checkbox
-            parent = item.parent
+            parent = item_resource.parent
             if parent is None:
                 return None
-            if item.name in parent.__fixed_handlers__:
+            if item_resource.name in parent.__fixed_handlers__:
                 return None
-            id = resource.get_canonical_path().get_pathto(item.get_abspath())
+            id = resource.get_canonical_path().get_pathto(brain.abspath)
             id = str(id)
             return id, False
         elif column == 'icon':
             # icon
-            path_to_icon = item.get_resource_icon(16)
+            path_to_icon = item_resource.get_resource_icon(16)
             if path_to_icon.startswith(';'):
-                path_to_icon = Path('%s/' % item.name).resolve(path_to_icon)
+                path_to_icon = Path('%s/' % brain.name).resolve(path_to_icon)
             return path_to_icon
         elif column == 'name':
             # Name
-            id = resource.get_canonical_path().get_pathto(item.get_abspath())
+            id = resource.get_canonical_path().get_pathto(brain.abspath)
             id = str(id)
-            view = item.get_view(None)
+            view = item_resource.get_view(None)
             if view is None:
                 return id
-            href = '%s/' % context.get_link(item)
+            href = '%s/' % context.get_link(item_resource)
             return id, href
         elif column == 'title':
             # Title
-            return item.get_property('title')
+            return item_resource.get_property('title')
         elif column == 'format':
             # Type
-            return item.class_title.gettext()
+            return item_resource.class_title.gettext()
         elif column == 'mtime':
             # Last Modified
             accept = context.accept_language
-            return format_datetime(item.get_mtime(), accept=accept)
+            return format_datetime(brain.mtime, accept=accept)
         elif column == 'last_author':
             # Last author
-            if not isinstance(item, VersioningAware):
-                return None
-            revisions = item.get_revisions(context)
-            if not revisions:
-                return None
-            username = revisions[0]['username']
-            try:
-                user = resource.get_resource('/users/%s' % username)
-            except LookupError:
-                return username
-            return user.get_title()
+            return context.root.get_user_title(brain.last_author)
         elif column == 'size':
             # Size
-            return item.get_human_size()
+            return item_resource.get_human_size()
         elif column == 'workflow_state':
             # The workflow state
-            if not isinstance(item, WorkflowAware):
-                return None
-            statename = item.get_statename()
-            state = item.get_state()
-            msg = state['title'].gettext().encode('utf-8')
-            path = context.get_link(item)
-            # TODO Include the template in the base table
-            state = ('<a href="%s/;edit_state" class="workflow">'
-                     '<strong class="wf_%s">%s</strong>'
-                     '</a>') % (path, statename, msg)
-            return XMLParser(state)
+            return get_workflow_preview(item_resource, context)
 
 
     table_actions = [
         RemoveButton, RenameButton, CopyButton, CutButton, PasteButton,
-        PublishButton]
+        PublishButton, RetireButton]
 
 
     #######################################################################
@@ -380,7 +373,7 @@ class Folder_BrowseContent(SearchForm):
         ids = form['ids']
 
         # Clean the copy cookie if needed
-        cut, paths = context.get_cookie('ikaaro_cp', type=CopyCookie)
+        cut, paths = context.get_cookie('ikaaro_cp', datatype=CopyCookie)
 
         # Remove resources
         removed = []
@@ -496,47 +489,52 @@ class Folder_BrowseContent(SearchForm):
     action_paste_schema = {}
     def action_paste(self, resource, context, form):
         # Check there is something to paste
-        cut, paths = context.get_cookie('ikaaro_cp', type=CopyCookie)
+        cut, paths = context.get_cookie('ikaaro_cp', datatype=CopyCookie)
         if len(paths) == 0:
             context.message = messages.MSG_NO_PASTE
             return
 
         # Paste
         target = resource
-        allowed_types = tuple(target.get_document_types())
         pasted = []
         not_allowed = []
         for path in paths:
-            # Check the resource actually exists
-            try:
-                resource = target.get_resource(path)
-            except LookupError:
-                continue
-            if not isinstance(resource, allowed_types):
-                not_allowed.append(resource.name)
+            # Check the source resource still exists
+            source = target.get_resource(path, soft=True)
+            if source is None:
                 continue
 
             # If cut&paste in the same place, do nothing
             if cut is True:
-                source = resource.parent
-                if target.get_canonical_path() == source.get_canonical_path():
-                    pasted.append(resource.name)
+                if target == source.parent:
+                    pasted.append(source.name)
                     continue
 
-            name = generate_name(resource.name, target.get_names(), '_copy_')
-            pasted.append(name)
+            name = generate_name(source.name, target.get_names(), '_copy_')
             if cut is True:
                 # Cut&Paste
-                target.move_resource(path, name)
+                try:
+                    target.move_resource(path, name)
+                except ConsistencyError:
+                    not_allowed.append(source.name)
+                    continue
+                else:
+                    pasted.append(name)
             else:
                 # Copy&Paste
-                target.copy_resource(path, name)
-                # Fix state
-                resource = target.get_resource(name)
-                if isinstance(resource, WorkflowAware):
-                    metadata = resource.metadata
-                    metadata.set_property('state',
-                                          resource.workflow.initstate)
+                try:
+                    target.copy_resource(path, name)
+                except ConsistencyError:
+                    not_allowed.append(source.name)
+                    continue
+                else:
+                    pasted.append(name)
+                    # Fix state
+                    copy = target.get_resource(name)
+                    if isinstance(copy, WorkflowAware):
+                        metadata = copy.metadata
+                        metadata.set_property('state',
+                                              copy.workflow.initstate)
 
         # Cut, clean cookie
         if cut is True:
@@ -554,10 +552,10 @@ class Folder_BrowseContent(SearchForm):
         context.message = message
 
 
-    def action_publish(self, resource, context, form):
+    def _action_workflow(self, resource, context, form, transition, statename,
+                         message):
         resources = [ resource.get_resource(id) for id in form['ids'] ]
         user = context.user
-        transition = 'publish'
         # Check there is at least one item we can publish
         ac = resource.get_access_control()
         allowed = [ x for x in resources
@@ -568,24 +566,32 @@ class Folder_BrowseContent(SearchForm):
 
         # Publish
         for item in allowed:
-            if item.get_statename() == 'public':
+            if item.get_statename() == statename:
                 continue
             # Update workflow history
-            property = {'date': datetime.now(),
-                        'user': user.name,
-                        'name': transition,
-                        'comments': u""}
-            item.set_property('wf_transition', property)
-            item.do_trans(transition)
+            item.make_transition(transition)
 
         # Ok
-        context.message = messages.MSG_PUBLISHED
+        context.message = message
+
+
+    def action_publish(self, resource, context, form):
+        self._action_workflow(resource, context, form, 'publish', 'public',
+                              messages.MSG_PUBLISHED)
+
+
+    def action_retire(self, resource, context, form):
+        self._action_workflow(resource, context, form, 'retire', 'private',
+                              messages.MSG_RETIRED)
 
 
 
 class Folder_PreviewContent(Folder_BrowseContent):
 
     title = MSG(u'Preview Content')
+    styles = ['/ui/gallery/style.css']
+    scripts = ['/ui/gallery/javascript.js']
+
     context_menus = Folder_BrowseContent.context_menus + [ZoomMenu()]
     # Table
     table_template = '/ui/folder/browse_image.xml'
@@ -602,7 +608,7 @@ class Folder_PreviewContent(Folder_BrowseContent):
 
     def get_items(self, resource, context):
         # Show only images
-        query = OrQuery(PhraseQuery('is_image', '1'),
+        query = OrQuery(PhraseQuery('is_image', True),
                         PhraseQuery('format', 'folder'))
         return Folder_BrowseContent.get_items(self, resource, context, query)
 
@@ -648,9 +654,6 @@ class Folder_PreviewContent(Folder_BrowseContent):
 
 
     def get_table_namespace(self, resource, context, items):
-        context.styles.append('/ui/gallery/style.css')
-        context.scripts.append('/ui/gallery/javascript.js')
-
         # Get from the query
         query = context.query
         width = query['width']
@@ -674,8 +677,8 @@ class Folder_PreviewContent(Folder_BrowseContent):
             else:
                 onclick = None
             actions.append(
-                {'name': button.name,
-                 'value': button.title,
+                {'value': button.name,
+                 'title': button.title,
                  'class': button.css,
                  'onclick': onclick})
 
@@ -686,14 +689,15 @@ class Folder_PreviewContent(Folder_BrowseContent):
         columns = self._get_table_columns(resource, context)
         rows = []
         for item in items:
+            item_brain, item_resource = item
             row = {'checkbox': False,
                    # These are required for internal use
-                   'title_or_name': item.get_title(),
+                   'title_or_name': item_resource.get_title(),
                    'workflow_statename': None}
             # XXX Already hard-coded in the catalog search
-            row['is_folder'] = (item.class_id == 'folder')
-            if isinstance(item, WorkflowAware):
-                row['workflow_statename'] = item.get_statename()
+            row['is_folder'] = (item_brain.format == 'folder')
+            if isinstance(item_resource, WorkflowAware):
+                row['workflow_statename'] = item_resource.get_statename()
             for name, title, sortable in columns:
                 value = self.get_item_value(resource, context, item, name)
                 if value is None:
@@ -709,7 +713,7 @@ class Folder_PreviewContent(Folder_BrowseContent):
                         value, href = value
                         href = get_reference(href)
                         if row['is_folder']:
-                            href = href.resolve2(';preview_content')
+                            href = href.resolve_name(';preview_content')
                         href = href.replace(size=current_size, width=width,
                                 height=height)
                         href = str(href)
@@ -724,37 +728,19 @@ class Folder_PreviewContent(Folder_BrowseContent):
         widths = ", ".join([repr(o['name']) for o in ImageWidth.get_options()
                             if o['name'].strip()])
 
-        return {'size': current_size,
-                'width': width,
-                'height': height,
-                'widths': widths,
-                'css': self.table_css,
-                'columns': table_head,
-                'rows': rows,
-                'actions': actions}
+        return {
+            'root': resource.parent is None,
+            'size': current_size,
+            'width': width,
+            'height': height,
+            'widths': widths,
+            'css': self.table_css,
+            'columns': table_head,
+            'rows': rows,
+            'actions': actions}
 
 
 
-class Folder_LastChanges(Folder_BrowseContent):
-
-    title = MSG(u"Last Changes")
-
-    def get_query_schema(self):
-        schema = Folder_BrowseContent.get_query_schema(self)
-
-        # Override the default values
-        schema['reverse'] = Boolean(default=True)
-        schema['sort_by'] = String(default='mtime')
-        schema['search_subfolders'] = Boolean(default=True)
-        return schema
-
-
-    def get_items(self, resource, context):
-        # Show only version aware resources
-        query = PhraseQuery('is_version_aware', '1')
-        return Folder_BrowseContent.get_items(self, resource, context, query)
-
-
 
 class Folder_Orphans(Folder_BrowseContent):
     """Orphans are files not referenced in another resource of the database.
@@ -802,10 +788,6 @@ class Folder_Thumbnail(BaseView):
 
     default_icon = '/ui/gallery/folder.png'
 
-    def get_mtime(self, resource):
-        return resource.get_mtime()
-
-
     def GET(self, resource, context):
         from file import Image
 
@@ -839,8 +821,7 @@ class Folder_Thumbnail(BaseView):
         # The folder thumbnail was cached in the folder handler
         # Accessible images depend on too many parameters
 
-        response = context.response
-        response.set_header('Content-Type', 'image/%s' % format)
+        context.content_type = 'image/%s' % format
         return data
 
 
@@ -851,20 +832,28 @@ class GoToSpecificDocument(BaseView):
     title = MSG(u'Front Page')
     icon = 'view.png'
     specific_document = 'FrontPage'
+    specific_view = None
 
 
     def get_specific_document(self, resource, context):
         return self.specific_document
 
 
+    def get_specific_view(self, resource, context):
+        return self.specific_view
+
+
     def GET(self, resource, context):
         specific_document = self.get_specific_document(resource, context)
+        specific_view = self.get_specific_view(resource, context)
         goto = '%s/%s' % (context.get_link(resource), specific_document)
+        if specific_view:
+            goto = '%s/;%s' % (goto, specific_view)
         goto = get_reference(goto)
 
         # Keep the message
-        if context.has_form_value('message'):
-            message = context.get_form_value('message')
+        message = context.get_form_value('message')
+        if message:
             goto = goto.replace(message=message)
 
         return goto
