@@ -19,15 +19,19 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from itools
+from itools.core import merge_dicts
 from itools.datatypes import DataType, Date, Enumerate, Boolean
 from itools.gettext import MSG
-from itools.handlers import merge_dicts
 from itools.html import stream_to_str_as_xhtml, stream_to_str_as_html
 from itools.html import xhtml_doctype, sanitize_stream
 from itools.stl import stl
 from itools.web import STLForm, get_context
 from itools.xml import XMLParser
 
+# Import from ikaaro
+from ikaaro.workflow import get_workflow_preview
+
+
 
 stl_namespaces = {
     None: 'http://www.w3.org/1999/xhtml',
@@ -90,17 +94,20 @@ def get_default_widget(datatype):
 
 class Widget(object):
 
-    type = 'text'
     size = None
+    tip = None
+    suffix = None
+    type = 'text'
 
     template = list(XMLParser(
-        """<input type="${type}" name="${name}" value="${value}" size="${size}"
-        />""",
+        """<input type="${type}" id="${id}" name="${name}" value="${value}"
+          size="${size}" />""",
         stl_namespaces))
 
 
     def __init__(self, name, template=None, template_multiple=None, **kw):
         self.name = name
+        self.id = name.replace('_', '-')
         if template is not None:
             self.template = template
         if template_multiple is not None:
@@ -121,6 +128,7 @@ class Widget(object):
         return {
             'type': self.type,
             'name': self.name,
+            'id': self.id,
             'value': value,
             'size': self.size}
 
@@ -160,7 +168,7 @@ class PasswordWidget(Widget):
 class ReadOnlyWidget(Widget):
 
     template = list(XMLParser(
-        """<input type="hidden" name="${name}" value="${value}" />
+        """<input type="hidden" id="${id}" name="${name}" value="${value}" />
            ${displayed}""", stl_namespaces))
 
 
@@ -185,6 +193,7 @@ class ReadOnlyWidget(Widget):
 
         return {
             'name': self.name,
+            'id': self.id,
             'value': value,
             'displayed': displayed}
 
@@ -196,7 +205,7 @@ class MultilineWidget(Widget):
     cols = 60
 
     template = list(XMLParser(
-        """<textarea rows="${rows}" cols="${cols}" name="${name}"
+        """<textarea rows="${rows}" cols="${cols}" id="${id}" name="${name}"
         >${value}</textarea>""",
         stl_namespaces))
 
@@ -204,6 +213,7 @@ class MultilineWidget(Widget):
     def get_namespace(self, datatype, value):
         return {
             'name': self.name,
+            'id': self.id,
             'value': value,
             'rows': self.rows,
             'cols': self.cols}
@@ -213,7 +223,7 @@ class MultilineWidget(Widget):
 class CheckBoxWidget(Widget):
 
     template = list(XMLParser("""
-        <input type="checkbox" name="${name}" value="${value}"
+        <input type="checkbox" id="${id}" name="${name}" value="${value}"
           checked="${is_selected}" />
         """, stl_namespaces))
 
@@ -221,6 +231,7 @@ class CheckBoxWidget(Widget):
     def get_namespace(self, datatype, value):
         return {
             'name': self.name,
+            'id': self.id,
             'value': value,
             'is_selected': getattr(self, 'is_selected', False)}
 
@@ -229,7 +240,7 @@ class CheckBoxWidget(Widget):
 class BooleanCheckBox(Widget):
 
     template = list(XMLParser("""
-        <input type="checkbox" name="${name}" value="1"
+        <input type="checkbox" id="${id}" name="${name}" value="1"
           checked="${is_selected}" />
         """, stl_namespaces))
 
@@ -237,6 +248,7 @@ class BooleanCheckBox(Widget):
     def get_namespace(self, datatype, value):
         return {
             'name': self.name,
+            'id': self.id,
             'is_selected': value in [True, 1, '1']}
 
 
@@ -244,16 +256,16 @@ class BooleanCheckBox(Widget):
 class BooleanRadio(Widget):
 
     template = list(XMLParser("""
-        <label for="${name}_yes">${labels/yes}</label>
-        <input id="${name}_yes" name="${name}" type="radio" value="1"
+        <label for="${id}-yes">${labels/yes}</label>
+        <input id="${id}-yes" name="${name}" type="radio" value="1"
           checked="checked" stl:if="is_yes"/>
-        <input id="${name}_yes" name="${name}" type="radio" value="1"
+        <input id="${id}-yes" name="${name}" type="radio" value="1"
           stl:if="not is_yes"/>
 
-        <label for="${name}_no">${labels/no}</label>
-        <input id="${name}_no" name="${name}" type="radio" value="0"
+        <label for="${id}-no">${labels/no}</label>
+        <input id="${id}-no" name="${name}" type="radio" value="0"
           checked="checked" stl:if="not is_yes"/>
-        <input id="${name}_no" name="${name}" type="radio" value="0"
+        <input id="${id}-no" name="${name}" type="radio" value="0"
           stl:if="is_yes"/>
         """, stl_namespaces))
 
@@ -263,6 +275,7 @@ class BooleanRadio(Widget):
         labels = getattr(self, 'labels', default_labels)
         return {
             'name': self.name,
+            'id': self.id,
             'is_yes': value in [True, 1, '1'],
             'labels': labels}
 
@@ -271,7 +284,7 @@ class BooleanRadio(Widget):
 class SelectWidget(Widget):
 
     template = list(XMLParser("""
-        <select name="${name}" multiple="${multiple}" size="${size}"
+        <select id="${id}" name="${name}" multiple="${multiple}" size="${size}"
             class="${css}">
           <option value="" stl:if="has_empty_option"></option>
           <option stl:repeat="option options" value="${option/name}"
@@ -291,6 +304,7 @@ class SelectWidget(Widget):
             'css': getattr(self, 'css', None),
             'has_empty_option': getattr(self, 'has_empty_option', True),
             'name': self.name,
+            'id': self.id,
             'multiple': datatype.multiple,
             'options': value,
             'size':  getattr(self, 'size', None)}
@@ -305,23 +319,25 @@ class SelectRadio(Widget):
             stl:if="none_selected"/>
           <input type="radio" name="${name}" value=""
             stl:if="not none_selected"/>
-          <br/>
+          <stl:block stl:if="not is_inline"><br/></stl:block>
         </stl:block>
         <stl:block stl:repeat="option options">
-          <input type="radio" id="${name}_${option/name}" name="${name}"
+          <input type="radio" id="${id}-${option/name}" name="${name}"
             value="${option/name}" checked="checked"
             stl:if="option/selected"/>
-          <input type="radio" id="${name}_${option/name}" name="${name}"
+          <input type="radio" id="${id}-${option/name}" name="${name}"
             value="${option/name}" stl:if="not option/selected"/>
-          <label for="${name}_${option/name}">${option/value}</label><br/>
+          <label for="${id}-${option/name}">${option/value}</label>
+          <stl:block stl:if="not is_inline"><br/></stl:block>
         </stl:block>
         """, stl_namespaces))
 
     template_multiple = list(XMLParser("""
         <stl:block stl:repeat="option options">
-          <input type="checkbox" name="${name}" id="${name}_${option/name}"
+          <input type="checkbox" name="${name}" id="${id}-${option/name}"
             value="${option/name}" checked="${option/selected}" />
-          <label for="${name}_${option/name}">${option/value}</label><br/>
+          <label for="${id}-${option/name}">${option/value}</label>
+          <stl:block stl:if="not is_inline"><br/></stl:block>
         </stl:block>
         """, stl_namespaces))
 
@@ -343,6 +359,7 @@ class SelectRadio(Widget):
         else:
             options = value
 
+        is_inline = getattr(self, 'is_inline', False)
         has_empty_option = getattr(self, 'has_empty_option', True)
         for option in options:
             if option['selected'] is True:
@@ -354,6 +371,8 @@ class SelectRadio(Widget):
                 options[0]['selected'] = True
         return {
             'name': self.name,
+            'id': self.id,
+            'is_inline': is_inline,
             'has_empty_option': has_empty_option,
             'none_selected': none_selected,
             'options': options}
@@ -362,59 +381,21 @@ class SelectRadio(Widget):
 
 class DateWidget(Widget):
 
+    tip = MSG(u"Format: 'yyyy-mm-dd'")
+
     template = list(XMLParser("""
-        <input type="text" name="${name}" value="${value}" id="${name}"
-          size="${size}"/>
-        <input id="trigger_date_${name}" type="button" value="..."
-          name="trigger_date_${name}" class="${class}" />
+        <input type="text" name="${name}" value="${value}" id="${id}"
+          class="dateField" size="${size}" />
+        <input type="button" value="..." class="${class}" />
         <script language="javascript">
-          Calendar.setup({inputField: "${name}", ifFormat: "${format}",
-                          showsTime: ${show_time}, timeFormat: "24",
-                          button: "trigger_date_${name}"});
+          jQuery( "input.dateField" ).dynDateTime({
+            ifFormat: "${format}",
+            showsTime: ${show_time},
+            timeFormat: "24",
+            button: ".next()" });
         </script>
         """, stl_namespaces))
 
-    template_multiple = list(XMLParser("""
-        <table class="table_calendar">
-          <tr>
-            <td>
-              <textarea rows="5" cols="25" name="${name}" id="${name}"
-                >${value}</textarea>
-              <input type="button" value="update" id="btn_blur_${name}"
-                onclick="tableFlatOuputOnBlur(elt_${name}, cal_${name});" />
-            </td>
-            <td>
-              <div id="calendar-flat-${name}" style="float: left;"> </div>
-              <script type="text/javascript">
-                var MA_${name} = [];
-                <stl:block stl:repeat="date dates">
-                MA_${name}.push(str_to_date('${date}'));
-                </stl:block>
-                var cal_${name} = Calendar.setup({
-                    displayArea  : '${name}',
-                    flat         : 'calendar-flat-${name}',
-                    flatCallback : tableFlatCallback,
-                    multiple     : MA_${name},
-                    ifFormat     : '${format}'});
-                var elt_${name} = document.getElementById('${name}');
-                if (!browser.isIE) {
-                    $("#btn_blur_${name}").style.display = 'none';
-                    elt_${name}.setAttribute('onblur',
-                        'tableFlatOuputOnBlur(elt_${name}, cal_${name})');
-                }
-              </script>
-            </td>
-          </tr>
-        </table>
-        """, stl_namespaces))
-
-
-    def get_template(self, datatype, value):
-        if datatype.multiple:
-            return self.template_multiple
-        return self.template
-
-
     def get_namespace(self, datatype, value):
         if value is None:
             value = ''
@@ -425,58 +406,71 @@ class DateWidget(Widget):
         css = getattr(self, 'css', None)
         size = getattr(self, 'size', None)
 
-        if datatype.multiple:
-            if isinstance(value, list): # ['2007-08-01\r\n2007-08-02']
-                value = value[0]
-            return {'name': self.name, 'format': format,
-                    'show_time': show_time, 'class': css,
-                    'value': value, 'dates': value.splitlines()}
-
-        return {'name': self.name, 'format': format,
-                'show_time': show_time, 'class': css,
-                'size': size, 'value': value}
+        return {'name': self.name, 'id': self.id,
+                'format': format, 'show_time': show_time,
+                'class': css, 'size': size, 'value': value}
 
 
 
 class PathSelectorWidget(TextWidget):
 
-    target = '.'
+    action = 'add_link'
+    display_workflow = True
+
     template = list(XMLParser(
     """
-    <input type="text" id="selector_${name}" size="${size}" name="${name}"
+    <input type="text" id="selector-${id}" size="${size}" name="${name}"
       value="${value}" />
-    <input id="selector_button_${name}" type="button" value="..."
+    <input id="selector-button-${id}" type="button" value="..."
       name="selector_button_${name}"
-      onclick="popup(';add_link?target_id=selector_${name}&amp;mode=input&amp;bc_target=${target}', 620, 300);"/>
+      onclick="popup(';${action}?target_id=selector-${id}&amp;mode=input', 620, 300);"/>
+    ${workflow_state}
     """, stl_namespaces))
 
 
     def get_namespace(self, datatype, value):
-        namespace = TextWidget.get_namespace(self, datatype, value)
-        namespace['target'] = self.target
-        return namespace
+        workflow_state = None
+        if self.display_workflow:
+            context = get_context()
+            if type(value) is not str:
+                value = datatype.encode(value)
+            if value:
+                resource = context.resource.get_resource(value, soft=True)
+                if resource:
+                    workflow_state = get_workflow_preview(resource, context)
+
+        return {
+            'type': self.type,
+            'name': self.name,
+            'id': self.id,
+            'value': value,
+            'size': self.size,
+            'action': self.action,
+            'workflow_state': workflow_state}
 
 
 
-class ImageSelectorWidget(TextWidget):
+class ImageSelectorWidget(PathSelectorWidget):
 
+    action = 'add_image'
     width = 128
     height = 128
 
     template = list(XMLParser(
     """
-    <input type="text" id="selector_${name}" size="${size}" name="${name}"
+    <input type="text" id="selector-${id}" size="${size}" name="${name}"
       value="${value}" />
-    <input id="selector_button_${name}" type="button" value="..."
+    <input id="selector-button-${id}" type="button" value="..."
       name="selector_button_${name}"
-      onclick="popup(';add_image?target_id=selector_${name}&amp;mode=input', 620, 300);" />
+      onclick="popup(';${action}?target_id=selector-${id}&amp;mode=input', 620, 300);" />
+    ${workflow_state}
     <br/>
     <img src="${value}/;thumb?width=${width}&amp;height=${height}" stl:if="value"/>
     """, stl_namespaces))
 
 
     def get_namespace(self, datatype, value):
-        return merge_dicts(TextWidget.get_namespace(self, datatype, value),
+        return merge_dicts(PathSelectorWidget.get_namespace(self, datatype, value),
                            width=self.width, height=self.height)
 
 
@@ -504,7 +498,7 @@ class RTEWidget(Widget):
                 'formatselect')
     toolbar3 = None
     resizing = True
-    plugins = 'safari,table,media'
+    plugins = 'safari,table,media,advimage,advlink'
     # Extending the existing rule set.
     extended_valid_elements = None
     # css
@@ -550,6 +544,7 @@ class RTEWidget(Widget):
                 'css': ','.join(css_names),
                 'extended_valid_elements': self.extended_valid_elements,
                 'form_name': self.name,
+                'id': self.id,
                 'height': self.height,
                 'language': current_language,
                 'plugins': self.plugins,
@@ -595,7 +590,7 @@ class AutoForm(STLForm):
     required_msg = None
     template = '/ui/auto_form.xml'
     submit_value = MSG(u'Save')
-    submit_class = 'button_ok'
+    submit_class = 'button-ok'
 
 
     def get_widgets(self, resource, context):
@@ -603,6 +598,8 @@ class AutoForm(STLForm):
 
 
     def get_namespace(self, resource, context):
+        widgets_namespace = STLForm.get_namespace(self, resource, context)
+
         here = context.resource
         # Local Variables
         fields = self.get_schema(resource, context)
@@ -612,7 +609,7 @@ class AutoForm(STLForm):
         required_msg = self.required_msg
         if required_msg is None:
             required_msg = MSG(
-                u'The <span class="field_is_required">emphasized</span> fields'
+                u'The <span class="field-is-required">emphasized</span> fields'
                 u' are required.')
         required_msg = required_msg.gettext()
         required_msg = required_msg.encode('utf-8')
@@ -620,7 +617,6 @@ class AutoForm(STLForm):
 
         # Build widgets namespace
         has_required_widget = False
-        widgets_namespace = self.build_namespace(resource, context)
         ns_widgets = []
         for widget in widgets:
             datatype = fields[widget.name]
@@ -633,6 +629,8 @@ class AutoForm(STLForm):
             widget_namespace['mandatory'] = is_mandatory
             widget_namespace['multiple'] = datatype.multiple
             widget_namespace['is_date'] = issubclass(datatype, Date)
+            widget_namespace['tip'] = widget.tip
+            widget_namespace['suffix'] = widget.suffix
             widget_namespace['widget'] = widget.to_html(datatype, value)
             ns_widgets.append(widget_namespace)
 
