@@ -15,7 +15,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from itools
-from itools import get_abspath
+from itools.core import get_abspath
 
 # Import from ikaaro
 from ikaaro.skins import register_skin
