@@ -32,9 +32,8 @@ from itools.xml import XMLParser, START_ELEMENT
 # Import from ikaaro
 from ikaaro.file import Image
 from ikaaro.folder import Folder
-from ikaaro.html import ResourceWithHTML, WebPage
-from ikaaro.registry import register_resource_class
 from ikaaro.views import CompositeForm, ContextMenu
+from ikaaro.webpage import ResourceWithHTML, WebPage
 from ikaaro.wiki import WikiPage
 from ikaaro.workflow import WorkflowAware
 
@@ -59,32 +58,32 @@ class Dressable_Menu(ContextMenu):
                 warn(msg)
                 continue
             name, cls = value
-            if resource.has_resource(name):
-                # Add edit link
-                items.append(
-                    {'name': name,
-                     'title': MSG(u'%s %s' % (cls.class_title.gettext(), name)),
-                     'href': '%s%s/;edit' % (base_path, name),
-                     'class': 'nav_active'})
-            else:
+            if resource.get_resource(name, soft=True) is None:
                 # Add new_resource link
                 items.append(
                     {'name': name,
                      'title': MSG(u'Add new %s' % cls.class_title.gettext()),
                      'href': ('%s;new_resource?type=%s&title=%s' %
                               (base_path, cls.class_id, name)),
-                     'class': 'nav_active'})
+                     'class': 'nav-active'})
+            else:
+                # Add edit link
+                items.append(
+                    {'name': name,
+                     'title': MSG(u'%s %s' % (cls.class_title.gettext(), name)),
+                     'href': '%s%s/;edit' % (base_path, name),
+                     'class': 'nav-active'})
         items.sort(key=itemgetter('name'))
         # Dressable metadata
         items.insert(0,
             {'title': MSG(u'Metadata'),
              'href': '%s;edit' % base_path,
-             'class': 'nav_active'})
+             'class': 'nav-active'})
         # Back to preview
         items.append(
             {'title': MSG(u'Preview'),
              'href': '%s.' % base_path,
-             'class': 'nav_active'})
+             'class': 'nav-active'})
         return items
 
 
@@ -120,13 +119,13 @@ class Dressable_View(CompositeForm):
             content = ''
             if isinstance(value, tuple):
                 name, kk = value
-                if resource.has_resource(name):
-                    item = resource.get_resource(name)
+                item = resource.get_resource(name, soft=True)
+                if item is None:
+                    value = None
+                else:
                     # get view to show current item
                     method = self.get_view(resource, context, item)
                     value = method(item, context)
-                else:
-                    value = None
             else:
                 # get view to display
                 value = getattr(resource, 'data')(context)
@@ -238,5 +237,3 @@ class Dressable(Folder, ResourceWithHTML):
                 handler.set_changed()
                 handler.events = events
 
-
-register_resource_class(Dressable)
