@@ -30,19 +30,28 @@ from ikaaro import messages
 from ikaaro.buttons import Button
 from ikaaro.exceptions import ConsistencyError
 from ikaaro.folder import Folder
+from ikaaro.folder_views import Folder_NewResource, Folder_BrowseContent
+from ikaaro.folder_views import Folder_Rename, Folder_PreviewContent
+from ikaaro.folder_views import Folder_Thumbnail, GoToSpecificDocument
+from ikaaro.folder_views import Folder_Orphans
 from ikaaro.forms import PathSelectorWidget
 from ikaaro.forms import TextWidget, SelectWidget, ReadOnlyWidget
-from ikaaro.registry import register_resource_class
+from ikaaro.resource_views import DBResource_AddLink
+from ikaaro.revisions_views import DBResource_CommitLog
 from ikaaro.table import OrderedTableFile, OrderedTable
 from ikaaro.table_views import OrderedTable_View
-from ikaaro.workflow import WorkflowAware
+from ikaaro.workflow import get_workflow_preview
 
 
 
-def get_reference_and_path(path):
+def get_reference_and_path(value):
     """Return the reference associated to the path and the path
     without query/fragment.
     """
+    # Be robust if the path is multilingual (Unicode multiple)
+    path = value
+    if type(path) is unicode:
+        path = Unicode.encode(value)
     ref = get_reference(path)
     return ref, str(ref.path)
 
@@ -61,7 +70,7 @@ class Target(Enumerate):
 
 class MenuFile(OrderedTableFile):
 
-    record_schema = {
+    record_properties = {
         'title': Unicode(multiple=True),
         'path': String,
         'target': Target(mandatory=True, default='_top'),
@@ -74,15 +83,16 @@ class ChildButton(Button):
     access = 'is_allowed_to_edit'
     name = 'add_child'
     title = MSG(u'Add Child')
-    css = 'button_add'
+    css = 'button-add'
 
 
 
 class Menu_View(OrderedTable_View):
 
+    access = 'is_allowed_to_edit'
     schema = {
-        'ids': Integer(multiple=True, mandatory=True),
-    }
+        'ids': Integer(multiple=True, mandatory=True)}
+    styles = ['/ui/future/style.css']
 
     table_actions = [ChildButton] + OrderedTable_View.table_actions
 
@@ -121,22 +131,21 @@ class Menu_View(OrderedTable_View):
             if ref.path.is_absolute():
                 site_root = context.resource.get_site_root()
                 path = site_root.get_abspath().resolve2('.%s' % path)
-            if resource.has_resource(path) is False:
-                # Broken link
+            resource_item = resource.get_resource(path, soft=True)
+            # Broken link
+            if resource_item is None:
                 return value
-            resource_item = resource.get_resource(path)
             # Build the new reference with the right path
             ref2 = deepcopy(ref)
             ref2.path = context.get_link(resource_item)
             return value, str(ref2)
         elif column == 'child':
-            child = None
-            parent = resource.parent
-            child_path = value
-            if value and parent.has_resource(value):
-                child = parent.get_resource(value)
-                child = 'edit', context.get_link(child)
-            return child
+            if not value:
+                return None
+            child = resource.parent.get_resource(value, soft=True)
+            if child is None:
+                return None
+            return 'edit', context.get_link(child)
         elif column == 'workflow_state':
             path = get_value(item, 'path')
             if not path:
@@ -150,8 +159,9 @@ class Menu_View(OrderedTable_View):
             if ref.path.is_absolute():
                 site_root = context.resource.get_site_root()
                 path = site_root.get_abspath().resolve2('.%s' % path)
-            if resource.has_resource(path) is False:
-                # Broken link
+            item_resource = resource.get_resource(path, soft=True)
+            # Broken link
+            if item_resource is None:
                 title = MSG(u'The resource does not exist anymore.')
                 title = title.gettext().encode('utf-8')
                 label = MSG(u'Broken').gettext().encode('utf-8')
@@ -160,27 +170,11 @@ class Menu_View(OrderedTable_View):
                          'title="%s"><strong class="broken">%s</strong>'
                          '</a>') % (link, item.id, title, label)
                 return XMLParser(state)
-            item_resource = resource.get_resource(path)
-            if not isinstance(item_resource, WorkflowAware):
-                return None
-            statename = item_resource.get_statename()
-            state = item_resource.get_state()
-            msg = state['title'].gettext().encode('utf-8')
-            # Build the new reference with the right path
-            ref2 = deepcopy(ref)
-            ref2.path = context.get_link(item_resource)
-            # TODO Include the template in the base table
-            state = ('<a href="%s/;edit_state" class="workflow">'
-                     '<strong class="wf_%s">%s</strong>'
-                     '</a>') % (str(ref2), statename, msg)
-            return XMLParser(state)
+            # The workflow state
+            return get_workflow_preview(item_resource, context)
 
         return OrderedTable_View.get_item_value(self, resource, context, item,
-                                               column)
-
-    def get_namespace(self, resource, context):
-        context.styles.append('/ui/future/style.css')
-        return OrderedTable_View.get_namespace(self, resource, context)
+                                                column)
 
 
     #######################################################################
@@ -219,7 +213,7 @@ class Menu_View(OrderedTable_View):
         context.message = message
 
         # Reindex the resource
-        context.server.change_resource(resource)
+        context.database.change_resource(resource)
 
 
     def action_add_child(self, resource, context, form):
@@ -230,7 +224,7 @@ class Menu_View(OrderedTable_View):
             parent_record = handler.get_record(parent_id)
             # check if the child already exists
             child_path = handler.get_record_value(parent_record, 'child')
-            if child_path and parent.has_resource(child_path):
+            if child_path and parent.get_resource(child_path, soft=True):
                 continue
 
             names = parent.get_names()
@@ -252,6 +246,13 @@ class Menu_View(OrderedTable_View):
 
 
 
+class Menu_AddLink(DBResource_AddLink):
+
+    def get_start(self, resource):
+        return resource.get_site_root()
+
+
+
 class Menu(OrderedTable):
 
     class_id = 'ikaaro-menu'
@@ -260,6 +261,7 @@ class Menu(OrderedTable):
     class_views = ['view', 'add_record']
 
     # Views
+    add_link = Menu_AddLink()
     view = Menu_View()
 
 
@@ -272,24 +274,25 @@ class Menu(OrderedTable):
     def del_record(self, id):
         handler = self.handler
         record = handler.get_record(id)
-        record_schema = handler.record_schema
+        record_properties = handler.record_properties
 
         # Delete submenu
-        if 'child' in record_schema:
+        if 'child' in record_properties:
             child_path = handler.get_record_value(record, 'child')
-            container = self.parent
-            user = get_context().user
-            if child_path and container.has_resource(child_path):
-                child = container.get_resource(child_path)
-                ac = child.get_access_control()
-                if ac.is_allowed_to_remove(user, child):
-                    if child.handler.get_n_records():
+            if child_path:
+                container = self.parent
+                child = container.get_resource(child_path, soft=True)
+                if child is not None:
+                    ac = child.get_access_control()
+                    user = get_context().user
+                    if ac.is_allowed_to_remove(user, child):
+                        if child.handler.get_n_records():
+                            raise NotAllowedError
+                        # Remove the child table
+                        # May raise a ConsistencyError
+                        container.del_resource(child_path)
+                    else:
                         raise NotAllowedError
-                    # Remove the child table
-                    # May raise a ConsistencyError
-                    container.del_resource(child_path)
-                else:
-                    raise NotAllowedError
 
         # Delete the record
         handler.del_record(id)
@@ -311,26 +314,25 @@ class Menu(OrderedTable):
             path, method = path.split(';')
             if ref.path.is_absolute():
                 path = site_root_abspath.resolve2('.%s' % path)
-            if self.has_resource(path):
-                resource = self.get_resource(path)
-                ac = resource.get_access_control()
-                view = resource.get_view(method, ref.query)
-                # Check ACL
-                if view:
-                    return ac.is_access_allowed(user, resource, view)
-            # Miss
-            return False
+            resource = self.get_resource(path, soft=True)
+            if resource is None:
+                return False
+            # Check ACL
+            ac = resource.get_access_control()
+            view = resource.get_view(method, ref.query)
+            if view:
+                return ac.is_access_allowed(user, resource, view)
         else:
             # Internal link
             if ref.path.is_absolute():
                 path = site_root_abspath.resolve2('.%s' % path)
-            if self.has_resource(path) is False:
-                # Broken link
+            resource = self.get_resource(path, soft=True)
+            # Broken link
+            if resource is None:
                 return False
 
             # FIXME We should take into account the show_first_child
             # parameter
-            resource = self.get_resource(path)
             ac = resource.get_access_control()
             return ac.is_allowed_to_view(user, resource)
 
@@ -339,8 +341,9 @@ class Menu(OrderedTable):
                                  use_first_child=False, flat=False):
         parent = self.parent
         handler = self.handler
+        menu_abspath = self.get_abspath()
         here = context.resource
-        here_abspath = here.get_abspath()
+        here_abspath = here.get_canonical_path()
         site_root_abspath = here.get_site_root().get_abspath()
         items = []
         tabs = {}
@@ -363,18 +366,26 @@ class Menu(OrderedTable):
                 if not ref.scheme:
                     # FIXME We should check the ACL of the method
                     # Get the resource associated to the method
-                    _path, _method = path.split(';')
+                    _path, method_name = path.split(';')
                     if ref.path.is_absolute():
                         real_path = site_root_abspath.resolve2('.%s' % _path)
                     else:
                         real_path = _path
                     resource = self.get_resource(real_path)
-                    if here_abspath == resource.get_abspath():
-                        if context.view_name == _method:
-                            css = 'in_path'
-                    id += _method
+                    if here_abspath == resource.get_canonical_path():
+                        if context.view_name == method_name:
+                            css = 'in-path'
+                    id += method_name
+                    # Build the new reference with the right path
+                    ref2 = deepcopy(ref)
+                    ref2.path = '%s/;%s' % (context.get_link(resource),
+                                            method_name)
+                else:
+                    id += str(record.id)
+                    ref2 = ref
+
                 items.append({'id': id,
-                              'path': str(ref),
+                              'path': str(ref2),
                               'real_path': None,
                               'title': title,
                               'description': None,
@@ -386,45 +397,63 @@ class Menu(OrderedTable):
                 # Internal link
                 if ref.path.is_absolute():
                     path = site_root_abspath.resolve2('.%s' % path)
-                resource = self.get_resource(path)
+                resource = self.get_resource(path, soft=True)
+                # Broken link
+                if resource is None:
+                    continue
                 name = resource.name
 
-                # use first child by default we use the resource itself
+                # Use first child by default we use the resource itself
                 resource_path = path
+                # Keep the real path to avoid highlight problems
+                resource_original_path = path
                 if depth > 1:
                     # Check the child
                     child_path = get_value(record, 'child')
-                    if child_path and parent.has_resource(child_path):
-                        # Sub level
-                        child = parent.get_resource(child_path)
-                        get_menu_ns_lvl = child.get_menu_namespace_level
-                        subtabs = get_menu_ns_lvl(context, url, depth - 1,
-                                                  use_first_child, flat)
-                        # use first child
-                        if subtabs['items']:
-                            sub_path = subtabs['items'][0]['real_path']
-                            # if the first child is an external link => skip it
-                            if sub_path is not None:
-                                resource_path = sub_path
+                    if child_path:
+                        child = parent.get_resource(child_path, soft=True)
+                        if child is not None:
+                            # Sub level
+                            get_menu_ns_lvl = child.get_menu_namespace_level
+                            subtabs = get_menu_ns_lvl(context, url, depth - 1,
+                                                      use_first_child, flat)
+                            # use first child
+                            if use_first_child and subtabs['items']:
+                                sub_path = subtabs['items'][0]['real_path']
+                                # if the first child is an external link =>
+                                # skip it
+                                if sub_path is not None:
+                                    resource_path = sub_path
 
                 # Set active, in_path
-                active, in_path = False, name in url
-                if here_abspath == resource.get_abspath():
+                active = False
+                if here_abspath == resource.get_canonical_path():
                     active, in_path = True, False
+                else:
+                    # Use the original path for the highlight
+                    res_abspath = menu_abspath.resolve2(resource_original_path)
+                    common_prefix = here_abspath.get_prefix(res_abspath)
+                    in_path = False
+                    # Avoid to always set the site_root entree 'in_path'
+                    # If common prefix equals site root abspath
+                    # set in_path to False otherwise compare
+                    # common_prefix and res_abspath
+                    if common_prefix != site_root_abspath:
+                        in_path = (common_prefix == res_abspath)
 
-                # Set css class to 'active', 'in_path' or None
-                css = (active and 'in_path') or (in_path and 'in_path') or None
+                # Set css class to 'active', 'in-path' or None
+                css = 'in-path' if (active or in_path) else None
 
                 # Build the new reference with the right path
                 ref2 = deepcopy(ref)
-                resource = self.get_resource(path)
+                resource = self.get_resource(resource_path)
                 ref2.path = context.get_link(resource)
                 items.append({'id': 'menu_%s' % name,
                               'path': str(ref2),
                               'real_path': resource.get_abspath(),
                               'title': title,
                               'description': None, # FIXME
-                              'in_path': css == 'in_path' or active,
+                              'in_path': css == 'in-path' or active,
                               'active': active,
                               'class': css,
                               'target': target})
@@ -449,7 +478,7 @@ class Menu(OrderedTable):
             ref, path = get_reference_and_path(uri)
             if ref.scheme:
                 # External link
-                uris.append(uri)
+                uris.append(ref)
                 continue
             if self._is_allowed_to_access(context, uri) is False:
                 continue
@@ -471,10 +500,9 @@ class Menu(OrderedTable):
 
     def get_links(self):
         base = self.get_abspath()
-        # FIXME The context is not available when updating the catalog.
-        site_root_abspath = self.parent.parent.get_abspath()
+        site_root_abspath = self.get_site_root().get_abspath()
         handler = self.handler
-        record_schema = handler.record_schema
+        record_properties = handler.record_properties
         links = []
 
         for record in handler.get_records_in_order():
@@ -489,16 +517,23 @@ class Menu(OrderedTable):
                 uri = site_root_abspath.resolve2('.%s' % path)
             else:
                 uri = base.resolve2(path)
-            links.append(str(uri))
+            try:
+                # Use the canonical path instead of the uri stocked
+                resource = self.get_resource(uri)
+                link = str(resource.get_canonical_path())
+            except LookupError:
+                # If the resource does not exist, simply use the uri
+                link = str(uri)
+            links.append(link)
             # Submenu resources
-            if not 'child' in record_schema:
+            if not 'child' in record_properties:
                 continue
             path = handler.get_record_value(record, 'child')
             if path:
                 container = self.parent
-                if container.has_resource(path):
+                child = container.get_resource(path, soft=True)
+                if child is not None:
                     # take into account the submenu if it is not empty
-                    child = container.get_resource(path)
                     if child.handler.get_n_records():
                         uri = site_root_abspath.resolve(path)
                         links.append(str(uri))
@@ -506,7 +541,7 @@ class Menu(OrderedTable):
         return links
 
 
-    def change_link(self, old_path, new_path):
+    def update_links(self, source, target):
         handler = self.handler
         base = self.get_abspath()
         # FIXME The context is not available when updating the catalog.
@@ -524,10 +559,11 @@ class Menu(OrderedTable):
             else:
                 uri = base.resolve2(path)
             uri = str(uri)
-            if uri == old_path:
+            if uri == source:
                 # Hit the old name
-                new_path2 = base.get_pathto(Path(new_path))
+                new_path2 = base.get_pathto(Path(target))
                 handler.update_record(record.id, **{'path': str(new_path2)})
+        get_context().database.change_resource(self)
 
 
 
@@ -535,10 +571,25 @@ class MenuFolder(Folder):
 
     class_id = 'menu-folder'
     class_title = MSG(u'iKaaro Menu')
+    class_views = ['view']
     __fixed_handlers__ = Folder.__fixed_handlers__ + ['menu']
-    # Your menu ressource (for overriding the record_schema and form)
+    # Your menu ressource (for overriding the record_properties and form)
     class_menu = Menu
 
+    # Views
+    view = GoToSpecificDocument(specific_document='menu',
+                                title=MSG(u'View'),
+                                access='is_allowed_to_edit')
+    # Disable all default views
+    new_resource = Folder_NewResource(access='is_admin')
+    browse_content = Folder_BrowseContent(access='is_admin')
+    rename = Folder_Rename(access='is_admin')
+    preview_content = Folder_PreviewContent(access='is_admin')
+    commit_log = DBResource_CommitLog(access='is_admin')
+    orphans = Folder_Orphans(access='is_admin')
+    thumb = Folder_Thumbnail(access='is_admin')
+
+
     @staticmethod
     def _make_resource(cls, folder, name, **kw):
         Folder._make_resource(cls, folder, name, **kw)
@@ -601,22 +652,19 @@ def get_menu_namespace(context, depth=3, show_first_child=False, flat=True,
     Activate "use_first_child" to automatically point to the first child of
     each item instead of the item itself.
     """
-
-    request = context.request
-    request_uri = str(request.request_uri)
-    site_root = context.resource.get_site_root()
-    method = context.view_name or context.resource.get_default_view_name()
-    path = context.uri.path
-    url = [seg.name for seg in path if seg.name]
-    if method:
-        url += [';%s' % method]
+    resource = context.resource
+    url = list(context.uri.path)
+    if not url or url[-1][0] != ';':
+        method = resource.get_default_view_name()
+        url.append(';%s' % method)
 
     # Get the menu
     tabs = {'items': []}
     if src is None:
         src = 'menu'
-    if site_root.has_resource(src):
-        menu = site_root.get_resource(src)
+    site_root = resource.get_site_root()
+    menu = site_root.get_resource(src, soft=True)
+    if menu is not None:
         tabs = menu.get_menu_namespace_level(context, url, depth,
                                              show_first_child)
 
@@ -631,7 +679,7 @@ def get_menu_namespace(context, depth=3, show_first_child=False, flat=True,
         while (items is not None) and exist_items:
             exist_items = False
             for item in items:
-                if item['class'] in ['active', 'in_path']:
+                if item['class'] in ['active', 'in-path']:
                     if item['items']:
                         items = exist_items = item['items']
                         if items:
@@ -643,8 +691,3 @@ def get_menu_namespace(context, depth=3, show_first_child=False, flat=True,
                         break
     return tabs
 
-#
-# Register
-#
-register_resource_class(Menu)
-register_resource_class(MenuFolder)
