@@ -19,25 +19,23 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-# Import from the Standard Library
-from datetime import datetime
-
 # Import from itools
-from itools.datatypes import Unicode
-from itools.gettext import MSG
+from itools.datatypes import Unicode, String, Integer, Boolean, DateTime
+from itools.log import log_warning
+from itools.uri import Path
 from itools.web import Resource, get_context
-from itools.xapian import CatalogAware
-from itools.xapian import TextField, KeywordField, IntegerField, BoolField
-from itools.xapian import PhraseQuery
+from itools.xapian import CatalogAware, PhraseQuery
 
 # Import from ikaaro
-from lock import Lock
 from metadata import Metadata
-from resource_views import DBResource_NewInstance, DBResource_Edit
+from registry import register_field, register_resource_class
+from resource_views import DBResource_Edit, DBResource_Backlinks
 from resource_views import DBResource_AddImage, DBResource_AddLink
-from resource_views import LoginView, LogoutView
+from resource_views import DBResource_AddMedia, LoginView, LogoutView
+from resource_views import Put_View, Delete_View
+from revisions_views import DBResource_CommitLog, DBResource_Changes
 from workflow import WorkflowAware
-
+from views_new import NewInstance
 
 
 class IResource(Resource):
@@ -84,7 +82,7 @@ class IResource(Resource):
 
 
     def get_title(self):
-        return self.name
+        return unicode(self.name)
 
 
     def get_page_title(self):
@@ -133,8 +131,23 @@ class IResource(Resource):
 
 
 
+###########################################################################
+# Database resources
+###########################################################################
+class DBResourceMetaclass(type):
+
+    def __new__(mcs, name, bases, dict):
+        cls = type.__new__(mcs, name, bases, dict)
+        if 'class_id' in dict:
+            register_resource_class(cls)
+        return cls
+
+
+
 class DBResource(CatalogAware, IResource):
 
+    __metaclass__ = DBResourceMetaclass
+
     def __init__(self, metadata):
         self.metadata = metadata
         self._handler = None
@@ -148,7 +161,7 @@ class DBResource(CatalogAware, IResource):
         cls._make_resource(cls, container.handler, name, *args, **kw)
         resource = container.get_resource(name)
         # Events, add
-        get_context().server.add_resource(resource)
+        get_context().database.add_resource(resource)
 
         return resource
 
@@ -166,7 +179,7 @@ class DBResource(CatalogAware, IResource):
         if format is None:
             format = cls.class_id
 
-        if issubclass(cls, WorkflowAware):
+        if kw.get('state') is None and issubclass(cls, WorkflowAware):
             schema = cls.get_metadata_schema()
             state = schema['state'].get_default()
             if state is None:
@@ -180,20 +193,17 @@ class DBResource(CatalogAware, IResource):
         if self._handler is None:
             cls = self.class_handler
             database = self.metadata.database
+            fs = database.fs
             if self.parent is None:
-                uri = self.metadata.uri.resolve('.')
+                key = fs.resolve(self.metadata.key, '.')
             else:
-                uri = self.metadata.uri.resolve(self.name)
-            if database.has_handler(uri):
-                self._handler = database.get_handler(uri, cls=cls)
+                key = fs.resolve(self.metadata.key, self.name)
+            if database.has_handler(key):
+                handler = database.get_handler(key, cls=cls)
             else:
                 handler = cls()
-                handler.database = database
-                handler.uri = uri
-                handler.timestamp = None
-                handler.dirty = datetime.now()
-                database.add_to_cache(uri, handler)
-                self._handler = handler
+                database.push_phantom(key, handler)
+            self._handler = handler
         return self._handler
 
     handler = property(get_handler, None, None, '')
@@ -225,110 +235,170 @@ class DBResource(CatalogAware, IResource):
 
 
     def set_property(self, name, value, language=None):
-        get_context().server.change_resource(self)
+        get_context().database.change_resource(self)
         self.metadata.set_property(name, value, language=language)
 
 
     def del_property(self, name, language=None):
-        get_context().server.change_resource(self)
+        get_context().database.change_resource(self)
         self.metadata.del_property(name, language=language)
 
 
+    ########################################################################
+    # Versioning
+    ########################################################################
+    def get_files_to_archive(self, content=False):
+        raise NotImplementedError
+
+
+    def get_revisions(self, n=None, content=False):
+        files = self.get_files_to_archive(content)
+        database = get_context().database
+        return database.get_revisions(files, n)
+
+
+    def get_last_revision(self):
+        files = self.get_files_to_archive()
+        database = get_context().database
+        return database.get_last_revision(files)
+
+
+    def get_owner(self):
+        revisions = self.get_revisions()
+        if not revisions:
+            return None
+        return revisions[-1]['username']
+
+
+    def get_last_author(self):
+        revision = self.get_last_revision()
+        return revision['username'] if revision else None
+
+
+    def get_mtime(self):
+        revision = self.get_last_revision()
+        return revision['date'] if revision else None
+
+
     ########################################################################
     # Indexing
     ########################################################################
     def to_text(self):
+        """This function must return:
+           1) An unicode text.
+            or
+           2) A dict in a multilingual context:
+              {'fr': u'....',
+               'en': u'....' ....}
+        """
         raise NotImplementedError
 
 
-    def get_catalog_fields(self):
-        return [
-            KeywordField('abspath', is_stored=True),
-            TextField('text'),
-            TextField('title', is_stored=True),
-            BoolField('is_role_aware'),
-            BoolField('is_image'),
-            KeywordField('format', is_stored=True),
-            KeywordField('workflow_state', is_stored=True),
-            KeywordField('members'),
-            # For referencial-integrity, keep links between cms resources,
-            # where a link is the physical path.
-            KeywordField('links'),
-            # Folder's view
-            KeywordField('parent_path'),
-            KeywordField('name', is_stored=True),
-            KeywordField('mtime', is_indexed=True, is_stored=True),
-            IntegerField('size', is_indexed=False, is_stored=True)]
-
-
-    def get_catalog_values(self):
+    def _get_catalog_values(self):
         from access import RoleAware
         from file import File, Image
 
+        # Values
         abspath = self.get_canonical_path()
-        mtime = self.get_mtime()
-        if mtime is None:
-            mtime = datetime.now()
-
-        # FIXME Index a single language for now
+        # Get the languages
         site_root = self.get_site_root()
         languages = site_root.get_property('website_languages')
-        language = languages[0]
 
-        document = {
-            'name': self.name,
-            'abspath': str(abspath),
-            'format': self.metadata.format,
-            'title': self.get_title(language=language),
-            'mtime': mtime.strftime('%Y%m%d%H%M%S')}
+        # Titles
+        title = {}
+        for language in languages:
+            title[language] = self.get_title(language=language)
+
+        # Descriptions
+        description = {}
+        for language in languages:
+            description[language] = self.get_property('description',
+                                                      language=language)
+
+        # Subjects (keywords)
+        subject = {}
+        for language in languages:
+            subject[language] = self.get_property('subject',
+                                                  language=language)
 
         # Full text
         context = get_context()
-        if context is not None and context.server.index_text:
+        text = None
+        try:
+            server = context.server
+        except AttributeError:
+            server = None
+        if server is not None and server.index_text:
             try:
                 text = self.to_text()
             except NotImplementedError:
                 pass
-            except:
-                server = context.server
-                server.log_error(context)
-                log = "%s failed" % self.get_abspath()
-                server.event_log.write(log)
-                server.event_log.flush()
-            else:
-                document['text'] = text
-
-        # Links
-        document['links'] = self.get_links()
+            except Exception:
+                log = 'Indexation failed: %s' % abspath
+                log_warning(log, domain='ikaaro')
 
         # Parent path
-        if str(abspath) != '/':
+        parent_path = None
+        abspath_str = str(abspath)
+        if abspath_str != '/':
             parent_path = abspath.resolve2('..')
-            document['parent_path'] = str(parent_path)
+            parent_path = str(parent_path)
 
         # Size
         if isinstance(self, File):
             # FIXME We add an arbitrary size so files will always be bigger
             # than folders. This won't work when there is a folder with more
             # than that size.
-            document['size'] = 2**30 + self.get_size()
+            size = 2**30 + self.get_size()
         else:
             names = self.get_names()
-            document['size'] = len(names)
+            size = len(names)
 
         # Workflow state
         if isinstance(self, WorkflowAware):
-            document['workflow_state'] = self.get_workflow_state()
+            workflow_state = self.get_workflow_state()
+        else:
+            workflow_state = None
 
         # Role Aware
-        if isinstance(self, RoleAware):
-            document['is_role_aware'] = True
-            document['members'] = self.get_members()
+        is_role_aware = isinstance(self, RoleAware)
+        if is_role_aware:
+            members = self.get_members()
+        else:
+            members = None
 
-        # Browse in image mode
-        document['is_image'] = isinstance(self, Image)
+        # Ok
+        return {
+            'name': self.name,
+            'abspath': abspath_str,
+            'format': self.metadata.format,
+            'title': title,
+            'subject': subject,
+            'text': text,
+            'links': self.get_links(),
+            'parent_path': parent_path,
+            # This should be defined by subclasses
+            'is_image': isinstance(self, Image),
+            'is_role_aware': is_role_aware,
+            'members': members,
+            'size': size,
+            'workflow_state': workflow_state}
 
-        return document
+
+    def get_catalog_values(self):
+        # This method is called only from icms-update-catalog.py
+        values = self._get_catalog_values()
+
+        # Get last revision
+        revision = self.get_last_revision()
+        if not revision:
+            return values
+
+        # Ok
+        root = get_context().root
+        values['last_author'] = root.get_user_title(revision['username'])
+        values['mtime'] = revision['date']
+        return values
 
 
     ########################################################################
@@ -351,46 +421,41 @@ class DBResource(CatalogAware, IResource):
         return [(self.name, new_name)]
 
 
-    def update_links(self, new_name, base_path):
-        """The resource must update its links to itself.
+    def _on_move_resource(self, source):
+        """This method updates the links from/to other resources.  It is
+        called when the resource has been moved and/or renamed.
+
+        This method is called by 'Database._before_commit', the 'source'
+        parameter is the place the resource has been moved from.
         """
-        # Check referencial-integrity
-        catalog = get_context().server.catalog
-        # The catalog is not available when updating (icms-update.py)
-        # FIXME We do not guarantee referencial-integrity when updating
-        if catalog is None:
-            return
+        # (1) Update links to other resources
+        self.update_relative_links(Path(source))
 
-        old_path = self.get_abspath()
-        new_path = base_path.resolve2(new_name)
-
-        # Get all the resources that have a link to me
-        query = PhraseQuery('links', str(old_path))
-        results = catalog.search(query).get_documents()
+        # (2) Update resources that link to me
+        database = get_context().database
+        target = self.get_canonical_path()
+        query = PhraseQuery('links', source)
+        results = database.catalog.search(query).get_documents()
         for result in results:
-            resource = self.get_resource(result.abspath)
-            resource.change_link(old_path, new_path)
+            path = result.abspath
+            path = database.resources_old2new.get(path, path)
+            resource = self.get_resource(path)
+            resource.update_links(source, target)
 
 
-    def change_link(self, old_path, new_path):
-        """The resource "old_name" has a "new_name", we must update its link
+    def update_links(self, source, target):
+        """The resource identified by 'source' is going to be moved to
+        'target'.  Update our links to it.
+
+        The parameters 'source' and 'target' are absolute 'Path' objects.
         """
-        pass
 
 
-    def get_mtime(self):
-        handlers = [self.metadata] + self.get_handlers()
-
-        mtimes = []
-        for handler in handlers:
-            if handler is not None:
-                mtime = handler.get_mtime()
-                if mtime is not None:
-                    mtimes.append(mtime)
-
-        if not mtimes:
-            return None
-        return max(mtimes)
+    def update_relative_links(self, source):
+        """Update the relative links coming out from this resource after it
+        was moved, so they are not broken. The old path is in parameter. The
+        new path is "self.get_canonical_path()".
+        """
 
 
     def get_links(self):
@@ -435,50 +500,6 @@ class DBResource(CatalogAware, IResource):
         metadata.version = version
 
 
-    ########################################################################
-    # Lock/Unlock/Put
-    ########################################################################
-    def lock(self):
-        lock = Lock(username=get_context().user.name)
-
-        self = self.get_real_resource()
-        if self.parent is None:
-            self.handler.set_handler('.lock', lock)
-        else:
-            self.parent.handler.set_handler('%s.lock' % self.name, lock)
-
-        return lock.key
-
-
-    def unlock(self):
-        self = self.get_real_resource()
-        if self.parent is None:
-            self.handler.del_handler('.lock')
-        else:
-            self.parent.handler.del_handler('%s.lock' % self.name)
-
-
-    def is_locked(self):
-        self = self.get_real_resource()
-        if self.parent is None:
-            return self.handler.has_handler('.lock')
-        return self.parent.handler.has_handler('%s.lock' % self.name)
-
-
-    def get_lock(self):
-        self = self.get_real_resource()
-        if self.parent is None:
-            return self.handler.get_handler('.lock')
-        return self.parent.handler.get_handler('%s.lock' % self.name)
-
-
-    def put(self, context):
-        # Save the data
-        body = context.get_form_value('body')
-        self.handler.load_state_from_string(body)
-        context.server.change_resource(self)
-
-
     ########################################################################
     # User interface
     ########################################################################
@@ -487,10 +508,7 @@ class DBResource(CatalogAware, IResource):
         if title:
             return title
         # Fallback to the resource's name
-        title = self.name
-        if isinstance(title, MSG):
-            return title.gettext(language)
-        return title
+        return unicode(self.name)
 
 
     def get_content_language(self, context, languages=None):
@@ -507,11 +525,65 @@ class DBResource(CatalogAware, IResource):
         return context.accept_language.select_language(languages)
 
 
+    ########################################################################
+    # Cut & Paste Resources
+    ########################################################################
+    def can_paste(self, source):
+        """Is the source resource can be pasted into myself.
+        Question is "can I handle this type of resource?"
+        """
+        raise NotImplementedError
+
+
+    def can_paste_into(self, target):
+        """Can I be pasted into the given target.
+        Question is "Is this container compatible with myself?"
+        """
+        # No restriction by default. Functional modules will want to keep
+        # their specific resources for them.
+        return True
+
+
     # Views
-    new_instance = DBResource_NewInstance()
+    new_instance = NewInstance()
     login = LoginView()
     logout = LogoutView()
     edit = DBResource_Edit()
     add_image = DBResource_AddImage()
     add_link = DBResource_AddLink()
+    add_media = DBResource_AddMedia()
+    commit_log = DBResource_CommitLog()
+    changes = DBResource_Changes()
+    backlinks = DBResource_Backlinks()
+    http_put = Put_View()
+    http_delete = Delete_View()
 
+
+
+###################################
+# Register the new catalog fields #
+###################################
+
+register_field(
+    'abspath', String(is_key_field=True, is_stored=True, is_indexed=True))
+register_field('text', Unicode(is_indexed=True))
+register_field('title', Unicode(is_stored=True, is_indexed=True))
+register_field('description', Unicode(is_indexed=True))
+register_field('subject', Unicode(is_indexed=True))
+register_field('is_role_aware', Boolean(is_indexed=True))
+register_field('is_image', Boolean(is_indexed=True))
+register_field('format', String(is_stored=True, is_indexed=True))
+register_field('workflow_state', String(is_stored=True, is_indexed=True))
+register_field('members', String(is_indexed=True, multiple=True))
+# Versioning
+register_field('mtime', DateTime(is_stored=True, is_indexed=True))
+register_field('last_author', String(is_stored=True, is_indexed=False))
+# For referencial-integrity, keep links between resources, where a link is the
+# physical path.
+register_field('links', String(is_indexed=True, multiple=True))
+# Folder's view
+register_field('parent_path', String(is_indexed=True))
+register_field('name', String(is_stored=True, is_indexed=True))
+register_field('size', Integer(is_stored=True, is_indexed=False))
+# Optimize 'Server.find_site_root'
+register_field('vhosts', String(is_indexed=True, multiple=True))
