@@ -20,27 +20,31 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from the Standard Library
+from datetime import datetime
 from operator import itemgetter
-from urllib import quote
 
 # Import from itools
-from itools.datatypes import String, Unicode
+from itools.core import merge_dicts
+from itools.datatypes import DateTime, String, Unicode
 from itools.gettext import MSG
 from itools.handlers import checkid
+from itools.http import Conflict, NotImplemented
 from itools.i18n import get_language_name
-from itools.handlers import merge_dicts
-from itools.uri import Path, get_reference
-from itools.vfs import FileName
-from itools.web import get_context, BaseView, STLForm, INFO, ERROR
+from itools.uri import Path, get_reference, get_uri_path
+from itools.fs import FileName
+from itools.web import BaseView, STLForm, INFO, ERROR
+from itools.xapian import PhraseQuery
 
 # Import from ikaaro
-from datatypes import FileDataType
+from datatypes import FileDataType, CopyCookie
+from exceptions import ConsistencyError
+from folder_views import Folder_BrowseContent
 from forms import AutoForm, title_widget, description_widget, subject_widget
-from forms import TextWidget
-import messages
+from forms import timestamp_widget
 from registry import get_resource_class
-from utils import get_parameters, reduce_string
-from views import NewInstanceForm, ContextMenu
+from utils import reduce_string
+from views import ContextMenu
+import messages
 
 
 
@@ -56,79 +60,11 @@ class EditLanguageMenu(ContextMenu):
         return [
             {'title': get_language_name(x),
              'href': context.uri.replace(content_language=x),
-             'class': 'nav_active' if (x == content_language) else None}
+             'class': 'nav-active' if (x == content_language) else None}
             for x in languages ]
 
 
 
-class AddResourceMenu(ContextMenu):
-
-    title = MSG(u'Add Resource')
-
-    def get_items(self, resource, context):
-        base = '%s/;new_resource' % context.get_link(resource)
-        document_types = resource.get_document_types()
-        return [
-            {'src': '/ui/' + cls.class_icon16,
-             'title': cls.class_title.gettext(),
-             'href': '%s?type=%s' % (base, quote(cls.class_id))}
-            for cls in document_types ]
-
-
-class DBResource_NewInstance(NewInstanceForm, AutoForm):
-
-    access = 'is_allowed_to_add'
-    query_schema = {
-        'type': String}
-    schema = {
-        'name': String,
-        'title': Unicode}
-    widgets = [
-        title_widget,
-        TextWidget('name', title=MSG(u'Name'), default='')]
-    submit_value = MSG(u'Add')
-    context_menus = [AddResourceMenu()]
-
-
-    def get_title(self, context):
-        if self.title is not None:
-            return self.title
-        type = context.get_query_value('type')
-        if not type:
-            return MSG(u'Add resource').gettext()
-        cls = get_resource_class(type)
-        class_title = cls.class_title.gettext()
-        title = MSG(u'Add $class_title')
-        return title.gettext(class_title=class_title)
-
-
-    def get_new_resource_name(self, form):
-        # If the name is not explicitly given, use the title
-        name = form['name']
-        title = form['title'].strip()
-        if name is None:
-            return title
-        return name or title
-
-
-    def action(self, resource, context, form):
-        name = form['name']
-        title = form['title']
-
-        # Create the resource
-        class_id = context.query['type']
-        cls = get_resource_class(class_id)
-        child = cls.make_resource(cls, resource, name)
-        # The metadata
-        metadata = child.metadata
-        language = resource.get_content_language(context)
-        metadata.set_property('title', title, language=language)
-
-        goto = './%s/' % name
-        return context.come_back(messages.MSG_NEW_RESOURCE, goto=goto)
-
-
-
 class DBResource_Edit(AutoForm):
 
     access = 'is_allowed_to_edit'
@@ -139,16 +75,48 @@ class DBResource_Edit(AutoForm):
     schema = {
         'title': Unicode,
         'description': Unicode,
-        'subject': Unicode}
-    widgets = [title_widget, description_widget, subject_widget]
+        'subject': Unicode,
+        'timestamp': DateTime(readonly=True)}
+    widgets = [
+        timestamp_widget, title_widget, description_widget, subject_widget]
 
 
     def get_value(self, resource, context, name, datatype):
+        if name == 'timestamp':
+            return datetime.now()
         language = resource.get_content_language(context)
         return resource.get_property(name, language=language)
 
 
+    def check_edit_conflict(self, resource, context, form):
+        context.edit_conflict = False
+
+        timestamp = form['timestamp']
+        if timestamp is None:
+            context.message = messages.MSG_EDIT_CONFLICT
+            context.edit_conflict = True
+            return
+
+        root = context.root
+        results = root.search(abspath=str(resource.get_canonical_path()))
+        brain = results.get_documents()[0]
+        mtime = brain.mtime
+        if mtime is not None and timestamp < mtime:
+            # Conflict unless we are overwriting our own work
+            last_author = resource.get_last_author()
+            if last_author != context.user.name:
+                user = root.get_user_title(last_author)
+                context.message = messages.MSG_EDIT_CONFLICT2(user=user)
+                context.edit_conflict = True
+
+
     def action(self, resource, context, form):
+        # Check edit conflict
+        self.check_edit_conflict(resource, context, form)
+        if context.edit_conflict:
+            return
+
+        # Save changes
         title = form['title']
         description = form['description']
         subject = form['subject']
@@ -161,118 +129,40 @@ class DBResource_Edit(AutoForm):
 
 
 
+class DBResource_Backlinks(Folder_BrowseContent):
+    """Backlinks are the list of resources pointing to this resource.  This
+    view answers the question "where is this resource used?" You'll see all
+    WebPages and WikiPages (for example) referencing it.  If the list is
+    empty, you can consider it is "orphan".
+    """
+
+    access = 'is_allowed_to_view'
+    title = MSG(u"Backlinks")
+    icon = 'rename.png'
+
+    search_template = None
+    search_schema = {}
+
+    # Skip AddResourceMenu
+    context_menus = []
+
+    def get_table_columns(self, resource, context):
+        cols = Folder_BrowseContent.get_table_columns(self, resource, context)
+        return [ col for col in cols if col[0] != 'checkbox' ]
+
+
+    def get_items(self, resource, context):
+        query = PhraseQuery('links', str(resource.get_canonical_path()))
+        return context.root.search(query)
+
+
+    table_actions = []
+
+
+
 ###########################################################################
 # Interface to add images from the TinyMCE editor
 ###########################################################################
-class Breadcrumb(object):
-    """Instances of this class will be used as namespaces for STL templates.
-    The built namespace contains the breadcrumb, that is to say, the path from
-    the tree root to another tree node, and the content of that node.
-    """
-
-    def __init__(self, filter_types=None, root=None, start=None,
-            icon_size=16):
-        """The 'start' must be a handler, 'filter_type' must be a handler
-        class.
-        """
-        from file import Image
-        from folder import Folder
-        from resource_ import DBResource
-
-        if filter_types is None:
-            filter_types = (DBResource,)
-
-        context = get_context()
-        here = context.resource
-        if root is None:
-            root = here.get_site_root()
-        if start is None:
-            start = root
-
-        # Get the query parameters
-        parameters = get_parameters('bc', id=None, target=None)
-        id = parameters['id']
-        # Get the target folder
-        target_path = parameters['target']
-        if target_path is None:
-            if isinstance(start, Folder):
-                target = start
-            else:
-                target = start.parent
-        else:
-            target = root.get_resource(target_path)
-        self.target_path = str(target.get_abspath())
-
-        # Resource to link
-        item = context.get_form_value('item', default='')
-        if item == '':
-            item = '.'
-        self.item = item
-
-        # The breadcrumb
-        breadcrumb = []
-        node = target
-        while node is not root.parent:
-            url = context.uri.replace(bc_target=str(root.get_pathto(node)))
-            title = node.get_title()
-            short_title = reduce_string(title, 12, 40)
-            quoted_title = short_title.replace("'", "\\'")
-            breadcrumb.insert(0, {'name': node.name,
-                                  'title': title,
-                                  'short_title': short_title,
-                                  'quoted_title': quoted_title,
-                                  'url': url})
-            node = node.parent
-        self.path = breadcrumb
-
-        # Content
-        items = []
-        self.is_submit = False
-        user = context.user
-        filters = (Folder,) + filter_types
-        for resource in target.search_resources(cls=filters):
-            ac = resource.get_access_control()
-            if not ac.is_allowed_to_view(user, resource):
-                continue
-            path = here.get_pathto(resource)
-            bc_target = str(root.get_pathto(resource))
-            url = context.uri.replace(bc_target=bc_target)
-
-            self.is_submit = True
-            # Calculate path
-            is_image = isinstance(resource, Image)
-            if is_image:
-                path_to_icon = ";thumb?width=%s&height=%s" % (icon_size,
-                                                              icon_size)
-            else:
-                path_to_icon = resource.get_resource_icon(icon_size)
-            if path:
-                path_to_resource = Path(str(path) + '/')
-                path_to_icon = path_to_resource.resolve(path_to_icon)
-            title = resource.get_title()
-            short_title = reduce_string(title, 12, 40)
-            quoted_title = short_title.replace("'", "\\'")
-            items.append({'name': resource.name,
-                          'title': title,
-                          'short_title': short_title,
-                          'quoted_title': quoted_title,
-                          'is_folder': isinstance(resource, Folder),
-                          'is_image': is_image,
-                          'is_selectable': True,
-                          'path': path,
-                          'url': url,
-                          'icon': path_to_icon,
-                          'type': type(resource),
-                          'item_type': resource.handler.get_mimetype()})
-
-        items.sort(key=itemgetter('is_folder'), reverse=True)
-        self.items = items
-
-        # Avoid general template
-        response = context.response
-        response.set_header('Content-Type', 'text/html; charset=UTF-8')
-
-
 
 class DBResource_AddBase(STLForm):
     """Base class for 'DBResource_AddImage' and 'DBResource_AddLink' (used
@@ -283,83 +173,183 @@ class DBResource_AddBase(STLForm):
 
     element_to_add = None
 
-    configuration = {}
-
     schema = {
         'target_path': String(mandatory=True),
         'target_id': String(default=None),
-        'mode': String(mandatory=True),
-      }
-
-    styles = ['/ui/bo.css',
-              '/ui/aruni/style.css']
-
-    base_scripts = ['/ui/jquery.js',
-                    '/ui/javascript.js']
-
-
-    scripts = {'wiki': ['/ui/wiki/javascript.js'],
-               'tiny_mce': ['/ui/tiny_mce/javascript.js',
-                            '/ui/tiny_mce/tiny_mce_src.js',
-                            '/ui/tiny_mce/tiny_mce_popup.js'],
-               'input': []}
-
+        'mode': String(mandatory=True)}
 
     action_upload_schema = merge_dicts(schema,
                                        file=FileDataType(mandatory=True))
 
-
-    additional_javascript = """
-          function select_element(type, value, caption) {
-            window.opener.$("#%s").val(value);
-            window.close();
-          }
-          """
+    item_classes = ()
+    folder_classes = ()
 
 
-    def get_filter_types(self):
+    def get_item_classes(self):
+        from resource_ import DBResource
+        return self.item_classes if self.item_classes else (DBResource,)
+
+
+    def get_folder_classes(self):
+        from folder import Folder
+        return self.folder_classes if self.folder_classes else (Folder,)
+
+
+    def get_configuration(self):
+        return {}
+
+
+    def get_root(self, context):
+        return context.resource.get_site_root()
+
+
+    def get_start(self, resource):
         from file import File
-        return (File,)
+        if isinstance(resource, File):
+            return resource.parent
+        return resource
+
+
+    def is_folder(self, resource):
+        bases = self.get_folder_classes()
+        return isinstance(resource, bases)
+
+
+    def is_item(self, resource):
+        bases = self.get_item_classes()
+        return isinstance(resource, bases)
+
+
+    def can_upload(self, cls):
+        bases = self.get_item_classes()
+        return issubclass(cls, bases)
 
 
     def get_namespace(self, resource, context):
-        from file import File
-        # Get some informations
-        mode = context.get_form_value('mode')
+        from file import Image
+        from folder import Folder
+
         # For the breadcrumb
-        filter_types = self.get_filter_types()
-        if isinstance(resource, File):
-            start = resource.parent
+        start = self.get_start(resource)
+
+        # Default parameter values
+        root = self.get_root(context)
+
+        # Get the query parameters
+        target_path = context.get_form_value('target')
+        # Get the target folder
+        if target_path is None:
+            if isinstance(start, Folder):
+                target = start
+            else:
+                target = start.parent
         else:
-            start = resource
-        # Construct namespace
-        namespace = self.configuration
-        namespace.update({
-            'additional_javascript': self.get_additional_javascript(context),
-            'bc': Breadcrumb(filter_types=filter_types, start=start,
-                             icon_size=48),
-            'element_to_add': self.element_to_add,
-            'target_id': context.get_form_value('target_id'),
-            'message': context.message,
-            'mode': mode,
-            'resource_action': self.get_resource_action(context),
-            'styles': self.styles,
-            'scripts': self.get_scripts(mode)})
+            target = root.get_resource(target_path)
+
+        # The breadcrumb
+        breadcrumb = []
+        node = target
+        while node is not root.parent:
+            url = context.uri.replace(target=str(root.get_pathto(node)))
+            title = node.get_title()
+            short_title = reduce_string(title, 12, 40)
+            quoted_title = short_title.replace("'", "\\'")
+            breadcrumb.insert(0, {'name': node.name,
+                                  'title': title,
+                                  'short_title': short_title,
+                                  'quoted_title': quoted_title,
+                                  'url': url})
+            node = node.parent
+
+        # Content
+        folders = []
+        items = []
+        user = context.user
+        for resource in target.search_resources():
+            is_folder = self.is_folder(resource)
+            is_item = self.is_item(resource)
+            if not is_folder and not is_item:
+                continue
+
+            ac = resource.get_access_control()
+            if not ac.is_allowed_to_view(user, resource):
+                continue
+            path = context.resource.get_pathto(resource)
+            url = context.uri.replace(target=str(root.get_pathto(resource)))
+
+            # Calculate path
+            if isinstance(resource, Image):
+                path_to_icon = ";thumb?width48=&height=48"
+            else:
+                path_to_icon = resource.get_resource_icon(48)
+            if path:
+                path_to_resource = Path(str(path) + '/')
+                path_to_icon = path_to_resource.resolve(path_to_icon)
+            title = resource.get_title()
+            short_title = reduce_string(title, 12, 40)
+            quoted_title = short_title.replace("'", "\\'")
+            item = {
+                'title': title,
+                'short_title': short_title,
+                'quoted_title': quoted_title,
+                'path': path,
+                'url': url,
+                'icon': path_to_icon,
+                'item_type': resource.handler.get_mimetype()}
+            if is_folder:
+                folders.append(item)
+            if is_item:
+                items.append(item)
+
+        # Sort
+        items.sort(key=itemgetter('short_title'))
+        folders.sort(key=itemgetter('short_title'))
+
+        # Avoid general template
+        context.content_type = 'text/html; charset=UTF-8'
+
+        # Build and return the namespace
+        namespace = self.get_configuration()
+        additional_javascript = self.get_additional_javascript(context)
+        namespace['additional_javascript'] = additional_javascript
+        namespace['target_path'] = str(target.get_abspath())
+        namespace['breadcrumb'] = breadcrumb
+        namespace['folders'] = folders
+        namespace['items'] = items
+        namespace['element_to_add'] = self.element_to_add
+        namespace['target_id'] = context.get_form_value('target_id')
+        namespace['message'] = context.message
+        namespace['mode'] = context.get_form_value('mode')
+        namespace['resource_action'] = self.get_resource_action(context)
+        namespace['scripts'] = self.get_scripts(context)
         return namespace
 
 
-    def get_scripts(self, mode):
-        if mode is None:
-            return self.base_scripts
-        return self.base_scripts + self.scripts[mode]
+    def get_scripts(self, context):
+        mode = context.get_form_value('mode')
+        if mode == 'wiki':
+            return ['/ui/wiki/javascript.js']
+        elif mode == 'tiny_mce':
+            return ['/ui/tiny_mce/javascript.js',
+                    '/ui/tiny_mce/tiny_mce_src.js',
+                    '/ui/tiny_mce/tiny_mce_popup.js']
+        return []
 
 
     def get_additional_javascript(self, context):
         mode = context.get_form_value('mode')
-        if mode!='input':
+        if mode != 'input':
             return ''
+
+        additional_javascript = """
+            function select_element(type, value, caption) {
+                window.opener.$("#%s").val(value);
+                window.close();
+            }
+            """
+
         target_id = context.get_form_value('target_id')
-        return self.additional_javascript % target_id
+        return additional_javascript % target_id
 
 
     def action_upload(self, resource, context, form):
@@ -376,22 +366,15 @@ class DBResource_AddBase(STLForm):
         container = context.root.get_resource(form['target_path'])
 
         # Check the name is free
-        if container.has_resource(name):
+        if container.get_resource(name, soft=True) is not None:
             context.message = messages.MSG_NAME_CLASH
             return
 
         # Check it is of the expected type
-        filter_types = self.get_filter_types()
         cls = get_resource_class(mimetype)
-        is_compatible = False
-        for filter_type in filter_types:
-            if issubclass(cls, filter_type):
-                is_compatible = True
-                break
-        if is_compatible is False:
-            class_ids = ', '.join([x.class_id for x in filter_types])
-            context.message = ERROR(u'The given file is none of the types '
-                                    u'$class_ids.', class_ids=class_ids)
+        if not self.can_upload(cls):
+            error = u'The given file is not of the expected type.'
+            context.message = ERROR(error)
             return
 
         # Add the image to the resource
@@ -403,18 +386,18 @@ class DBResource_AddBase(STLForm):
         # Add an action to the resource
         action = self.get_resource_action(context)
         if action:
-            path = path.resolve2('.%s' % action)
+            path = '%s%s' % (path, action)
         # Return javascript
-        mode = form['mode']
-        context.scripts.extend(self.get_scripts(mode))
+        scripts = self.get_scripts(context)
+        context.add_script(*scripts)
         return self.get_javascript_return(context, path)
 
 
     def get_javascript_return(self, context, path):
         return """
             <script type="text/javascript">
-                %s
-                select_element('%s', '%s', '');
+              %s
+              select_element('%s', '%s', '');
             </script>""" % (self.get_additional_javascript(context),
                             self.element_to_add, path)
 
@@ -426,22 +409,26 @@ class DBResource_AddBase(STLForm):
 
 class DBResource_AddImage(DBResource_AddBase):
 
+    template = '/ui/html/addimage.xml'
     element_to_add = 'image'
 
-    template = '/ui/html/addimage.xml'
 
-    configuration = {'show_browse': True,
-                     'show_upload': True}
-
-
-    def get_filter_types(self):
+    def get_item_classes(self):
         from file import Image
-        return (Image,)
+        return self.item_classes if self.item_classes else (Image,)
+
+
+    def get_configuration(self):
+        return {
+            'show_browse': True,
+            'show_external': False,
+            'show_insert': False,
+            'show_upload': True}
 
 
     def get_resource_action(self, context):
         mode = context.get_form_value('mode')
-        if mode=='tiny_mce':
+        if mode == 'tiny_mce':
             return '/;download'
         return DBResource_AddBase.get_resource_action(self, context)
 
@@ -450,16 +437,17 @@ class DBResource_AddImage(DBResource_AddBase):
 class DBResource_AddLink(DBResource_AddBase):
 
     template = '/ui/html/addlink.xml'
-
     element_to_add = 'link'
 
     action_add_resource_schema = merge_dicts(DBResource_AddImage.schema,
                                              title=String(mandatory=True))
 
-    configuration = {'show_browse': True,
-                     'show_external': True,
-                     'show_insert': True,
-                     'show_upload': True}
+    def get_configuration(self):
+        return {
+            'show_browse': True,
+            'show_external': True,
+            'show_insert': True,
+            'show_upload': True}
 
 
     def action_add_resource(self, resource, context, form):
@@ -473,7 +461,7 @@ class DBResource_AddLink(DBResource_AddBase):
         root = context.root
         container = root.get_resource(context.get_form_value('target_path'))
         # Check the name is free
-        if container.has_resource(name):
+        if container.get_resource(name, soft=True) is not None:
             context.message = messages.MSG_NAME_CLASH
             return
         # Get the type of resource to add
@@ -481,23 +469,44 @@ class DBResource_AddLink(DBResource_AddBase):
         # Create the resource
         child = cls.make_resource(cls, container, name)
         path = context.resource.get_pathto(child)
-        context.scripts.extend(self.get_scripts(mode))
+        scripts = self.get_scripts(context)
+        context.add_script(*scripts)
         return self.get_javascript_return(context, path)
 
 
     def get_page_type(self, mode):
         """Return the type of page to add corresponding to the mode
         """
-        if mode=='tiny_mce':
-            from wiki.page import WikiPage
-            return WikiPage
-        elif mode=='wiki':
-            from ikaaro.html import WebPage
+        if mode == 'tiny_mce':
+            from webpage import WebPage
             return WebPage
+        elif mode == 'wiki':
+            from wiki import WikiPage
+            return WikiPage
         else:
             raise ValueError, 'Incorrect mode %s' % mode
 
 
+
+class DBResource_AddMedia(DBResource_AddImage):
+
+    template = '/ui/html/addmedia.xml'
+    element_to_add = 'media'
+
+    def get_item_classes(self):
+        from file import Flash, Video
+        return self.item_classes if self.item_classes else (Flash, Video)
+
+
+    def get_configuration(self):
+        return {
+            'show_browse': True,
+            'show_external': True,
+            'show_insert': False,
+            'show_upload': True}
+
+
+
 ###########################################################################
 # Views / Login, Logout
 ###########################################################################
@@ -509,31 +518,25 @@ class LoginView(STLForm):
     template = '/ui/base/login.xml'
     schema = {
         'username': Unicode(mandatory=True),
-        'password': String(mandatory=True),
-    }
-
-
-    def get_namespace(self, resource, context):
-        return {
-            'username': context.get_form_value('username'),
-        }
+        'password': String(mandatory=True)}
+    meta = [('robots', 'noindex, follow', None)]
 
 
     def action(self, resource, context, form):
-        email = form['username']
+        email = form['username'].strip()
         password = form['password']
 
         # Check the user exists
         root = context.root
         user = root.get_user_from_login(email)
         if user is None:
-            message = ERROR(u'The user "$username" does not exist.',
+            message = ERROR(u'The user "{username}" does not exist.',
                             username=email)
             context.message = message
             return
 
         # Check the password is right
-        if not user.authenticate(password):
+        if not user.authenticate(password, clear=True):
             context.message = ERROR(u'The password is wrong.')
             return
 
@@ -544,19 +547,15 @@ class LoginView(STLForm):
         context.user = user
 
         # Come back
-        goto = None
-        referrer = context.request.referrer
-        if referrer:
-            if not referrer.path:
-                goto = referrer
-            else:
-                params = referrer.path[-1].params
-                if not params:
-                    goto = referrer
-                elif params[0] != 'login':
-                    goto = referrer
-        if goto is None:
+        referrer = context.get_referrer()
+        if referrer is None:
             goto = get_reference('./')
+        else:
+            path = get_uri_path(referrer)
+            if path.endswith(';login'):
+                goto = get_reference('./')
+            else:
+                goto = referrer
 
         return context.come_back(INFO(u"Welcome!"), goto)
 
@@ -576,3 +575,48 @@ class LogoutView(BaseView):
 
         message = INFO(u'You Are Now Logged out.')
         return context.come_back(message, goto='./')
+
+
+
+###########################################################################
+# Views / HTTP, WebDAV
+###########################################################################
+
+class Put_View(BaseView):
+
+    access = 'is_allowed_to_put'
+
+
+    def PUT(self, resource, context):
+        range = context.get_header('content-range')
+        if range:
+            raise NotImplemented
+
+        # Save the data
+        body = context.get_form_value('body')
+        resource.handler.load_state_from_string(body)
+        context.database.change_resource(resource)
+
+
+
+class Delete_View(BaseView):
+
+    access = 'is_allowed_to_remove'
+
+
+    def DELETE(self, resource, context):
+        name = resource.name
+        parent = resource.parent
+        try:
+            parent.del_resource(name)
+        except ConsistencyError:
+            raise Conflict
+
+        # Clean the copy cookie if needed
+        cut, paths = context.get_cookie('ikaaro_cp', datatype=CopyCookie)
+        # Clean cookie
+        if str(resource.get_abspath()) in paths:
+            context.del_cookie('ikaaro_cp')
+            paths = []
+
+
