@@ -22,20 +22,26 @@
 from copy import deepcopy
 
 # Import from itools
-from itools import get_abspath
+from itools.core import get_abspath
+from itools.datatypes import Unicode
+from itools.fs import lfs
 from itools.gettext import MSG
-from itools.handlers import File, Folder, Image
+from itools.handlers import File, Folder, Image, RODatabase
 from itools.http import NotFound
 from itools.i18n import has_language
 from itools.stl import stl
-from itools.web import get_context, BaseView, ERROR
-from itools.xml import XMLFile
+from itools.uri import Path
+from itools.web import get_context, BaseView, ERROR, INFO
+from itools.xmlfile import XMLFile
 
 # Import from ikaaro
 from resource_ import IResource
 from skins_views import LanguagesTemplate, LocationTemplate
 
 
+ui_database = RODatabase(fs=lfs)
+
+
 
 class FileGET(BaseView):
 
@@ -47,17 +53,15 @@ class FileGET(BaseView):
 
 
     def GET(self, resource, context):
-        response = context.response
-        response.set_header('Content-Type', resource.get_mimetype())
+        context.content_type = resource.get_mimetype()
         return resource.to_str()
 
 
 
 class UIFile(IResource, File):
 
-    def clone(self, cls=None, exclude=('database', 'uri', 'timestamp',
-                                       'dirty', 'parent', 'name')):
-        return File.clone(self, cls=cls, exclude=exclude)
+    database = ui_database
+    clone_exclude = File.clone_exclude | frozenset(['parent', 'name'])
 
 
     download = FileGET()
@@ -91,6 +95,7 @@ map = {
 
 class UIFolder(IResource, Folder):
 
+    database = ui_database
     class_title = MSG(u'UI')
     class_icon48 = 'icons/48x48/folder.png'
 
@@ -101,16 +106,19 @@ class UIFolder(IResource, Folder):
 
 
     def _get_resource(self, name):
-        if self.has_handler(name):
-            handler = self.get_handler(name)
-        else:
+        # Not an exact match: trigger language negotiation
+        if not self.has_handler(name):
+            name = '%s.' % name
             n = len(name)
-            names = [ x for x in self.get_handler_names() if x[:n] == name ]
-            languages = [ x.split('.')[-1] for x in names ]
-            languages = [ x for x in languages if has_language(x) ]
+            languages = []
+            for x in self.get_handler_names():
+                if x[:n] == name:
+                    language = x[n:]
+                    if has_language(language):
+                        languages.append(language)
 
             if not languages:
-                raise LookupError, 'resource "%s" not found' % name
+                return None
 
             # Get the best variant
             context = get_context()
@@ -124,16 +132,18 @@ class UIFolder(IResource, Folder):
             # (XXX we need a way to define the default)
             if language is None:
                 language = languages[0]
-            handler = self.get_handler('%s.%s' % (name, language))
+            name = '%s%s' % (name, language)
 
-        if isinstance(handler, Folder):
-            handler = UIFolder(handler.uri)
+        # Find out the class to use
+        key = lfs.resolve2(self.key, name)
+        if lfs.is_folder(key):
+            cls = UIFolder
         else:
-            format = handler.get_mimetype()
+            format = lfs.get_mimetype(key)
             cls = map.get(format, UIFile)
-            handler = cls(handler.uri)
-        handler.database = self.database
-        return handler
+
+        # Ok
+        return self.database.get_handler(key, cls)
 
 
 
@@ -155,61 +165,79 @@ class Skin(UIFolder):
         """Return the title to give to the template document.
         """
         here = context.resource
-        # In the Root
         root = here.get_site_root()
+        root_title = root.get_title()
+
+        # Choose the template
         if root is here:
-            return root.get_title()
-        # Somewhere else
-        message = MSG(u"${root_title}: ${here_title}")
-        return message.gettext(root_title=root.get_title(),
-                               here_title=here.get_title())
+            template = MSG(u"{root_title} - {view_title}")
+            here_title = None
+        else:
+            template = MSG(u"{root_title} - {here_title} - {view_title}")
+            here_title = here.get_title()
+
+        # The view
+        view_title = context.view.get_title(context)
+        if type(view_title) is MSG:
+            view_title = view_title.gettext()
+
+        # Ok
+        return template.gettext(root_title=root_title, here_title=here_title,
+                                view_title=view_title)
 
 
     def get_styles(self, context):
+        # Generic
         styles = [
-            # BackOffice style
             '/ui/bo.css',
-            # Calendar JS Widget (http://dynarch.com/mishoo/calendar.epl)
-            '/ui/calendar/calendar-aruni.css',
-            # Calendar
-            '/ui/ical/calendar.css',
-            # Table
-            '/ui/table/style.css']
+            '/ui/js_calendar/calendar-aruni.css']
 
-        # This skin's style
+        # Skin
         if self.has_handler('style.css'):
-            styles.append('%s/style.css' % self.get_abspath())
-        # Dynamic styles
-        for style in context.styles:
-            styles.append(style)
+            styles.append('%s/style.css' % self.get_canonical_path())
 
+        # View
+        get_styles = getattr(context.view, 'get_styles', None)
+        if get_styles is None:
+            extra = getattr(context.view, 'styles', [])
+        else:
+            extra = get_styles(context)
+        styles.extend(extra)
+
+        # Ok
         return styles
 
 
     def get_scripts(self, context):
-        scripts = []
-        # Aruni (default skin)
-        scripts.append('/ui/jquery.js')
-        scripts.append('/ui/javascript.js')
-        # Calendar (http://dynarch.com/mishoo/calendar.epl)
-        scripts.append('/ui/calendar/calendar.js')
+        scripts = [
+            '/ui/jquery.js',
+            '/ui/javascript.js']
+
+        # Calendar (http://code.google.com/p/dyndatetime/)
+        scripts.append('/ui/js_calendar/jquery.dynDateTime.pack.js')
         languages = [
             'af', 'al', 'bg', 'br', 'ca', 'da', 'de', 'du', 'el', 'en', 'es',
             'fi', 'fr', 'hr', 'hu', 'it', 'jp', 'ko', 'lt', 'lv', 'nl', 'no',
             'pl', 'pt', 'ro', 'ru', 'si', 'sk', 'sp', 'sv', 'tr', 'zh']
         accept = context.accept_language
         language = accept.select_language(languages)
-        scripts.append('/ui/calendar/lang/calendar-%s.js' % language)
-        scripts.append('/ui/calendar/calendar-setup.js')
-        # Table
-        scripts.append('/ui/table/javascript.js')
+        if language is None:
+            language = 'en'
+        scripts.append('/ui/js_calendar/lang/calendar-%s.js' % language)
+
         # This skin's JavaScript
         if self.has_handler('javascript.js'):
-            scripts.append('%s/javascript.js' % self.get_abspath())
-        # Dynamic scripts
-        for script in context.scripts:
-            scripts.append(script)
+            scripts.append('%s/javascript.js' % self.get_canonical_path())
 
+        # View
+        get_scripts = getattr(context.view, 'get_scripts', None)
+        if get_scripts is None:
+            extra = getattr(context.view, 'scripts', [])
+        else:
+            extra = get_scripts(context)
+        scripts.extend(extra)
+
+        # Ok
         return scripts
 
 
@@ -222,19 +250,45 @@ class Skin(UIFolder):
 
         meta = []
         # Set description
-        value, language = here.get_property_and_language('description')
-        if value:
-            meta.append({'name': 'description', 'lang': language,
-                         'content': value})
+        try:
+            value, language = here.get_property_and_language('description')
+        except ValueError:
+            pass
+        else:
+            if value:
+                meta.append({'name': 'description', 'lang': language,
+                             'content': value})
+
         # Set keywords for all languages
         for language in root.get_property('website_languages'):
-            value = here.get_property('subject', language)
+            try:
+                value = here.get_property('subject', language)
+            except ValueError:
+                continue
             if value is None:
                 continue
             value = value.strip()
             if value:
                 meta.append({'name': 'keywords', 'lang': language,
                              'content': value})
+
+        # Search engine optimization
+        for key, meta_name in [
+            ('google-site-verification', 'google-site-verification'),
+            ('yahoo_site_verification', 'y_key'),
+            ('bing_site_verification', 'msvalidate.01')]:
+            verification_key = root.get_property(key)
+            if verification_key:
+                meta.append({'name': meta_name,
+                             'lang': None,
+                             'content': verification_key})
+
+        # View
+        # meta are defined as a tuple (name, content, language)
+        extra_meta = getattr(context.view, 'meta', [])
+        for (name, content, lang) in extra_meta:
+            meta.append({'name': name, 'content': content, 'lang': lang})
+
         return meta
 
 
@@ -280,9 +334,13 @@ class Skin(UIFolder):
         # Text
         if context.message is not None:
             messages = context.message
+        elif 'error' in context.uri.query:
+            messages = ERROR(context.get_query_value('error', type=Unicode))
+        elif 'info' in context.uri.query:
+            messages = INFO(context.get_query_value('info', type=Unicode))
+        # XXX For backwards compatibility
         elif 'message' in context.uri.query:
-            # TODO Do not use this anymore
-            messages = context.get_query_value('message')
+            messages = INFO(context.get_query_value('message', type=Unicode))
         else:
             return None
 
@@ -321,8 +379,8 @@ class Skin(UIFolder):
 
         # The favicon.ico
         site_root = context.site_root
-        if site_root.has_resource('favicon'):
-            resource = site_root.get_resource('favicon')
+        resource = site_root.get_resource('favicon', soft=True)
+        if resource:
             favicon_href = '/favicon/;download'
             favicon_type = resource.metadata.format
         else:
@@ -346,11 +404,15 @@ class Skin(UIFolder):
         else:
             base_path = context.get_link(here)
 
+        # The view
+        view = context.view
+
         return {
             # HTML head
             'language': language,
             'title': self.get_template_title(context),
             'base_uri': str(uri),
+            'canonical_uri': view.get_canonical_uri(context),
             'styles': self.get_styles(context),
             'scripts': self.get_scripts(context),
             'meta_tags': self.get_meta_tags(context),
@@ -373,11 +435,12 @@ class Skin(UIFolder):
 
 
     def get_template(self):
-        try:
-            return self.get_resource('template.xhtml')
-        except LookupError:
-            # Default, aruni
-            return self.get_resource('/ui/aruni/template.xhtml')
+        template = self.get_resource('template.xhtml', soft=True)
+        if template is not None:
+            return template
+
+        # Default: aruni
+        return self.get_resource('/ui/aruni/template.xhtml')
 
 
     def template(self, content):
@@ -387,7 +450,7 @@ class Skin(UIFolder):
         namespace['body'] = content
 
         # Set the encoding to UTF-8
-        context.response.set_header('Content-Type', 'text/html; charset=UTF-8')
+        context.content_type = 'text/html; charset=UTF-8'
 
         # Load the template
         handler = self.get_template()
@@ -425,7 +488,9 @@ class UI(UIFolder):
     def _get_resource(self, name):
         if name in skin_registry:
             skin = skin_registry[name]
-            skin.database = self.database
             return skin
         return UIFolder._get_resource(self, name)
 
+
+    def get_canonical_path(self):
+        return Path('/ui')
