@@ -22,7 +22,7 @@
 from itools.csv import UniqueError, Property, is_multilingual
 from itools.datatypes import Integer, Enumerate, Tokens
 from itools.gettext import MSG
-from itools.web import INFO, ERROR, BaseView
+from itools.web import INFO, ERROR, BaseView, FormError
 from itools.xapian import PhraseQuery
 
 # Import from ikaaro
@@ -51,7 +51,7 @@ class Table_View(SearchForm):
 
 
     def get_search_schema(self, resource, context):
-        return resource.handler.record_schema
+        return resource.handler.record_properties
 
 
     def get_items(self, resource, context):
@@ -85,13 +85,16 @@ class Table_View(SearchForm):
         sort_by = context.query['sort_by']
         reverse = context.query['reverse']
         if sort_by:
-            items.sort(cmp=lambda x,y: cmp(getattr(x, sort_by),
-                       getattr(y, sort_by)), reverse=reverse)
+            get_record_value = resource.handler.get_record_value
+            items.sort(key=lambda x: get_record_value(x, sort_by),
+                       reverse=reverse)
 
         # Batch
         start = context.query['batch_start']
         size = context.query['batch_size']
-        return items[start:start+size]
+        if size > 0:
+            return items[start:start+size]
+        return items[start:]
 
 
     def get_table_columns(self, resource, context):
@@ -132,10 +135,8 @@ class Table_View(SearchForm):
                 value.sort()
             value_length = len(value)
             if value_length > 0:
-                rmultiple = value_length > 1
                 value = value[0]
             else:
-                rmultiple = False
                 value = None
 
         # Enumerate
@@ -143,8 +144,6 @@ class Table_View(SearchForm):
         if is_enumerate:
             value = datatype.get_value(value)
 
-        if is_multiple or is_tokens:
-            return value, rmultiple
         return value
 
 
@@ -159,7 +158,7 @@ class Table_View(SearchForm):
         for id in ids:
             resource.handler.del_record(id)
         # Reindex the resource
-        context.server.change_resource(resource)
+        context.database.change_resource(resource)
 
         context.message = INFO(u'Record deleted.')
 
@@ -182,6 +181,16 @@ class Table_AddEditRecord(AutoForm):
         return resource.get_form()
 
 
+    def get_field_title(self, resource, name):
+        for widget in resource.get_form():
+            if widget.name == name:
+                title = getattr(widget, 'title', None)
+                if title:
+                    return title.gettext()
+                return name
+        return name
+
+
     def action(self, resource, context, form):
         """Code shared by the add & edit actions.  It builds a new record
         from the form.
@@ -208,10 +217,10 @@ class Table_AddEditRecord(AutoForm):
         try:
             self.action_add_or_edit(resource, context, record)
         except UniqueError, error:
-            title = resource.get_field_title(error.name)
-            context.message = ERROR(error, field=title, value=error.value)
+            title = self.get_field_title(resource, error.name)
+            context.message = ERROR(str(error), field=title, value=error.value)
         except ValueError, error:
-            message = ERROR(u'Error: $msg', msg=str(error))
+            message = ERROR(u'Error: {msg}', msg=str(error))
             context.message = message
         else:
             return self.action_on_success(resource, context)
@@ -228,7 +237,7 @@ class Table_AddRecord(Table_AddEditRecord):
     def action_add_or_edit(self, resource, context, record):
         resource.handler.add_record(record)
         # Reindex the resource
-        context.server.change_resource(resource)
+        context.database.change_resource(resource)
 
 
     def action_on_success(self, resource, context):
@@ -240,10 +249,24 @@ class Table_AddRecord(Table_AddEditRecord):
 
 class Table_EditRecord(Table_AddEditRecord):
 
-    title = MSG(u'Edit record ${id}')
+    title = MSG(u'Edit record {id}')
     query_schema = {'id': Integer(mandatory=True)}
 
 
+    def get_query(self, context):
+        query = Table_AddEditRecord.get_query(self, context)
+        # Test the id is valid
+        id = query['id']
+        resource = context.resource
+        handler = resource.get_handler()
+        record = handler.get_record(id)
+        if record is None:
+            context.query = query
+            raise FormError, MSG(u'The {id} record is missing.', id=id)
+        # Ok
+        return query
+
+
     def get_value(self, resource, context, name, datatype):
         handler = resource.get_handler()
         # Get the record
@@ -266,7 +289,7 @@ class Table_EditRecord(Table_AddEditRecord):
         id = context.query['id']
         resource.handler.update_record(id, **record)
         # Reindex the resource
-        context.server.change_resource(resource)
+        context.database.change_resource(resource)
 
 
     def action_on_success(self, resource, context):
@@ -274,9 +297,9 @@ class Table_EditRecord(Table_AddEditRecord):
 
 
 
-###########################################################################
-# Oredered Views
-###########################################################################
+##########################################################################
+# Ordered Views
+##########################################################################
 class OrderedTable_View(Table_View):
 
     def get_items(self, resource, context):
@@ -319,15 +342,13 @@ class OrderedTable_View(Table_View):
     table_actions = [RemoveButton, OrderUpButton, OrderDownButton,
                      OrderTopButton, OrderBottomButton]
 
-    #######################################################################
+    ######################################################################
     # Form Actions
-    #######################################################################
+    ######################################################################
     def action_remove(self, resource, context, form):
         ids = form['ids']
         for id in ids:
             resource.del_record(id)
-        # Reindex the resource
-        context.server.change_resource(resource)
 
         context.message = INFO(u'Record deleted.')
 
@@ -388,7 +409,7 @@ class Table_ExportCSV(BaseView):
 
 
     def get_mtime(self, resource):
-        return resource.get_mtime()
+        return resource.handler.get_mtime()
 
 
     def GET(self, resource, context):
@@ -398,11 +419,8 @@ class Table_ExportCSV(BaseView):
         csv = handler.to_csv(columns, separator=self.multiple_separator,
                              language=language)
 
-        response = context.response
-        # Filename
-        filename = "%s.csv" % resource.name
-        response.set_header('Content-Disposition',
-                            'inline; filename="%s"' % filename)
-        # Content-Type
-        response.set_header('Content-Type', 'text/comma-separated-values')
+        # Ok
+        context.set_content_type('text/comma-separated-values')
+        context.set_content_disposition('inline', '%s.csv' % resource.name)
         return csv.to_str(separator=self.csv_separator)
+
