@@ -18,9 +18,12 @@
 from operator import itemgetter
 
 # Import from itools
-from itools.datatypes import Enumerate, String, Integer, Boolean
+from itools.datatypes import Enumerate, String
 from itools.web import get_context
 
+# Import from ikaaro
+from ikaaro.datatypes import FileDataType
+
 
 
 class TrackerList(Enumerate):
@@ -106,38 +109,49 @@ class ProductInfoList(Enumerate):
 
 
 class UsersList(Enumerate):
+
+    excluded_roles = None
+
     @classmethod
     def get_options(cls):
         site_root = cls.tracker.get_site_root()
+        # Members
+        excluded_roles = cls.excluded_roles
+        if excluded_roles:
+            members = set()
+            for rolename in site_root.get_role_names():
+                if rolename in excluded_roles:
+                    continue
+                usernames = site_root.get_property(rolename)
+                members = members.union(usernames)
+        else:
+            members = site_root.get_members()
+
         users = site_root.get_resource('/users')
-        options = [{'name': x,
-                    'value': users.get_resource(x).get_title()}
-                    for x in site_root.get_members()]
+        options = []
+        for name in members:
+            user = users.get_resource(name, soft=True)
+            if user is None:
+                continue
+            value = user.get_title()
+            options.append({'name': name, 'value': value})
         options.sort(key=itemgetter('value'))
         return options
 
 
-def get_issue_fields(resource_tracker):
-    return {'title': String(mandatory=True),
-            'product': TrackerList(element='product',
-                                   tracker=resource_tracker, mandatory=True),
-            'module': ProductInfoList(element='module',
-                                      tracker=resource_tracker),
-            'version': ProductInfoList(element='version',
-                                       tracker=resource_tracker),
-            'type': TrackerList(element='type', tracker=resource_tracker,
-                                mandatory=True),
-            'state': TrackerList(element='state', tracker=resource_tracker,
-                                 mandatory=True),
-            'priority': TrackerList(element='priority',
-                                    tracker=resource_tracker),
-            'assigned_to': UsersList(tracker=resource_tracker),
-            'cc_list': UsersList(tracker=resource_tracker, multiple=True),
-            'comment': String,
-            'file': String}
-
-
-
-
 
+def get_issue_fields(tracker):
+    return {
+        'title': String(mandatory=True),
+        'product': TrackerList(element='product', tracker=tracker,
+                               mandatory=True),
+        'module': ProductInfoList(element='module', tracker=tracker),
+        'version': ProductInfoList(element='version', tracker=tracker),
+        'type': TrackerList(element='type', tracker=tracker, mandatory=True),
+        'state': TrackerList(element='state', tracker=tracker, mandatory=True),
+        'priority': TrackerList(element='priority', tracker=tracker),
+        'assigned_to': UsersList(tracker=tracker, excluded_roles=('guests',)),
+        'cc_list': UsersList(tracker=tracker, multiple=True),
+        'comment': String,
+        'file': FileDataType}
 
