@@ -22,21 +22,20 @@
 
 # Import from the Standard Library
 from datetime import datetime
-from string import Template
 
 # Import from itools
 from itools.csv import Table
 from itools.datatypes import DateTime, Integer, String, Unicode, Tokens
 from itools.gettext import MSG
 from itools.handlers import checkid
-from itools.vfs import FileName
-from itools.xapian import IntegerField, KeywordField
+from itools.fs import FileName
+from itools.uri import Path
 from itools.web import get_context
 
 # Import from ikaaro
 from ikaaro.file import File
 from ikaaro.folder import Folder
-from ikaaro.registry import register_resource_class, get_resource_class
+from ikaaro.registry import get_resource_class, register_field
 from ikaaro.utils import generate_name
 from issue_views import Issue_Edit, Issue_EditResources, Issue_History
 from issue_views import IssueTrackerMenu
@@ -44,7 +43,7 @@ from issue_views import IssueTrackerMenu
 
 class History(Table):
 
-    record_schema = {
+    record_properties = {
         'datetime': DateTime,
         'username': String,
         'title': Unicode,
@@ -56,7 +55,7 @@ class History(Table):
         'priority': Integer,
         'assigned_to': String,
         'comment': Unicode,
-        'cc_list': Tokens(),
+        'cc_list': Tokens,
         'file': String}
 
 
@@ -76,29 +75,21 @@ class Issue(Folder):
         folder.set_handler('%s/.history' % name, History())
 
 
-    def get_catalog_fields(self):
-        fields = Folder.get_catalog_fields(self)
-        # Metadata
-        names = [
-            'id', 'product', 'module', 'version', 'type', 'state', 'priority']
-        for name in names:
-            field = IntegerField(name, is_stored=True)
-            fields.append(field)
-        # Assign To
-        field = KeywordField('assigned_to', is_stored=True)
-        fields.append(field)
-        # Ok
-        return fields
-
-
-    def get_catalog_values(self):
-        document = Folder.get_catalog_values(self)
+    def _get_catalog_values(self):
+        document = Folder._get_catalog_values(self)
         document['id'] = int(self.name)
-        names = 'product', 'module', 'version', 'type', 'priority', 'state'
-        for name in names:
-            document[name] = self.get_value(name)
-        document['assigned_to'] = self.get_value('assigned_to') or 'nobody'
-        document['title'] = self.get_value('title')
+
+        # Get the last record
+        history = self.get_history()
+        record = history.get_record(-1)
+        if record:
+            get_record_value = history.get_record_value
+            names = 'product', 'module', 'version', 'type', 'priority', 'state'
+            for name in names:
+                document[name] = get_record_value(record, name)
+            assigned_to = get_record_value(record, 'assigned_to') or 'nobody'
+            document['assigned_to'] = assigned_to
+            document['title'] = get_record_value(record, 'title')
         return document
 
 
@@ -106,6 +97,13 @@ class Issue(Folder):
         return [File]
 
 
+    def get_files_to_archive(self, content=False):
+        files = Folder.get_files_to_archive(self, content)
+        history = self.get_history().key
+        files.append(history)
+        return files
+
+
     def get_mtime(self):
         """Return the datetime of the last record.
         """
@@ -120,31 +118,40 @@ class Issue(Folder):
         base = self.get_abspath()
 
         links = []
-        for record in self.get_history_records():
-            filename = record.file
+        history = self.get_history()
+        for record in history.get_records():
+            filename = history.get_record_value(record, 'file')
             if filename:
                 links.append(str(base.resolve2(filename)))
         return links
 
 
-    def change_link(self, old_path, new_path):
+    def update_links(self, source, target):
         base = self.get_abspath()
-        old_name = base.get_pathto(old_path)
+        resources_new2old = get_context().database.resources_new2old
+        base = str(base)
+        old_base = resources_new2old.get(base, base)
+        old_base = Path(old_base)
+        new_base = Path(base)
+
         history = self.get_history()
         for record in history.get_records():
-            file = history.get_record_value(record, 'file')
-            if file == old_name:
-                value = str(base.get_pathto(new_path))
+            filename = history.get_record_value(record, 'file')
+            if not filename:
+                continue
+            path = old_base.resolve2(filename)
+            if path == source:
+                value = str(new_base.get_pathto(target))
                 history.update_record(record.id, **{'file': value})
-        # Reindex
-        get_context().server.change_resource(self)
+
+        get_context().database.change_resource(self)
 
 
     #######################################################################
     # API
     #######################################################################
     def get_title(self, language=None):
-        return '#%s %s' % (self.name, self.get_value('title'))
+        return u'#%s %s' % (self.name, self.get_value('title'))
 
 
     def get_calendar(self):
@@ -192,7 +199,7 @@ class Issue(Folder):
         # Version, Priority, etc.
         for name in ['product', 'module', 'version', 'type', 'state',
                      'priority', 'assigned_to', 'comment']:
-            type = History.record_schema[name]
+            type = History.record_properties[name]
             value = context.get_form_value(name, type=type)
             if type == Unicode:
                 value = value.strip()
@@ -200,10 +207,11 @@ class Issue(Folder):
         # CCs
         cc_list = set(self.get_value('cc_list') or ())
         cc_remove = context.get_form_value('cc_remove')
+        datatype = String(multiple=True)
         if cc_remove:
-            cc_remove = context.get_form_values('cc_list')
+            cc_remove = context.get_form_value('cc_list', type=datatype)
             cc_list = cc_list.difference(cc_remove)
-        cc_add = context.get_form_values('cc_add')
+        cc_add = context.get_form_value('cc_add', type=datatype)
         if cc_add:
             cc_list = cc_list.union(cc_add)
         record['cc_list'] = list(cc_list)
@@ -251,18 +259,22 @@ class Issue(Folder):
         tracker_title = self.parent.get_property('title') or 'Tracker Issue'
         subject = '[%s #%s] %s' % (tracker_title, self.name, title)
         # Notify / Body
-        if context.resource.class_id == 'tracker':
-            uri = context.uri.resolve('%s/;edit' % self.name)
-        else:
+        if isinstance(context.resource, self.__class__):
             uri = context.uri.resolve(';edit')
-        body = '#%s %s %s\n\n' % (self.name, self.get_value('title'),
-                                  str(uri))
-        message = MSG(u'The user $title did some changes.')
+        else:
+            uri = context.uri.resolve('%s/;edit' % self.name)
+
+        message = MSG(u'DO NOT REPLY TO THIS EMAIL. To comment on this '
+                u'issue, please visit:\n{issue_uri}')
+        body = message.gettext(issue_uri=uri)
+        body += '\n\n'
+        body += '#%s %s\n\n' % (self.name, self.get_value('title'))
+        message = MSG(u'The user {title} did some changes.')
         body +=  message.gettext(title=user_title)
         body += '\n\n'
         if file:
             filename = unicode(filename, 'utf-8')
-            message = MSG(u'New Attachment: $filename')
+            message = MSG(u'New Attachment: {filename}')
             message = message.gettext(filename=filename)
             body += message + '\n'
         comment = context.get_form_value('comment', type=Unicode)
@@ -272,10 +284,9 @@ class Issue(Folder):
         if comment:
             title = MSG(u'Comment').gettext()
             separator = len(title) * u'-'
-            template = u'${title}\n${separator}\n\n${comment}\n'
-            template = Template(template)
-            body += template.substitute(title=title, separator=separator,
-                                        comment=comment)
+            template = u'{title}\n{separator}\n\n{comment}\n'
+            body += template.format(title=title, separator=separator,
+                                    comment=comment)
         # Notify / Send
         for to_addr in to_addrs:
             user = root.get_user(to_addr)
@@ -293,11 +304,11 @@ class Issue(Folder):
         history = self.get_history()
         if history.get_n_records() > 0:
             # Edit issue
-            template = MSG(u'$field: $old_value to $new_value')
+            template = MSG(u'{field}: {old_value} to {new_value}')
             empty = MSG(u'[empty]').gettext()
         else:
             # New issue
-            template = MSG(u'$field: $old_value$new_value')
+            template = MSG(u'{field}: {old_value}{new_value}')
             empty = u''
         # Modification of title
         last_title = self.get_value('title') or empty
@@ -325,7 +336,10 @@ class Issue(Folder):
             csv = self.parent.get_resource(name).handler
             if last_value or last_value == 0:
                 rec = csv.get_record(last_value)
-                last_title = csv.get_record_value(rec, 'title')
+                if rec is None:
+                    last_title = 'undefined'
+                else:
+                    last_title = csv.get_record_value(rec, 'title')
             if new_value or new_value == 0:
                 rec = csv.get_record(new_value)
                 new_title = csv.get_record_value(rec, 'title')
@@ -420,4 +434,8 @@ class Issue(Folder):
 ###########################################################################
 # Register
 ###########################################################################
-register_resource_class(Issue)
+for name in ['id', 'product', 'module', 'version', 'type', 'state',
+             'priority']:
+    register_field(name, Integer(is_stored=True, is_indexed=True))
+register_field('assigned_to', String(is_stored=True, is_indexed=True))
+
