@@ -26,8 +26,7 @@ from re import compile
 from textwrap import TextWrapper
 
 # Import from itools
-from itools.datatypes import Boolean, Date, Integer, String, Unicode
-from itools.datatypes import XMLContent
+from itools.datatypes import Date, Integer, String, Unicode, XMLContent
 from itools.gettext import MSG
 from itools.ical import Time
 from itools.html import xhtml_uri
@@ -132,6 +131,8 @@ class Issue_Edit(STLForm):
     title = MSG(u'Edit Issue')
     icon = 'edit.png'
     template = '/ui/tracker/edit_issue.xml'
+    styles = ['/ui/tracker/style.css']
+    scripts = ['/ui/tracker/tracker.js']
 
 
     def get_schema(self, resource, context):
@@ -145,18 +146,13 @@ class Issue_Edit(STLForm):
 
 
     def get_namespace(self, resource, context):
-        # Set Style & JS
-        context.styles.append('/ui/tracker/tracker.css')
-        context.scripts.append('/ui/tracker/tracker.js')
+        namespace = STLForm.get_namespace(self, resource, context)
 
         # Local variables
         users = resource.get_resource('/users')
         history = resource.get_history()
         record = history.get_record(-1)
 
-        # Build the namespace
-        namespace = self.build_namespace(resource, context)
-
         # Comments
         comments = []
         i = 0
@@ -168,9 +164,8 @@ class Issue_Edit(STLForm):
             rdatetime = record.datetime
             # solid in case the user has been removed
             username = record.username
-            user_title = username
-            if users.has_resource(username):
-                user_title = users.get_resource(username).get_title()
+            user = users.get_resource(username, soft=True)
+            user_title = user and user.get_title() or username
             i += 1
             comments.append({
                 'number': i,
@@ -191,7 +186,9 @@ class Issue_Edit(STLForm):
                               'class': None}
         cc_value = namespace['cc_list']['value']
         add_value = namespace['cc_add']['value']
-        for user in UsersList(tracker=resource.parent).get_options():
+
+        cc_list_userslist = self.get_schema(resource, context)['cc_list']
+        for user in cc_list_userslist.get_options():
             user['selected'] = False
             if user['name'] in cc_list:
                 cc_value.append(user)
@@ -214,7 +211,7 @@ class Issue_Edit(STLForm):
         # Edit
         resource._add_record(context, form)
         # Change
-        context.server.change_resource(resource)
+        context.database.change_resource(resource)
         context.message = MSG_CHANGES_SAVED
 
 
@@ -225,12 +222,10 @@ class Issue_History(STLView):
     title = MSG(u'History')
     icon = 'history.png'
     template = '/ui/tracker/issue_history.xml'
+    styles = ['/ui/tracker/style.css']
 
 
     def get_namespace(self, resource, context):
-        # Set Style
-        context.styles.append('/ui/tracker/tracker.css')
-
         # Local variables
         users = resource.get_resource('/users')
         tracker = resource.parent
@@ -266,9 +261,8 @@ class Issue_History(STLView):
             cc_list = record.get_value('cc_list') or ()
             file = record.get_value('file')
             # Solid in case the user has been removed
-            user_exist = users.has_resource(username)
-            usertitle = (user_exist and
-                         users.get_resource(username).get_title() or username)
+            user = users.get_resource(username, soft=True)
+            usertitle = user and user.get_title() or username
             comment = XMLContent.encode(Unicode.encode(comment))
             comment = XMLParser(comment.replace('\n', '<br />'))
             i += 1
@@ -292,7 +286,7 @@ class Issue_History(STLView):
             if version != previous_version:
                 previous_version = version
                 row_ns['version'] = ' '
-                if module is not None:
+                if version is not None:
                     version = versions.get_record(int(version))
                     if version:
                         value = versions.get_record_value(version, 'title')
@@ -331,11 +325,11 @@ class Issue_History(STLView):
                         row_ns['priority'] = value
             if assigned_to != previous_assigned_to:
                 previous_assigned_to = assigned_to
-                if assigned_to and users.has_resource(assigned_to):
-                    assigned_to_user = users.get_resource(assigned_to)
-                    row_ns['assigned_to'] = assigned_to_user.get_title()
-                else:
-                    row_ns['assigned_to'] = ' '
+                row_ns['assigned_to'] = ' '
+                if assigned_to:
+                    assigned_to_user = users.get_resource(assigned_to, soft=True)
+                    if assigned_to_user is not None:
+                        row_ns['assigned_to'] = assigned_to_user.get_title()
             if cc_list != previous_cc_list:
                 root = context.root
                 previous_cc_list = cc_list
@@ -363,7 +357,7 @@ class Issue_ViewResources(Table_View):
     search_template = None
 
     batch_msg1 = MSG(u"There is 1 assignment.")
-    batch_msg2 = MSG(u"There are ${n} assignments.")
+    batch_msg2 = MSG(u"There are {n} assignments.")
 
 
     def get_items(self, resource, context):
@@ -441,7 +435,8 @@ class Issue_AddEditResource(STLForm):
         return {
             'action': action,
             'id': id,
-            'users': resource.parent.get_members_namespace(user),
+            'users': UsersList(tracker=resource,
+               excluded_roles=('guests',)).get_namespace(user),
             'd_start': d_start.strftime('%Y-%m-%d'),
             't_start': t_start,
             'd_end': d_end.strftime('%Y-%m-%d'),
