@@ -25,7 +25,6 @@ from itools.gettext import MSG
 from itools.handlers import ConfigFile
 
 # Import from ikaaro
-from ikaaro.registry import register_resource_class
 from ikaaro.text import Text
 
 
@@ -50,7 +49,6 @@ class StoredSearchFile(ConfigFile):
 class StoredSearch(Text):
 
     class_id = 'stored_search'
-    class_version = '20071215'
     class_title = MSG(u'Stored Search')
     class_handler = StoredSearchFile
 
@@ -64,8 +62,3 @@ class StoredSearch(Text):
             value = ' '.join(value)
         self.handler.set_value(name, value)
 
-
-
-
-# Register
-register_resource_class(StoredSearch)
