@@ -19,18 +19,17 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from itools
-from itools.datatypes import Boolean, Integer, String, Unicode
+from itools.core import merge_dicts
+from itools.datatypes import Boolean, String, Unicode
 from itools.datatypes import Enumerate
 from itools.gettext import MSG
-from itools.handlers import merge_dicts
 from itools.xapian import PhraseQuery, AndQuery
-from itools.web import ERROR, INFO
 
 # Import from ikaaro
 from ikaaro.forms import title_widget, BooleanCheckBox, SelectWidget
-from ikaaro.registry import register_resource_class
+from ikaaro.forms import ReadOnlyWidget
 from ikaaro.table import OrderedTable, OrderedTableFile
-from ikaaro.table_views import OrderedTable_View
+from ikaaro.table_views import OrderedTable_View, Table_EditRecord
 
 
 
@@ -50,10 +49,16 @@ class ProductsEnumerate(Enumerate):
 ###########################################################################
 class SelectTable_View(OrderedTable_View):
 
+    access = 'is_allowed_to_view'
+
     def get_table_columns(self, resource, context):
         cls = OrderedTable_View
         columns = cls.get_table_columns(self, resource, context)
         columns.append(('issues', MSG(u'Issues')))
+        if resource.name == 'product':
+            # Add specific columns for the product table
+            columns.append(('modules', MSG(u'Modules')))
+            columns.append(('versions', MSG(u'Versions')))
         return columns
 
 
@@ -73,19 +78,43 @@ class SelectTable_View(OrderedTable_View):
             if count == 0:
                 return 0, None
             return count, '../;view?%s=%s' % (filter, item.id)
+        # Don't show the "edit" link when i am not an admin.
+        elif column == 'id':
+            ac = resource.get_access_control()
+            is_admin =  ac.is_admin(context.user, resource)
+
+            id = item.id
+            if is_admin:
+                link = context.get_link(resource)
+                return id, '%s/;edit_record?id=%s' % (link, id)
+            else:
+                return id
 
         # Default
         cls = OrderedTable_View
         value = cls.get_item_value(self, resource, context, item, column)
 
-        # NOTE The field 'product' is reserved to make a reference to the
+        # FIXME The field 'product' is reserved to make a reference to the
         # 'products' table.  Currently it is used by the 'versions' and
         # 'modules' tables.
+        # The fields 'modules' and 'versions' is reserved to make reference to
+        # the 'modules' and 'versions' tables. Currently it is used by the
+        # 'product' table
         if column == 'product':
             value = int(value)
             handler = resource.parent.get_resource('product').handler
             record = handler.get_record(value)
             return handler.get_record_value(record, 'title')
+        elif column in ('modules', 'versions'):
+            # Strip 's' modules -> module
+            associated_table = resource.get_resource('../%s' % column[:-1])
+            handler = associated_table.handler
+            # Search
+            results = handler.search(PhraseQuery('product', str(item.id)))
+            count = len(results)
+            if count == 0:
+                return 0, None
+            return count, '%s/;view' % context.get_link(associated_table)
 
         return value
 
@@ -107,13 +136,60 @@ class SelectTable_View(OrderedTable_View):
         return items[start:start+size]
 
 
+    def action_remove(self, resource, context, form):
+        ids = form['ids']
+        parent = resource.parent
+        # Search all issues
+        query_terms = parent.get_issues_query_terms()
+        query = AndQuery(*query_terms)
+        results = context.root.search(query)
+
+        # Associated modules and versions
+        check_associated = (resource.name == 'product')
+        module = parent.get_resource('module')
+        version = parent.get_resource('version')
+        module_handler = module.handler
+        version_handler = version.handler
+
+        # Remove values only if no issues have them
+        handler = resource.handler
+        filter = str(resource.name)
+        removed = []
+        for id in ids:
+            query = PhraseQuery(filter, id)
+            count = len(results.search(query))
+            if check_associated:
+                product_query = PhraseQuery('product', str(id))
+                module_count = len(module_handler.search(product_query))
+                version_count = len(version_handler.search(product_query))
+                count = count + module_count + version_count
+            if count == 0:
+                handler.del_record(id)
+                removed.append(str(id))
+
+        # Ok
+        message = MSG(u'Resources removed: {resources}.')
+        message = message.gettext(resources=', '.join(removed)).encode('utf-8')
+        context.message = message
+
+
+
+class SelectTable_EditRecord(Table_EditRecord):
+
+    def get_widgets(self, resource, context):
+        widgets = Table_EditRecord.get_widgets(self, resource, context)
+        return [ widget if widget.name != 'product' else
+                 ReadOnlyWidget('product', title=MSG(u'Product'))
+                 for widget in widgets ]
+
+
 
 ###########################################################################
 # Resources
 ###########################################################################
 class Tracker_TableHandler(OrderedTableFile):
 
-    record_schema = {'title': Unicode(multiple=True)}
+    record_properties = {'title': Unicode(multiple=True)}
 
 
 
@@ -154,39 +230,14 @@ class Tracker_TableResource(OrderedTable):
 
 
     view = SelectTable_View()
-
-
-    def del_record_action(self, context):
-        # Check input
-        ids = context.get_form_values('ids', type=Integer)
-        if not ids:
-            return context.come_back(ERROR(u'No resource selected.'))
-
-        # Search all issues
-        query_terms = resource.parent.get_issues_query_terms()
-        query = AndQuery(*query_terms)
-        results = context.root.search(query)
-
-        # Remove values only if no issues have them
-        filter = str(self.name)
-        removed = []
-        for id in ids:
-            query = PhraseQuery(filter, id)
-            count = results.search(query).get_n_documents()
-            if count == 0:
-                self.handler.del_record(id)
-                removed.append(str(id))
-
-        # Ok
-        message = INFO(u'Resources removed: $resources.')
-        return context.come_back(message, resources=', '.join(removed))
+    edit_record = SelectTable_EditRecord()
 
 
 
 class ModulesHandler(Tracker_TableHandler):
 
-    record_schema = {
-        'product': String(mandatory=True),
+    record_properties = {
+        'product': String(mandatory=True, is_indexed=True),
         'title': Unicode(multiple=True, mandatory=True)}
 
 
@@ -199,7 +250,7 @@ class ModulesResource(Tracker_TableResource):
     def get_schema(self):
         products = self.parent.get_resource('product')
         return merge_dicts(
-            ModulesHandler.record_schema,
+            ModulesHandler.record_properties,
             product=ProductsEnumerate(products=products, mandatory=True))
 
 
@@ -211,8 +262,8 @@ class ModulesResource(Tracker_TableResource):
 
 class VersionsHandler(Tracker_TableHandler):
 
-    record_schema = {
-        'product': String(mandatory=True),
+    record_properties = {
+        'product': String(mandatory=True, is_indexed=True),
         'title': Unicode(multiple=True, mandatory=True),
         'released': Boolean}
 
@@ -226,7 +277,7 @@ class VersionsResource(Tracker_TableResource):
     def get_schema(self):
         products = self.parent.get_resource('product')
         return merge_dicts(
-            VersionsHandler.record_schema,
+            VersionsHandler.record_properties,
             product=ProductsEnumerate(products=products, mandatory=True))
 
 
@@ -235,11 +286,3 @@ class VersionsResource(Tracker_TableResource):
         title_widget,
         BooleanCheckBox('released', title=MSG(u'Released'))]
 
-
-
-###########################################################################
-# Register
-###########################################################################
-register_resource_class(Tracker_TableResource)
-register_resource_class(ModulesResource)
-register_resource_class(VersionsResource)
