@@ -23,22 +23,23 @@
 from datetime import datetime
 
 # Import from itools
+from itools.core import merge_dicts
 from itools.csv import CSVFile, Property
-from itools.datatypes import Boolean, Integer, String, Unicode, Enumerate
+from itools.datatypes import Boolean, Integer, String, Unicode
 from itools.gettext import MSG
 from itools.i18n import format_datetime
-from itools.handlers import merge_dicts
+from itools.stl import stl
 from itools.uri import encode_query, Reference
-from itools.web import BaseView, BaseForm, STLForm
-from itools.web import INFO, ERROR
+from itools.web import BaseView, BaseForm, STLForm, FormError, INFO, ERROR
 from itools.web.views import process_form
 
 # Import from ikaaro
 from ikaaro.buttons import Button
 from ikaaro.forms import HiddenWidget, TextWidget
 from ikaaro import messages
-from ikaaro.resource_views import DBResource_NewInstance
 from ikaaro.views import BrowseForm, SearchForm as BaseSearchForm, ContextMenu
+from ikaaro.views_new import NewInstance
+from ikaaro.registry import get_resource_class
 
 # Import from ikaaro.tracker
 from issue import Issue
@@ -90,12 +91,10 @@ class StoreSearchMenu(ContextMenu):
     def get_namespace(self, resource, context):
         # This search exists ?
         search_name = context.get_query_value('search_name')
-        search = None
         if search_name:
-            try:
-                search = resource.get_resource(search_name)
-            except LookupError:
-                pass
+            search = resource.get_resource(search_name, soft=True)
+        else:
+            search = None
 
         # Set the get function and the title
         if search:
@@ -150,13 +149,13 @@ class StoredSearchesMenu(ContextMenu):
             issues_nb = len(root.search(query))
             kw = {'search_title': item.get_property('title'),
                   'issues_nb': issues_nb}
-            title = MSG(u'${search_title} (${issues_nb})')
+            title = MSG(u'{search_title} ({issues_nb})')
             title = title.gettext(language=language, **kw)
 
             # Namespace
             items.append({'title': title,
                           'href': '%s?search_name=%s' % (base, item.name),
-                          'class': 'nav_active' if (item.name == search_name)
+                          'class': 'nav-active' if (item.name == search_name)
                                                 else None})
         items.sort(lambda x, y: cmp(x['title'], y['title']))
 
@@ -190,30 +189,33 @@ class TrackerViewMenu(ContextMenu):
 ###########################################################################
 # Views
 ###########################################################################
-class Tracker_NewInstance(DBResource_NewInstance):
+class Tracker_NewInstance(NewInstance):
 
-    schema = merge_dicts(
-        DBResource_NewInstance.schema,
-        product=Unicode(mandatory=True))
-
-    widgets = DBResource_NewInstance.widgets + \
-        [TextWidget('product', title=MSG(u'Give the title of one Product'))]
+    schema = merge_dicts(NewInstance.schema, product=Unicode(mandatory=True))
+    widgets = NewInstance.widgets + [
+        TextWidget('product', title=MSG(u'Give the title of one Product'))]
 
 
     def action(self, resource, context, form):
-        ok = DBResource_NewInstance.action(self, resource, context, form)
-        if ok is None:
-            return
-
-        # Add the initial product
         name = form['name']
+        title = form['title']
+
+        # Create the resource
+        class_id = context.query['type']
+        cls = get_resource_class(class_id)
+        child = cls.make_resource(cls, resource, name)
+        # The metadata
+        metadata = child.metadata
+        language = resource.get_content_language(context)
+        metadata.set_property('title', title, language=language)
+        # Add the initial product
         product = form['product']
         table = resource.get_resource('%s/product' % name).get_handler()
         product = Property(product, language='en')
         table.add_record({'title': product})
 
-        # Ok
-        return ok
+        goto = './%s/' % name
+        return context.come_back(messages.MSG_NEW_RESOURCE, goto=goto)
 
 
 
@@ -223,26 +225,31 @@ class Tracker_AddIssue(STLForm):
     title = MSG(u'Add')
     icon = 'new.png'
     template = '/ui/tracker/add_issue.xml'
+    styles = ['/ui/tracker/style.css']
+    scripts = ['/ui/tracker/tracker.js']
 
 
     def get_schema(self, resource, context):
         return get_issue_fields(resource)
 
 
+    def get_value(self, resource, context, name, datatype):
+        if getattr(datatype, 'mandatory', False):
+            datatype = datatype(mandatory=False)
+        return context.get_query_value(name, type=datatype)
+
+
     def get_namespace(self, resource, context):
-        context.styles.append('/ui/tracker/tracker.css')
-        context.scripts.append('/ui/tracker/tracker.js')
-
-        namespace =  self.build_namespace(resource, context)
+        namespace = STLForm.get_namespace(self, resource, context)
         namespace['list_products'] = resource.get_list_products_namespace()
-
         return namespace
 
 
     def action(self, resource, context, form):
         # Add
         id = resource.get_new_id()
-        issue = Issue.make_resource(Issue, resource, id)
+        issue_cls = resource.issue_class
+        issue = issue_cls.make_resource(issue_cls, resource, id)
         issue._add_record(context, form)
 
         # Ok
@@ -257,6 +264,8 @@ class Tracker_View(BrowseForm):
     access = 'is_allowed_to_view'
     title = MSG(u'View')
     icon = 'view.png'
+    styles = ['/ui/tracker/style.css']
+
 
     schema = {
         'ids': String(multiple=True, mandatory=True)}
@@ -265,7 +274,7 @@ class Tracker_View(BrowseForm):
         # Do not batch
         'batch_size': Integer(default=0),
         # search_fields
-        'search_name': Unicode(),
+        'search_name': String(),
         'mtime': Integer(default=0),
         'product': Integer(multiple=True),
         'module': Integer(multiple=True),
@@ -291,14 +300,32 @@ class Tracker_View(BrowseForm):
                            self.tracker_schema)
 
 
+    def on_query_error(self, resource, context):
+        query = encode_query(context.uri.query)
+        return context.come_back(None, goto=';search?%s' % query)
+
+
+    def get_page_title(self, resource, context):
+        query = getattr(context, 'query', {})
+        search_name = query.get('search_name')
+        if search_name:
+            search = context.resource.get_resource(search_name, soft=True)
+            if search:
+                search = search.get_title()
+                template = MSG(u'{title} - {search}')
+                title = self.title.gettext()
+                return template.gettext(title=title, search=search)
+
+        return self.title
+
+
     def GET(self, resource, context):
         # Check stored search
         search_name = context.query['search_name']
         if search_name:
-            try:
-                search = resource.get_resource(search_name)
-            except LookupError:
-                msg = MSG(u'Unknown stored search "${sname}".')
+            search = resource.get_resource(search_name, soft=True)
+            if search is None:
+                msg = MSG(u'Unknown stored search "{sname}".')
                 goto = ';search'
                 return context.come_back(msg, goto=goto, sname=search_name)
         # Ok
@@ -306,9 +333,6 @@ class Tracker_View(BrowseForm):
 
 
     def get_namespace(self, resource, context):
-        # Set Style
-        context.styles.append('/ui/tracker/tracker.css')
-
         # Default table namespace
         namespace = BrowseForm.get_namespace(self, resource, context)
 
@@ -345,12 +369,44 @@ class Tracker_View(BrowseForm):
             sort_by = query['sort_by']
         if query['reverse'] is not None:
             reverse = query['reverse']
+
+        # If an ordered table => sort the result with its natural order
+        if sort_by in ('product', 'module', 'version', 'type', 'state',
+                       'priority'):
+            # Make the key function
+            table_handler = resource.get_resource(sort_by).handler
+            sorted_ids = list(table_handler.get_record_ids_in_order())
+
+            def cmp_xy(x, y):
+                x_value = getattr(x, sort_by)
+                try:
+                    x_idx = sorted_ids.index(x_value)
+                except ValueError:
+                    x_idx = None
+
+                y_value = getattr(y, sort_by)
+                try:
+                    y_idx = sorted_ids.index(y_value)
+                except ValueError:
+                    y_idx = None
+
+                return cmp(x_idx, y_idx)
+
+            # Sort issues
+            issues = results.get_documents()
+            issues.sort(cmp_xy, reverse=reverse)
+
+            # Return the result
+            return issues
+
+        # Else, ...
         return results.get_documents(sort_by=sort_by, reverse=reverse)
 
 
     def get_item_value(self, resource, context, item, column):
         if column == 'checkbox':
-            selected_issues = context.get_form_values('ids') or []
+            datatype = String(multiple=True)
+            selected_issues = context.get_form_value('ids', type=datatype)
             return item.name, item.name in selected_issues
         if column == 'id':
             id = item.name
@@ -364,15 +420,13 @@ class Tracker_View(BrowseForm):
         # Assigned to
         if column == 'assigned_to':
             users = resource.get_resource('/users')
-            try:
-                user = users.get_resource(value)
-            except LookupError:
+            user = users.get_resource(value, soft=True)
+            if user is None:
                 return None
             return user.get_title()
         # Mtime
         if column == 'mtime':
-            mtime = datetime.strptime(value, '%Y%m%d%H%M%S')
-            return format_datetime(mtime)
+            return format_datetime(value)
 
         # Tables
         table = resource.get_resource(column).handler
@@ -397,11 +451,14 @@ class Tracker_Search(BaseSearchForm, Tracker_View):
     access = 'is_allowed_to_view'
     title = MSG(u'Search')
     icon = 'search.png'
+    styles = ['/ui/tracker/style.css']
+    scripts = ['/ui/tracker/tracker.js']
+
 
     # Search Form
     search_template = '/ui/tracker/search.xml'
     search_schema = {
-        'search_name': Unicode(),
+        'search_name': String(),
         'search_title': Unicode(),
         'text': Unicode(),
         'mtime': Integer(),
@@ -416,11 +473,23 @@ class Tracker_Search(BaseSearchForm, Tracker_View):
 
     context_menus = []
 
-    def get_search_namespace(self, resource, context):
-        # Set Style & JS
-        context.styles.append('/ui/tracker/tracker.css')
-        context.scripts.append('/ui/tracker/tracker.js')
 
+    def get_query(self, context):
+        try:
+            return BaseSearchForm.get_query(self, context)
+        except FormError:
+            schema = self.get_query_schema()
+            query = {}
+            for name in schema:
+                default = schema[name].get_default()
+                query[name] = context.uri.query.get(name, default)
+            return query
+
+
+    on_query_error = BaseSearchForm.on_query_error
+
+
+    def get_search_namespace(self, resource, context):
         # Search Form
         get_resource = resource.get_resource
         query = context.query
@@ -468,16 +537,18 @@ class Tracker_Search(BaseSearchForm, Tracker_View):
                                  tracker=resource).get_namespace(state),
            'priorities': TrackerList(element='priority',
                                  tracker=resource).get_namespace(priority),
-           'assigned_to': UsersList(tracker=resource).get_namespace(
-                                                         assigned_to),
+           'assigned_to': UsersList(tracker=resource,
+               excluded_roles=('guests',)).get_namespace(assigned_to),
            'list_products': resource.get_list_products_namespace()}
 
 
     def get_namespace(self, resource, context):
-        namespace = BaseSearchForm.get_namespace(self, resource, context)
-        namespace['batch'] = None
-        namespace['table'] = None
-        return namespace
+        search_template = resource.get_resource(self.search_template)
+        search_namespace = self.get_search_namespace(resource, context)
+        return {
+            'batch': None,
+            'table': None,
+            'search': stl(search_template, search_namespace)}
 
 
 
@@ -527,7 +598,8 @@ class Tracker_RememberSearch(BaseForm):
         search.handler.load_state_from_string('')
 
         # Set title
-        search.set_property('title', title)
+        language = resource.get_content_language(context)
+        search.set_property('title', title, language=language)
 
         # Save the value
         for name, type in StoredSearchFile.schema.iteritems():
@@ -566,11 +638,8 @@ class Tracker_GoToIssue(BaseView):
         if not issue_name:
             return context.come_back(messages.MSG_NAME_MISSING)
 
-        if not resource.has_resource(issue_name):
-            return context.come_back(ERROR(u'Issue not found.'))
-
-        issue = resource.get_resource(issue_name)
-        if not isinstance(issue, Issue):
+        issue = resource.get_resource(issue_name, soft=True)
+        if issue is None or not isinstance(issue, Issue):
             return context.come_back(ERROR(u'Issue not found.'))
 
         return context.uri.resolve2('../%s/;edit' % issue_name)
@@ -661,11 +730,9 @@ class Tracker_ExportToCSV(BaseView):
                 row.append(value)
             csv.add_row(row)
 
-        # Set response type
-        response = context.response
-        response.set_header('Content-Type', 'text/comma-separated-values')
-        response.set_header('Content-Disposition',
-                            'attachment; filename=export.csv')
+        # Ok
+        context.set_content_type('text/comma-separated-values')
+        context.set_content_disposition('attachment', 'export.csv')
         return csv.to_str(separator=separator)
 
 
@@ -727,6 +794,7 @@ class Tracker_ChangeSeveralBugs(Tracker_View):
     access = 'is_allowed_to_view'
     title = MSG(u'Change Several Issues')
     template = '/ui/tracker/change_bugs.xml'
+    scripts = ['/ui/tracker/tracker.js']
     schema = {
         'comment': Unicode,
         'ids': String(multiple=True),
@@ -746,7 +814,6 @@ class Tracker_ChangeSeveralBugs(Tracker_View):
 
 
     def get_namespace(self, resource, context):
-        context.scripts.append('/ui/tracker/tracker.js')
         namespace = Tracker_View.get_namespace(self, resource, context)
         # Edit several bugs at once
         get_resource = resource.get_resource
@@ -756,7 +823,8 @@ class Tracker_ChangeSeveralBugs(Tracker_View):
         namespace['priorities'] = get_resource('priority').get_options()
         namespace['types'] = get_resource('type').get_options()
         namespace['states'] = get_resource('state').get_options()
-        namespace['users'] = resource.get_members_namespace('')
+        namespace['assigned_to'] = UsersList(tracker=resource,
+                excluded_roles=('guests',)).get_namespace('')
         namespace['list_products'] = resource.get_list_products_namespace()
 
         # Ok
@@ -829,7 +897,7 @@ class Tracker_ChangeSeveralBugs(Tracker_View):
             if assigned_to and assigned_to != new_assigned_to:
                 users_issues.setdefault(assigned_to, []).append(info)
             # Change
-            context.server.change_resource(issue)
+            context.database.change_resource(issue)
 
         # Send mails
         root = context.root
@@ -837,7 +905,8 @@ class Tracker_ChangeSeveralBugs(Tracker_View):
             user_title = MSG(u'ANONYMOUS').gettext()
         else:
             user_title = user.get_title()
-        template = MSG(u'--- Comment from: $user ---\n\n$comment\n\n$issues')
+        template = MSG(
+            u'--- Comment from: {user} ---\n\n{comment}\n\n{issues}')
         tracker_title = resource.get_property('title') or 'Tracker Issue'
         subject = u'[%s]' % tracker_title
         for user_id in users_issues:
@@ -885,15 +954,12 @@ def get_issue_informations(resource, item):
     infos['assigned_to'] = ''
     if assigned_to:
         users = resource.get_resource('/users')
-        if users.has_resource(assigned_to):
-            user = users.get_resource(assigned_to)
+        user = users.get_resource(assigned_to, soft=True)
+        if user is not None:
             infos['assigned_to'] = user.get_title()
 
     # Modification Time
-    mtime_sort = item.mtime
-    mtime = datetime.strptime(mtime_sort, '%Y%m%d%H%M%S')
-    infos['mtime'] = format_datetime(mtime)
-    infos['mtime_sort'] = mtime_sort
+    infos['mtime'] = format_datetime(item.mtime)
 
     return infos
 
