@@ -7,19 +7,19 @@ update:"Aktualisieren",
 cancel:"Abbrechen",
 close:"Schlie\u00DFen",
 browse:"Durchsuchen",
-class_name:"Klasse",
-not_set:"-- Nicht gesetzt --",
+class_name:"CSS-Klasse",
+not_set:"- unbestimmt -",
 clipboard_msg:"Kopieren, Ausschneiden und Einf\u00FCgen sind im Mozilla Firefox nicht m\u00F6glich.\nM\u00F6chten Sie mehr \u00FCber dieses Problem erfahren?",
 clipboard_no_support:"Wird derzeit in Ihrem Browser nicht unterst\u00FCtzt. Bitte benutzen Sie stattdessen die Tastenk\u00FCrzel.",
-popup_blocked:"Leider hat Ihr Popup-Blocker ein Fenster unterbunden, das f\u00FCr den Betrieb dieses Programms n\u00F6tig ist. Bitte deaktivieren Sie den Popup-Blocker, um die volle Funktionalit\u00E4t zu erlangen.",
+popup_blocked:"Leider hat Ihr Popup-Blocker ein Fenster unterbunden, das f\u00FCr den Betrieb dieses Programms n\u00F6tig ist. Bitte deaktivieren Sie den Popup-Blocker f\u00FCr diese Seite.",
 invalid_data:"Fehler: Sie haben ung\u00FCltige Werte eingegeben (rot markiert).",
 more_colors:"Weitere Farben"
 },
 contextmenu:{
 align:"Ausrichtung",
-left:"Links",
-center:"Mittig",
-right:"Rechts",
+left:"Linksb\u00FCndig",
+center:"Zentriert",
+right:"Rechtsb\u00FCndig",
 full:"Blocksatz"
 },
 insertdatetime:{
@@ -28,7 +28,7 @@ time_fmt:"%H:%M:%S",
 insertdate_desc:"Datum einf\u00FCgen",
 inserttime_desc:"Zeit einf\u00FCgen",
 months_long:"Januar,Februar,M\u00E4rz,April,Mai,Juni,Juli,August,September,Oktober,November,Dezember",
-months_short:"Jan,Feb,M\u00E4rz,Apr,Mai,Juni,Juli,Aug,Sept,Okt,Nov,Dez",
+months_short:"Jan,Feb,M\u00E4r,Apr,Mai,Juni,Juli,Aug,Sept,Okt,Nov,Dez",
 day_long:"Sonntag,Montag,Dienstag,Mittwoch,Donnerstag,Freitag,Samstag,Sonntag",
 day_short:"So,Mo,Di,Mi,Do,Fr,Sa,So"
 },
@@ -44,8 +44,8 @@ rtl_desc:"Schrift von rechts nach links"
 },
 layer:{
 insertlayer_desc:"Neue Ebene einf\u00FCgen",
-forward_desc:"Vor verschieben",
-backward_desc:"Zur\u00FCck verschieben",
+forward_desc:"Nach vorne holen",
+backward_desc:"Nach hinten legen",
 absolute_desc:"Absolute Positionierung",
 content:"Neue Ebene..."
 },
@@ -88,9 +88,11 @@ style:{
 desc:"CSS-Styles bearbeiten"
 },
 paste:{
-paste_text_desc:"Als normalen Text einf\u00FCgen",
+paste_text_desc:"Als einfachen Text einf\u00FCgen",
 paste_word_desc:"Mit Formatierungen (aus Word) einf\u00FCgen",
-selectall_desc:"Alles ausw\u00E4hlen"
+selectall_desc:"Alles ausw\u00E4hlen",
+plaintext_mode_sticky:"Einf\u00FCgemodus ist nun \"Nur Text\". Erneut klicken (oder das Einf\u00FCgen aus der Zwischenablage) stellt den Normalmodus wieder her.",
+plaintext_mode:"Einf\u00FCgemodus ist nun \"Nur Text\". Erneut klicken stellt den Normalmodus wieder her."
 },
 paste_dlg:{
 text_title:"Dr\u00FCcken Sie auf Ihrer Tastatur Strg+V, um den Text einzuf\u00FCgen.",
@@ -98,15 +100,15 @@ text_linebreaks:"Zeilenumbr\u00FCche beibehalten",
 word_title:"Dr\u00FCcken Sie auf Ihrer Tastatur Strg+V, um den Text einzuf\u00FCgen."
 },
 table:{
-desc:"Tabelle erstellen",
+desc:"Tabelle erstellen/bearbeiten",
 row_before_desc:"Zeile oberhalb einf\u00FCgen",
 row_after_desc:"Zeile unterhalb einf\u00FCgen",
-delete_row_desc:"Zeile entfernen",
+delete_row_desc:"Zeile l\u00F6schen",
 col_before_desc:"Spalte links einf\u00FCgen",
 col_after_desc:"Spalte rechts einf\u00FCgen",
 delete_col_desc:"Spalte l\u00F6schen",
-split_cells_desc:"Vereinte Zellen trennen",
-merge_cells_desc:"Zellen vereinen",
+split_cells_desc:"Verbundene Zellen trennen",
+merge_cells_desc:"Zellen verbinden",
 row_desc:"Eigenschaften der Zeile",
 cell_desc:"Eigenschaften der Zelle",
 props_desc:"Eigenschaften der Tabelle",
@@ -114,14 +116,16 @@ paste_row_before_desc:"Zeile oberhalb aus der Zwischenablage einf\u00FCgen",
 paste_row_after_desc:"Zeile unterhalb aus der Zwischenablage einf\u00FCgen",
 cut_row_desc:"Zeile ausschneiden",
 copy_row_desc:"Zeile kopieren",
-del:"Tabelle entfernen",
+del:"Tabelle l\u00F6schen",
 row:"Zeile",
 col:"Spalte",
 cell:"Zelle",
 cellprops_delta_width:"150"
 },
 autosave:{
-unload_msg:"Ihre \u00C4nderungen werden verloren gehen, wenn Sie die Seite verlassen."
+unload_msg:"Ihre \u00C4nderungen werden verloren gehen, wenn Sie die Seite verlassen.",
+restore_content:"Automatisch gespeicherten Inhalt wiederherstellen.",
+warning_message:"Wenn Sie den Inhalt wiederherstellen, gehen die aktuellen Daten im Editor verloren.\n\nSind sie sicher, dass Sie den Inhalt wiederherstellen m\u00F6chten?"
 },
 fullscreen:{
 desc:"Vollbildschirm"
@@ -134,10 +138,10 @@ fullpage:{
 desc:"Dokument-Eigenschaften"
 },
 template:{
-desc:"Vorgefertigten Vorlageninhalt einf\u00FCgen"
+desc:"Inhalt aus Vorlage einf\u00FCgen"
 },
 visualchars:{
-desc:"Sichtbarkeit der Steuerzeichen an/aus."
+desc:"Sichtbarkeit der Steuerzeichen an/aus"
 },
 spellchecker:{
 desc:"Rechtschreibpr\u00FCfung an/aus",
@@ -151,5 +155,17 @@ no_sug:"Keine Vorschl\u00E4ge",
 no_mpell:"Keine Rechtschreibfehler gefunden."
 },
 pagebreak:{
-desc:"Seitenumbruch Einf\u00FCgen."
+desc:"Seitenumbruch einf\u00FCgen"
+},
+advlist:{
+types:"Typen",
+def:"Standard",
+lower_alpha:"a. b. c.",
+lower_greek:"1. 2. 3.",
+lower_roman:"i. ii. iii.",
+upper_alpha:"A. B. C.",
+upper_roman:"I. II. III.",
+circle:"Kreis",
+disc:"Punkt",
+square:"Quadrat"
 }}});
