@@ -5,7 +5,7 @@ tab_advanced:"Erweitert",
 general:"Allgemein",
 title:"Titel",
 preview:"Vorschau",
-constrain_proportions:"Ausma\u00DFe",
+constrain_proportions:"Seitenverh\u00E4ltnis beibehalten",
 langdir:"Schriftrichtung",
 langcode:"Sprachcode",
 long_desc:"Ausf\u00FChrliche Beschreibung",
@@ -21,7 +21,7 @@ mouseover:"bei Mauskontakt",
 mouseout:"bei keinem Mauskontakt",
 misc:"Verschiedenes",
 example_img:"Aussehen der Vorschau",
-missing_alt:"Wollen Sie wirklich keine Beschreibung eingeben? Bestimmte Benutzer mit k\u00F6rperlichen Einschr\u00E4nkungen k\u00F6nnen so nicht darauf zugrifen, ebenso solche, die einen Textbrowser benutzen oder die Anzeige von Bildern deaktiviert haben.",
+missing_alt:"Wollen Sie wirklich keine Beschreibung eingeben? Bestimmte Benutzer mit k\u00F6rperlichen Einschr\u00E4nkungen k\u00F6nnen so nicht darauf zugreifen, ebenso solche, die einen Textbrowser benutzen oder die Anzeige von Bildern deaktiviert haben.",
 dialog_title:"Bild einf\u00FCgen/bearbeiten",
 src:"Adresse",
 alt:"Beschreibung",
