@@ -4,7 +4,7 @@ tab_appearance:"Apparence",
 tab_advanced:"Avanc\u00E9",
 general:"G\u00E9n\u00E9ral",
 title:"Titre",
-preview:"Pr\u00E9visualiser",
+preview:"Pr\u00E9visualisation",
 constrain_proportions:"Conserver les proportions",
 langdir:"Sens de lecture",
 langcode:"Code de la langue",
@@ -14,15 +14,15 @@ classes:"Classes",
 ltr:"De gauche \u00E0 droite",
 rtl:"De droite \u00E0 gauche",
 id:"Id",
-map:"Carte image",
-swap_image:"Alterner image",
+map:"Image cliquable",
+swap_image:"Image de remplacement",
 alt_image:"Image alternative",
-mouseover:"au passage de la souris",
+mouseover:"au survol de la souris",
 mouseout:"\u00E0 la sortie de la souris",
 misc:"Divers",
 example_img:"Apparence de l'image",
-missing_alt:"\u00CAtes-vous s\u00FBr de vouloir continuer sans inclure de description de l'image\u00A0? Sans description, l'image peut ne pas \u00EAtre accessible \u00E0 certains utilisateurs handicap\u00E9s visuellement, ceux utilisant un navigateur texte, ou ceux qui naviguent sans affichage des images.",
-dialog_title:"Ins\u00E9rer/\u00C9diter image",
+missing_alt:"\u00CAtes-vous s\u00FBr de vouloir continuer sans d\u00E9finir de description pour l'image ? Sans elle, l'image peut ne pas \u00EAtre accessible \u00E0 certains utilisateurs handicap\u00E9s, ceux utilisant un navigateur texte ou ceux qui naviguent sans affichage des images.",
+dialog_title:"Ins\u00E9rer / \u00E9diter une image",
 src:"URL de l'image",
 alt:"Description de l'image",
 list:"Liste d'images",
@@ -31,13 +31,13 @@ dimensions:"Dimensions",
 vspace:"Espacement vertical",
 hspace:"Espacement horizontal",
 align:"Alignement",
-align_baseline:"Base",
-align_top:"Haut",
-align_middle:"Milieu",
-align_bottom:"Bas",
-align_texttop:"Haut du texte",
-align_textbottom:"Bas du texte",
-align_left:"Gauche",
-align_right:"Droite",
+align_baseline:"Normal",
+align_top:"En haut",
+align_middle:"Au milieu",
+align_bottom:"En bas",
+align_texttop:"Texte en haut",
+align_textbottom:"Texte en bas",
+align_left:"Gauche (flottant)",
+align_right:"Droite (flottant)",
 image_list:"Liste d'images"
 });
