@@ -1,8 +1,11 @@
 /**
- * $Id: editor_plugin_src.js 952 2008-11-03 17:56:04Z spocke $
+ * editor_plugin_src.js
  *
- * @author Moxiecode
- * @copyright Copyright  2004-2008, Moxiecode Systems AB, All rights reserved.
+ * Copyright 2009, Moxiecode Systems AB
+ * Released under LGPL License.
+ *
+ * License: http://tinymce.moxiecode.com/license
+ * Contributing: http://tinymce.moxiecode.com/contributing
  */
 
 (function() {
@@ -61,7 +64,7 @@
 				if (ed.settings.content_css !== false)
 					ed.dom.loadCSS(url + "/css/content.css");
 
-				if (ed.theme.onResolveName) {
+				if (ed.theme && ed.theme.onResolveName) {
 					ed.theme.onResolveName.add(function(th, o) {
 						if (o.name == 'img') {
 							each(lo, function(v, k) {
@@ -165,14 +168,14 @@
 				o.content = o.content.replace(/_mce_value=/g, 'value=');
 			});
 
-			if (ed.getParam('media_use_script')) {
-				function getAttr(s, n) {
-					n = new RegExp(n + '=\"([^\"]+)\"', 'g').exec(s);
+			function getAttr(s, n) {
+				n = new RegExp(n + '=\"([^\"]+)\"', 'g').exec(s);
 
-					return n ? ed.dom.decode(n[1]) : '';
-				};
+				return n ? ed.dom.decode(n[1]) : '';
+			};
 
-				ed.onPostProcess.add(function(ed, o) {
+			ed.onPostProcess.add(function(ed, o) {
+				if (ed.getParam('media_use_script')) {
 					o.content = o.content.replace(/<img[^>]+>/g, function(im) {
 						var cl = getAttr(im, 'class');
 
@@ -185,8 +188,8 @@
 
 						return im;
 					});
-				});
-			}
+				}
+			});
 		},
 
 		getInfo : function() {
@@ -233,16 +236,20 @@
 
 			if (stc) {
 				ob = dom.create('span', {
-					mce_name : 'object',
+					id : p.id,
+					_mce_name : 'object',
 					type : 'application/x-shockwave-flash',
 					data : p.src,
+					style : dom.getAttrib(n, 'style'),
 					width : o.width,
 					height : o.height
 				});
 			} else {
 				ob = dom.create('span', {
-					mce_name : 'object',
+					id : p.id,
+					_mce_name : 'object',
 					classid : "clsid:" + o.classid,
+					style : dom.getAttrib(n, 'style'),
 					codebase : o.codebase,
 					width : o.width,
 					height : o.height
@@ -250,18 +257,18 @@
 			}
 
 			each (p, function(v, k) {
-				if (!/^(width|height|codebase|classid|_cx|_cy)$/.test(k)) {
+				if (!/^(width|height|codebase|classid|id|_cx|_cy)$/.test(k)) {
 					// Use url instead of src in IE for Windows media
-					if (o.type == 'application/x-mplayer2' && k == 'src')
+					if (o.type == 'application/x-mplayer2' && k == 'src' && !p.url)
 						k = 'url';
 
 					if (v)
-						dom.add(ob, 'span', {mce_name : 'param', name : k, '_mce_value' : v});
+						dom.add(ob, 'span', {_mce_name : 'param', name : k, '_mce_value' : v});
 				}
 			});
 
 			if (!stc)
-				dom.add(ob, 'span', tinymce.extend({mce_name : 'embed', type : o.type}, p));
+				dom.add(ob, 'span', tinymce.extend({_mce_name : 'embed', type : o.type, style : dom.getAttrib(n, 'style')}, p));
 
 			return ob;
 		},
@@ -337,13 +344,14 @@
 		_createImg : function(cl, n) {
 			var im, dom = this.editor.dom, pa = {}, ti = '', args;
 
-			args = ['id', 'name', 'width', 'height', 'bgcolor', 'align', 'flashvars', 'src', 'wmode', 'allowfullscreen', 'quality'];
+			args = ['id', 'name', 'width', 'height', 'bgcolor', 'align', 'flashvars', 'src', 'wmode', 'allowfullscreen', 'quality', 'data'];
 
 			// Create image
 			im = dom.create('img', {
 				src : this.url + '/img/trans.gif',
 				width : dom.getAttrib(n, 'width') || 100,
 				height : dom.getAttrib(n, 'height') || 100,
+				style : dom.getAttrib(n, 'style'),
 				'class' : cl
 			});
 
@@ -367,6 +375,12 @@
 				delete pa.movie;
 			}
 
+			// No src try data
+			if (!pa.src) {
+				pa.src = pa.data;
+				delete pa.data;
+			}
+
 			// Merge with embed args
 			n = dom.select('.mceItemEmbed', n)[0];
 			if (n) {
