@@ -5,7 +5,7 @@ fontdefault:"Schriftart",
 block:"Vorlage",
 paragraph:"Absatz",
 div:"Zusammenh\u00E4ngender Bereich",
-address:"Addresse",
+address:"Adresse",
 pre:"Rohdaten",
 h1:"\u00DCberschrift 1",
 h2:"\u00DCberschrift 2",
@@ -22,9 +22,9 @@ bold_desc:"Fett (Strg+B)",
 italic_desc:"Kursiv (Strg+I)",
 underline_desc:"Unterstrichen (Strg+U)",
 striketrough_desc:"Durchgestrichen",
-justifyleft_desc:"Links ausgerichtet",
-justifycenter_desc:"Mittig ausgerichtet",
-justifyright_desc:"Rechts ausgerichtet",
+justifyleft_desc:"Linksb\u00FCndig",
+justifycenter_desc:"Zentriert",
+justifyright_desc:"Rechtsb\u00FCndig",
 justifyfull_desc:"Blocksatz",
 bullist_desc:"Unsortierte Liste",
 numlist_desc:"Sortierte Liste",
