@@ -42,8 +42,8 @@ sup_desc:"Super\u00EDndice",
 hr_desc:"Insertar regla horizontal",
 removeformat_desc:"Limpiar formato",
 custom1_desc:"Su descripci\u00F3n personal aqu\u00ED",
-forecolor_desc:"Seleccionar color del texto",
-backcolor_desc:"Seleccionar color de fondo",
+forecolor_desc:"Elegir color del texto",
+backcolor_desc:"Elegir color de fondo",
 charmap_desc:"Insertar caracteres personalizados",
 visualaid_desc:"Mostrar/ocultar l\u00EDnea de gu\u00EDa/elementos invisibles",
 anchor_desc:"Insertar/editar ancla",
@@ -54,9 +54,9 @@ image_props_desc:"Propiedades de imagen",
 newdocument_desc:"Nuevo documento",
 help_desc:"Ayuda",
 blockquote_desc:"Cita",
-clipboard_msg:"Copiar/Cortar/Pegar no se encuentra disponible en Mozilla y Firefox.\r\n \u00BFDesea obtener m\u00E1s informaci\u00F3n acerca de este tema?",
+clipboard_msg:"Copiar/Cortar/Pegar no se encuentra disponible en Mozilla y Firefox.\n \u00BFQuiere m\u00E1s informaci\u00F3n sobre este tema?",
 path:"Ruta",
-newdocument:" \u00BFEst\u00E1 seguro que desea limpiar todo el contenido?",
+newdocument:" \u00BFSeguro que desea limpiar todo el contenido?",
 toolbar_focus:"Ir a los botones de herramientas - Alt+Q, Ir al editor - Alt-Z, Ir a la ruta del elemento - Alt-X",
 more_colors:"M\u00E1s colores"
 });
