@@ -12,7 +12,7 @@ anchor_title:"Insertar/editar ancla",
 anchor_name:"Nombre del ancla",
 code_title:"Editor del c\u00F3digo fuente HTML",
 code_wordwrap:"Ajustar al margen",
-colorpicker_title:"Seleccionar color",
+colorpicker_title:"Elegir color",
 colorpicker_picker_tab:"Selector",
 colorpicker_picker_title:"Paleta de color",
 colorpicker_palette_tab:"Paleta",
@@ -21,7 +21,7 @@ colorpicker_named_tab:"Nombrados",
 colorpicker_named_title:"Colores nombrados",
 colorpicker_color:"Color:",
 colorpicker_name:"Nombre:",
-charmap_title:"Seleccionar caracter personalizado",
+charmap_title:"Elegir caracter personalizado",
 image_title:"Insertar/editar imagen",
 image_src:"URL de la Imagen",
 image_alt:"Descripci\u00F3n de la Imagen",
