@@ -3,9 +3,9 @@ bold_desc:"Gras (Ctrl+B)",
 italic_desc:"Italique (Ctrl+I)",
 underline_desc:"Soulign\u00E9 (Ctrl+U)",
 striketrough_desc:"Barr\u00E9",
-bullist_desc:"Liste non-num\u00E9rot\u00E9e",
+bullist_desc:"Liste \u00E0 puces",
 numlist_desc:"Liste num\u00E9rot\u00E9e",
 undo_desc:"Annuler (Ctrl+Z)",
 redo_desc:"R\u00E9tablir (Ctrl+Y)",
-cleanup_desc:"Nettoyer le code non propre"
+cleanup_desc:"Nettoyer le code"
 });
