@@ -14,11 +14,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-# Import from itools
-from itools import vfs
 
-
-
-def is_instance_up_to_date(target):
-    # All tests pass
-    return True
+def is_instance_up_to_date(root):
+    # 0.60 Check the Git archive has been initalized
+    return root.metadata.database.fs.exists('.git')
