@@ -43,7 +43,6 @@ class User_ConfirmRegistration(STLForm):
 
     def get_namespace(self, resource, context):
         # Check register key
-        # FIXME This does not work
         must_confirm = resource.get_property('user_must_confirm')
         username = context.get_form_value('username', default='')
         if must_confirm is None:
@@ -199,7 +198,7 @@ class User_EditAccount(AutoForm):
         is_same_user = (resource.name == context.user.name)
         if is_same_user:
             password = form['password']
-            if not resource.authenticate(password):
+            if not resource.authenticate(password, clear=True):
                 context.message = ERROR(
                     u"You mistyped your actual password, your account is"
                     u" not changed.")
@@ -209,9 +208,9 @@ class User_EditAccount(AutoForm):
         # user with the same email in the database.
         if email != resource.get_property('email'):
             results = context.root.search(email=email)
-            if results.get_n_documents():
+            if len(results):
                 context.message = ERROR(
-                    u'There is another user with the email "${email}", please'
+                    u'There is another user with the email "{email}", please'
                     u' try again.', email=email).gettext()
                 return
 
@@ -258,7 +257,7 @@ class User_EditPreferences(STLForm):
         else:
             resource.set_property('user_language', value)
         # Ok
-        context.message = INFO(u'Application preferences changed.')
+        context.message = messages.MSG_CHANGES_SAVED
 
 
 
@@ -291,7 +290,7 @@ class User_EditPassword(STLForm):
         is_same_user = (resource.name == context.user.name)
         if is_same_user:
             password = form['password']
-            if not resource.authenticate(password):
+            if not resource.authenticate(password, clear=True):
                 context.message = ERROR(
                     u"You mistyped your actual password, your account is"
                     u" not changed.")
@@ -315,7 +314,7 @@ class User_EditPassword(STLForm):
             resource.set_auth_cookie(context, newpass)
 
         # Ok
-        context.message = INFO(u'Password changed.')
+        context.message = messages.MSG_CHANGES_SAVED
 
 
 
