@@ -20,13 +20,10 @@
 from random import sample
 from hashlib import sha1
 from sys import platform
-from urllib import quote
 
 # Import from itools
-from itools.utils import freeze
 from itools.web import get_context
 from itools.xapian import AllQuery, PhraseQuery, NotQuery, OrQuery, StartQuery
-from itools.xapian import AndQuery
 
 if platform[:3] == 'win':
     from utils_win import is_pid_running, kill
@@ -51,7 +48,7 @@ def get_parameters(prefix, **kw):
       get_parameters('resources', sortby='id', sortorder='up')
     """
     # Get the form field from the request (a zope idiom)
-    get_parameter = get_context().request.get_parameter
+    get_parameter = get_context().get_form_value
 
     # Get the parameters
     parameters = {}
@@ -62,27 +59,6 @@ def get_parameters(prefix, **kw):
     return parameters
 
 
-def preserve_parameters(preserve=freeze([])):
-    """Returns an HTML snippet with hidden input html elements, there will
-    be one element for each request parameter that starts with any of
-    the prefixes contained in the preserve parameter.
-
-    This lets to pass url request parameters through form actions, so we
-    don't lose important navigation information.
-    """
-    snippet = []
-
-    form = get_context().request.get_form()
-    for k, v in form.items():
-        for prefix in preserve:
-            if k.startswith(prefix):
-                snippet.append('<input type="hidden" name="%s" value="%s">'
-                               % (k, quote(v)))
-                break
-
-    return '\n'.join(snippet)
-
-
 
 ###########################################################################
 # Languages
@@ -119,7 +95,12 @@ def reduce_string(title='', word_treshold=15, phrase_treshold=40):
             words.insert(i, word)
     title = ' '.join(words)
     if len(title) > phrase_treshold:
-        title = title[:phrase_treshold] + ellipsis
+        # Remove right trailling whitespaces
+        title = title[:phrase_treshold].rstrip()
+        # Only add ellipsis if the last word does not already
+        # end with one
+        if not title.endswith(ellipsis):
+            title += ellipsis
     return title
 
 
