@@ -15,31 +15,24 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # Import from itools
+from itools.core import merge_dicts
 from itools.datatypes import Boolean, Integer, String, Unicode
 from itools.gettext import MSG
-from itools.handlers import checkid, merge_dicts
 from itools.stl import stl
-from itools.uri import Path
-from itools.web import FormError, get_context, BaseView, STLView, STLForm
+from itools.web import STLView, STLForm
 from itools.xml import XMLParser
 
-# Import from ikaaro
-import messages
-from registry import get_resource_class
-
 
 """This module contains some generic views used by different resources.
 """
 
 
-class CompositeForm(STLForm):
-    """This view renders the sub-views defined by the class variable
-    'subviews' one after the other.
-    """
+class CompositeView(STLView):
 
     template = '/ui/generic/cascade.xml'
     subviews = []
 
+
     def get_query_schema(self):
         schema = {}
         for view in self.subviews:
@@ -57,17 +50,23 @@ class CompositeForm(STLForm):
         return {'views': views}
 
 
+
+class CompositeForm(CompositeView, STLForm):
+    """This view renders the sub-views defined by the class variable
+    'subviews' one after the other.
+    """
+
     def get_schema(self, resource, context):
         # Check for specific schema
         action = context.form_action
         for view in self.subviews:
-            method = getattr(view, context.form_action, None)
+            method = getattr(view, action, None)
             if method is None:
                 continue
             schema = getattr(view, '%s_schema' % action, None)
             if schema is not None:
                 return schema
-            return view.schema
+            return view.get_schema(resource, context)
         return {}
 
 
@@ -102,55 +101,15 @@ class IconsView(STLView):
     template = '/ui/generic/icons_view.xml'
 
     def get_namespace(self, resource, context):
-        """TODO Write a docstring explaining the expected namespace.
+        """Example:
+        return {'batch': None,
+                'items': [{'icon': '/ui/..', 'title': ...,
+                           'description': ..., 'url': ...'}, ...]}
         """
         raise NotImplementedError
 
 
 
-class NewInstanceForm(STLForm):
-    """This is the base class for all ikaaro forms meant to create and
-    add a new resource to the database.
-    """
-
-    schema = {
-        'name': String}
-
-
-    def icon(self, resource, **kw):
-        type = kw.get('type')
-        cls = get_resource_class(type)
-        if cls is not None:
-            return cls.get_class_icon()
-        # Default
-        return 'new.png'
-
-
-    def get_new_resource_name(self, form):
-        return form['name'].strip()
-
-
-    def _get_form(self, resource, context):
-        form = STLForm._get_form(self, resource, context)
-        name = self.get_new_resource_name(form)
-
-        # Check the name
-        if not name:
-            raise FormError, messages.MSG_NAME_MISSING
-
-        name = checkid(name)
-        if name is None:
-            raise FormError, messages.MSG_BAD_NAME
-
-        # Check the name is free
-        if resource.has_resource(name):
-            raise FormError, messages.MSG_NAME_CLASH
-
-        # Ok
-        form['name'] = name
-        return form
-
-
 ###########################################################################
 # Browse View (batch + table)
 ###########################################################################
@@ -168,7 +127,7 @@ class BrowseForm(STLForm):
     # Batch
     batch_template = '/ui/generic/browse_batch.xml'
     batch_msg1 = MSG(u"There is 1 item.") # FIXME Use plural forms
-    batch_msg2 = MSG(u"There are ${n} items.")
+    batch_msg2 = MSG(u"There are {n} items.")
 
     # Content
     table_template = '/ui/generic/browse_table.xml'
@@ -178,6 +137,10 @@ class BrowseForm(STLForm):
     # Actions are external to current form
     external_form = False
 
+    # Keep the batch in the canonical URL
+    canonical_query_parameters = (STLForm.canonical_query_parameters
+                                  + ['batch_start'])
+
 
     def get_namespace(self, resource, context):
         batch = None
@@ -300,22 +263,25 @@ class BrowseForm(STLForm):
                 columns_ns.append({
                     'is_checkbox': False,
                     'title': title,
-                    'href': None})
+                    'href': None,
+                    'sortable': False})
             else:
                 # Type: normal
-                kw = {'sort_by': name}
+                base_href = context.uri.replace(sort_by=name)
                 if name == sort_by:
-                    col_reverse = (not reverse)
-                    order = 'up' if reverse else 'down'
+                    sort_up_active = reverse is False
+                    sort_down_active = reverse is True
                 else:
-                    col_reverse = False
-                    order = 'none'
-                kw['reverse'] = Boolean.encode(col_reverse)
+                    sort_up_active = sort_down_active = False
                 columns_ns.append({
                     'is_checkbox': False,
                     'title': title,
-                    'order': order,
-                    'href': context.uri.replace(**kw),
+                    'sortable': True,
+                    'href': context.uri.path,
+                    'href_up': base_href.replace(reverse=0),
+                    'href_down': base_href.replace(reverse=1),
+                    'sort_up_active': sort_up_active,
+                    'sort_down_active': sort_down_active
                     })
         return columns_ns
 
@@ -334,8 +300,8 @@ class BrowseForm(STLForm):
             else:
                 onclick = None
             actions.append(
-                {'name': button.name,
-                 'value': button.title,
+                {'value': button.name,
+                 'title': button.title,
                  'class': button.css,
                  'onclick': onclick})
 
