@@ -18,96 +18,42 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+# Import from the Standard Library
+import sys
+
 # Import from itools
-import itools
-from itools import get_abspath
+from itools.core import get_abspath, merge_dicts
 from itools.datatypes import Email, String, Unicode
 from itools.datatypes import Enumerate
 from itools.gettext import MSG
-from itools.handlers import merge_dicts
 from itools.stl import stl
-from itools import vfs
+from itools.fs import lfs
 from itools.web import STLView, INFO, ERROR
-from itools.xapian import PhraseQuery, OrQuery, AndQuery, TextField
+from itools.xapian import PhraseQuery, OrQuery, AndQuery, split_unicode
 
 # Import from ikaaro
-import ikaaro
+from config import get_config
 from forms import AutoForm, SelectWidget, MultilineWidget, TextWidget
-import messages
+from messages import MSG_NEW_RESOURCE
 from registry import get_resource_class
-from registry import get_register_websites, get_website_class
-from resource_views import AddResourceMenu
-from views import NewInstanceForm, SearchForm
 from utils import get_base_path_query
+from views import SearchForm
+from views_new import ProxyNewInstance
 
 
 
-class NewWebSiteForm(NewInstanceForm):
-
-    access = 'is_allowed_to_add'
-    title = MSG(u'Web Site')
-    template = '/ui/website/new_instance.xml'
-    schema = {
-        'name': String,
-        'title': Unicode,
-        'class_id': String}
-    context_menus = [AddResourceMenu()]
-
+class NotFoundView(STLView):
+    template = '/ui/root/not_found.xml'
 
     def get_namespace(self, resource, context):
-        type = context.get_query_value('type')
-        cls = get_resource_class(type)
-
-        # Specific Websites
-        websites = get_register_websites()
-        websites = list(websites)
-        if len(websites) == 1:
-            websites = None
-        else:
-            selected = context.get_form_value('class_id')
-            websites = [
-                {'title': x.class_title.gettext(),
-                 'class_id': x.class_id,
-                 'selected': x.class_id == selected,
-                 'icon': '/ui/' + x.class_icon16}
-                for x in websites ]
-            if selected is None:
-                websites[0]['selected'] = True
-
-        # Ok
-        return {
-            'class_id': cls.class_id,
-            'class_title': cls.class_title.gettext(),
-            'websites': websites}
+        return {'uri': str(context.uri)}
 
 
-    def get_new_resource_name(self, form):
-        # If the name is not explicitly given, use the title
-        return form['name'].strip() or form['title'].strip()
+class ForbiddenView(STLView):
+    template = '/ui/root/forbidden.xml'
 
-
-    def action(self, resource, context, form):
-        name = form['name']
-        title = form['title']
-        class_id = form['class_id']
-
-        # Find out the class id
-        if class_id is None:
-            websites = get_register_websites()
-            websites = list(websites)
-            class_id = websites[0].class_id
-
-        # Make resource
-        cls = get_website_class(class_id)
-        child = cls.make_resource(cls, resource, name)
-
-        # Add title
-        metadata = child.metadata
-        language = resource.get_site_root().get_default_language()
-        metadata.set_property('title', title, language=language)
-
-        goto = './%s/' % name
-        return context.come_back(messages.MSG_NEW_RESOURCE, goto=goto)
+    def POST(self, resource, context):
+        return self.GET
 
 
 
@@ -116,6 +62,7 @@ class ForgottenPasswordForm(AutoForm):
     access = True
     title = MSG(u'Forgotten password')
     submit_value = MSG(u'Ok')
+    meta = [('robots', 'noindex, follow', None)]
 
     widgets = [
         TextWidget('username', title=MSG(u'Type your email address')),
@@ -141,8 +88,8 @@ class ForgottenPasswordForm(AutoForm):
 
         # Get the user with the given login name
         results = root.search(username=username)
-        if results.get_n_documents() == 0:
-            message = ERROR(u'There is not a user identified as "$username"',
+        if len(results) == 0:
+            message = ERROR(u'There is not a user identified as "{username}"',
                       username=username)
             context.message = message
             return
@@ -217,10 +164,13 @@ class ContactOptions(Enumerate):
     def get_options(cls):
         resource = cls.resource
         users = resource.get_resource('/users')
-
-        return [
-            {'name': x, 'value': users.get_resource(x).get_title()}
-            for x in resource.get_property('contacts') ]
+        options = []
+        for name in resource.get_property('contacts'):
+            user = users.get_resource(name, soft=True)
+            if user is None:
+                continue
+            options.append({'name': name, 'value': user.get_title()})
+        return options
 
 
 
@@ -231,23 +181,31 @@ class ContactForm(AutoForm):
     submit_value = MSG(u'Send')
     query_schema = {'to': String,
                     'subject': Unicode,
-                    'body': Unicode}
+                    'message_body': Unicode}
 
     def get_schema(self, resource, context):
         return {
             'to': ContactOptions(resource=resource, mandatory=True),
             'from': Email(mandatory=True),
             'subject': Unicode(mandatory=True),
-            'body': Unicode(mandatory=True),
+            'message_body': Unicode(mandatory=True),
+            'captcha_answer': Unicode(mandatory=True),
         }
 
 
-    widgets = [
-        SelectWidget('to', title=MSG(u'Recipient')),
-        TextWidget('from', title=MSG(u'Your email address'), size=40),
-        TextWidget('subject', title=MSG(u'Message subject'), size=40),
-        MultilineWidget('body', title=MSG(u'Message body'), rows=8, cols=50),
-    ]
+    def get_widgets(self, resource, context):
+        captcha_question = resource.get_property('captcha_question')
+        captcha_title = MSG(u"Please answer this: {captcha_question}")
+        captcha_title = captcha_title.gettext(
+                captcha_question=captcha_question)
+        return [
+            SelectWidget('to', title=MSG(u'Recipient')),
+            TextWidget('from', title=MSG(u'Your email address'), size=40),
+            TextWidget('subject', title=MSG(u'Message subject'), size=40),
+            MultilineWidget('message_body', title=MSG(u'Message body'),
+                            rows=8, cols=50),
+            TextWidget('captcha_answer', title=captcha_title),
+        ]
 
 
     def get_value(self, resource, context, name, datatype):
@@ -263,11 +221,17 @@ class ContactForm(AutoForm):
 
 
     def action(self, resource, context, form):
+        # Check captcha first
+        captcha_answer = form['captcha_answer'].strip()
+        expected = resource.get_property('captcha_answer')
+        if captcha_answer != expected:
+            context.message = ERROR(u"Wrong answer to the question.")
+            return
         # Get form values
         contact = form['to']
         from_addr = form['from'].strip()
         subject = form['subject'].strip()
-        body = form['body'].strip()
+        body = form['message_body'].strip()
 
         # Find out the "to" address
         contact = resource.get_resource('/users/%s' % contact)
@@ -319,16 +283,23 @@ class SiteSearchView(SearchForm):
             return []
 
         # The Search Query
-        query = [ OrQuery(PhraseQuery('title', word),
-                  PhraseQuery('text', word))
-                    for word, kk in TextField.split(text) ]
-        if not query:
+        languages = resource.get_property('website_languages')
+        queries = []
+        for language in languages:
+            query = [ OrQuery(PhraseQuery('title', word),
+                              PhraseQuery('text', word))
+                      for word in split_unicode(text, language) ]
+            if query:
+                queries.append(AndQuery(*query))
+
+        if not queries:
             return []
+        query = OrQuery(*queries)
 
         # Search
         abspath = resource.get_canonical_path()
         q1= get_base_path_query(str(abspath))
-        query = AndQuery(q1, *query)
+        query = AndQuery(q1, query)
         root = context.root
         results = root.search(query=query)
         documents = results.get_documents()
@@ -377,10 +348,69 @@ class AboutView(STLView):
 
 
     def get_namespace(self, resource, context):
-        return  {
-            'itools_version': itools.__version__,
-            'ikaaro_version': ikaaro.__version__,
-        }
+        # Python, itools & ikaaro
+        packages = ['sys', 'itools', 'ikaaro']
+        config = get_config(context.server.target)
+        packages.extend(config.get_value('modules'))
+        # Try packages we frequently use
+        packages.extend([
+            'gio', 'xapian', 'pywin32', 'PIL.Image', 'docutils', 'reportlab',
+            'xlrd', 'lpod'])
+        # Mapping from package to version attribute
+        package2version = {
+            'gio': 'pygio_version',
+            'xapian': 'version_string',
+            'PIL.Image': 'VERSION',
+            'reportlab': 'Version',
+            'sys': 'version_info',
+            'xlrd': '__VERSION__'}
+        package2title = {
+            'gio': u'pygobject',
+            'lpod': u'lpOD',
+            'sys': u'Python',
+            }
+
+        # Namespace
+        packages_ns = []
+        for name in packages:
+            attribute = package2version.get(name, '__version__')
+            # Import
+            if '.' in name:
+                name, subname = name.split('.')
+                try:
+                    package = __import__(subname, fromlist=[name])
+                except ImportError:
+                    continue
+            else:
+                try:
+                    package = __import__(name)
+                except ImportError:
+                    continue
+
+            # Version
+            try:
+                version = getattr(package, attribute)
+            except AttributeError:
+                version = MSG(u'no version found')
+            else:
+                if hasattr(version, '__call__'):
+                    version = version()
+                if isinstance(version, tuple):
+                    version = '.'.join([str(v) for v in version])
+            # Ok
+            title = package2title.get(name, name)
+            packages_ns.append({'name': title, 'version': version})
+
+        # Insert first the platform
+        platform = {
+            'linux2': u'GNU/Linux',
+            'darwin': u'Mac OS X',
+            'win32': u'Windows'}.get(sys.platform, sys.platform)
+        packages_ns.insert(0,
+            {'name': MSG(u'Operating System'), 'version': platform})
+
+        namespace = {'packages': packages_ns}
+        return namespace
 
 
 
@@ -389,15 +419,55 @@ class CreditsView(STLView):
     access = True
     title = MSG(u'Credits')
     template = '/ui/root/credits.xml'
+    styles = ['/ui/credits.css']
 
 
     def get_namespace(self, resource, context):
-        context.styles.append('/ui/credits.css')
-
         # Build the namespace
         credits = get_abspath('CREDITS')
-        lines = vfs.open(credits).readlines()
+        lines = lfs.open(credits).readlines()
         names = [ x[3:].strip() for x in lines if x.startswith('N: ') ]
 
         return {'hackers': names}
 
+
+
+class WebSite_NewInstance(ProxyNewInstance):
+
+    template = '/ui/website/new_instance.xml.en'
+
+    schema = merge_dicts(ProxyNewInstance.schema, vhosts=String)
+
+    def get_namespace(self, resource, context):
+        namespace = ProxyNewInstance.get_namespace(self, resource, context)
+        # Add vhosts
+        vhosts = context.get_form_value('vhosts')
+        namespace['vhosts'] = vhosts
+
+        return namespace
+
+
+    def action(self, resource, context, form):
+        name = form['name']
+        title = form['title']
+        vhosts = form['vhosts']
+        vhosts = [ x.strip() for x in vhosts.splitlines() ]
+        vhosts = [ x for x in vhosts if x ]
+        vhosts = tuple(vhosts)
+
+        # Create the resource
+        class_id = form['class_id']
+        if class_id is None:
+            # Get it from the query
+            class_id = context.query['type']
+        cls = get_resource_class(class_id)
+        child = cls.make_resource(cls, resource, name)
+        # The metadata
+        metadata = child.metadata
+        language = resource.get_content_language(context)
+        metadata.set_property('title', title, language=language)
+        metadata.set_property('vhosts', vhosts)
+
+        goto = './%s/' % name
+        return context.come_back(MSG_NEW_RESOURCE, goto=goto)
+
